import SearchUtils
from time import clock
#import bitset
import Ini




USEMAGIC = True
if Ini.isWindows():
	import bitmagic_windows as bitmagic
elif Ini.isLinux():
	import bitmagic_linux as bitmagic
else:
	USEMAGIC = False


class bitstringVariables:
	"""Class containing variables used by all bit vectors. Includes lookup-table"""
	def __init__(self, shiftBits=16):
		self.shiftBits = shiftBits
		self.andValue = 0
		for i in range(shiftBits):
			self.andValue |= (1<<i)
		self.bc = [0]*(self.andValue+1)
		for i in range(self.andValue+1):
			self.bc[i] = self.iter_countBits(i)
		self.c = 0
		
	def iter_countBits(self, intValue):
		"""Iterative method of counting set bits. OK for small input values. Used to generate lookup table"""
		counter = 0
		while intValue != 0:
			if intValue & 1:
				counter += 1
			intValue = intValue>>1
		return counter



class magic_bitstring:
	#BITMAGIC
	def __init__(self, bsv, length):
		self.bsv = bsv
		self.max = 0
		self.bs = bitmagic.bitset_new()

	def __del__(self):
	#	"""Destructor"""
		bitmagic.bitset_delete(self.bs)



	def setBit(self, pos):
		"""Sets a bit in the bit vector to 1"""
		if pos > self.max: self.max = pos
		bitmagic.bitset_set(self.bs, pos)
	
	
	def andStrings(self, otherString):
		r = magic_bitstring(self.bsv, -1)
		bitmagic.bitset_copy(r.bs, self.bs)
		bitmagic.bitset_and(r.bs, otherString.bs)
		r.max = self.max
		return r
	
	def orStrings(self, otherString):
		r = magic_bitstring(self.bsv, -1)
		bitmagic.bitset_copy(r.bs, self.bs)
		bitmagic.bitset_or(r.bs, otherString.bs)
		r.max = self.max
		return r
	
	def countBits(self):
		"""Uses a bitstringVariables-lookup table to count 16 bits in each iteration of the loop"""
		return bitmagic.bitset_count(self.bs)
	
	def negate(self):
		pass
	
	def __str__(self):
		r = "max:"+str(self.max)+" "
		for i in range(self.max+1-1, -1, -1):
			t = bitstring(self.bsv, -1)
			t.setBit(i)
			if self.andStrings(t).countBits() == 1: r+= "1"
			else: r+= "0"

		return r
class python_bitstring:
	#Pure python, list
	def __init__(self, bsv, length):
		self.bsv = bsv
		self.bs = []
		self.masking = 65535
		self.max = 0

	def setBit(self, pos):
		"""Sets a bit in the bit vector to 1"""
		if( self.max < pos ):
			self.max = pos;
		lindex = pos/self.bsv.shiftBits
		while int(lindex) + 1 > len(self.bs):
			self.bs.append(0)
		
		self.bs[lindex] |= (1<<(pos%self.bsv.shiftBits))
		
	def andStrings(self, otherString):
		"""Use the logic "and"-operation on two strings"""
		
		if( len(self.bs) > len(otherString.bs) ):
			otherString.bs += [0]*(len(self.bs)-len(otherString.bs))
		else:
			self.bs += [0]*(len(otherString.bs)-len(self.bs))
			
		tempString = []
		for i in range(len(self.bs)):
			tempString.append(self.bs[i] & otherString.bs[i])
		
		r = python_bitstring(self.bsv, 0)
		r.bs = tempString
		return r
	
	def orStrings(self, otherString):
		"""Use the logic "or"-operation on two strings"""
		if( len(self.bs) > len(otherString.bs) ):
			otherString.bs += [0]*(len(self.bs)-len(otherString.bs))
		else:
			self.bs += [0]*(len(otherString.bs)-len(self.bs))
			
		tempString = []
		for i in range(len(self.bs)):
			tempString.append(self.bs[i] | otherString.bs[i])
		
		r = python_bitstring(self.bsv, 0)
		r.bs = tempString
		return r
	
	def countBits(self):
		"""Uses a bitstringVariables-lookup table to count 16 bits in each iteration of the loop"""
		counter = 0
		for line in self.bs:
			counter += self.bsv.bc[line]

		return counter
	
	def negate(self):
		rbs = []
		for i in range(len(self.bs)):
			rbs.append(self.bs[i]^self.masking)
		r = python_bitstring(self.bsv, 0)
		r.bs = rbs
		return r
	
	def __str__(self):
		bl = ""
		
		for line in self.bs:
			tempbl = ""
			while line != 0:
				if line & 1:
					tempbl += "1"
				else:
					tempbl+= "0"
				line = line>>1
			while len(tempbl) < 16:
				tempbl += "0"
			bl += tempbl
			
		
		bl = list(bl)
		bl.reverse()
		

		return "".join(bl)
			
bitstring = None
if USEMAGIC:
	bitstring = magic_bitstring

else:
	bitstring = python_bitstring


if __name__ == '__main__':
	#Run a benchmark test
	from random import shuffle, randint
	import time
	
	print "initializing...",
	bsv = bitstringVariables()

	rounds = 5000
	iterations = 10
	
	ma, mo, mc, pa, po, pc = [0.]*6
	print "beginning"
	for it in range(iterations):
		a = python_bitstring(bsv, 0)
		b = python_bitstring(bsv, 0)
		c = magic_bitstring(0, 0)
		d = magic_bitstring(0, 0)
		
		
		for j in range(5000):
			k = randint(0, 10000)
			j = randint(0, 10000)
			a.setBit(k)
			b.setBit(j)
			c.setBit(k)
			d.setBit(j)
		
		
		print "b",
		t = time.clock()
		for i in range(rounds):
			e = c.orStrings(d)
		mo += time.clock()-t
		print ".",
		t = time.clock()
		for i in range(rounds):	
			e = c.andStrings(d)
		ma += time.clock()-t
		print ".",
		t = time.clock()
		for i in range(rounds/2):	
			e = c.countBits()
			e = d.countBits()
		mc += time.clock()-t
		print ".",
		
		
		print "p",
		t = time.clock()
		for i in range(rounds):
			e = a.orStrings(b)
		po += time.clock()-t
		print ".",
		t = time.clock()
		for i in range(rounds):	
			e = a.andStrings(b)
		pa += time.clock()-t
		print ".",
		t = time.clock()
		for i in range(rounds/2):	
			e = a.countBits()
			e = b.countBits()
		pc += time.clock()-t
		print ".",
	
	print
	print iterations, "iterations"
	print
	print "Bitmagic:"
	print rounds, "rounds of the or-operation:", mo/iterations
	print rounds, "rounds of the and-operation:", ma/iterations
	print rounds, "bitcounts:", mc/iterations
	
	print 
	print "Python bitset:"
	print rounds, "rounds of the or-operation:", po/iterations
	print rounds, "rounds of the and-operation:", pa/iterations
	print rounds, "bitcounts:", pc/iterations
			
	print
	print "Factors:"
	print "or-operation: bitmagic is", po/mo,"times faster."
	print "and-operation: bitmagic is", pa/ma,"faster."
	print "bitcount: bitmagic is", pc/mc,"times faster."
	
		
		
	
	
	
	
	#import main
	#main.main()
"""
10 iterations

Bitmagic:
5000 rounds of the or-operation: 0.111092247758
5000 rounds of the and-operation: 0.108128267699
5000 bitcounts: 0.0457487194602

Python bitset:
5000 rounds of the or-operation: 2.15136032652
5000 rounds of the and-operation: 2.19670293291
5000 bitcounts: 1.03812755532

Factors:
or-operation: bitmagic is 19.3655306283 times faster.
and-operation: bitmagic is 20.3157137321 faster.
bitcount: bitmagic is 22.6919478308 times faster.
"""