import Ini
Ini.versionInit()

#Constants representing where results shuold be outputt'ed. If several of the constants are
#to be used,'or' them together. (example: FILE | SCREEN writes result to both file and screen)
#---------------------------------------------------------------------------------------------

#Results printed to screen
SCREEN = 1
#Results written to file
FILE = 2


#Force run even if equivalent result file exists.
#FORCE_RUN | FILE means overwriting the existing file.
FORCE_RUN = 4

#Constants representing training type

from copy import copy, deepcopy
from math import ceil
import time
from ConfigParser import ConfigParser

from BlocksInterface import getAlignment as getBlocksAlignment
from Classificators import IUPACID, HDISTID, PWMID, PBMID, OPTIMALID
from DB_constants import BASIC, BLOCKS, RAP, RAA, CVBASIC, CVBLOCKS, CALIGN

from Result import Result, OBJECTFOLDER, SPLIT

from Patterns import getPatternString

from dataset.empty.read import getDataset as getEmptySet
from library.sequence import sequence
from library.motif.motif import motif_known



class Super:
	def __init__(self, models, datasets, scorefunctions, motiflengths, writeMethod=SCREEN,\
				hdistSpecific=None, regSpecific=None, excluded=[]):
		self.models = models
		self.datasets = datasets
		self.scorefunctions = scorefunctions
		self.motiflengths = motiflengths
		self.hdistSpecific = hdistSpecific
		self.writeMethod = writeMethod
		self.regSpecific = regSpecific
		self.excluded = excluded

	
	def runAll(self):
		ti = time.clock()
		
		
		
		fil = file("temptimes.txt", "w")
		for model in self.models:
			for dataset in self.datasets:
				for scorefunction in self.scorefunctions:
					for motiflength in self.motiflengths:
						tti = time.clock()
						self.runTest(model, dataset, scorefunction, motiflength, self.writeMethod,\
						self.hdistSpecific, self.regSpecific, excluded=self.excluded)
						
						ttime = time.clock()-tti
						fil.write(str(ttime)+"\n")
						fil.flush()
						print "runTest() - time:", ttime
						print "#"*75
						
		
		
		
		
		
		fil.close()	
		
		
		print "runAll() - total time:", time.clock()-ti
	
	def fileExists(self, datasetname, tt, modelid, motiflength, sfid):
		
		filename = OBJECTFOLDER+datasetname+SPLIT+tt+SPLIT+modelid+SPLIT+'%(l)02d'%{'l':motiflength}+SPLIT+sfid+".res"
		print filename
		r = True
		try:
			file(filename)
		except:
			r = False
		return r


class CVSuper:
	def __init__(self, models, datasets, scorefunctions, motiflengths, writeMethod=SCREEN,\
				hdistSpecific=None, regSpecific=None, cvnum=1, excluded=[]):
		self.models = models
		self.datasets = datasets
		self.scorefunctions = scorefunctions
		self.motiflengths = motiflengths
		self.hdistSpecific = hdistSpecific
		self.writeMethod = writeMethod
		self.cvnum = cvnum
		self.regSpecific = regSpecific
		self.excluded = excluded
		
	
	def runAll(self):
		ti = time.clock()
		for model in self.models:
			for dataset in self.datasets:
				for scorefunction in self.scorefunctions:
					for motiflength in self.motiflengths:
						tti = time.clock()
						self.runTest(model, dataset, scorefunction, motiflength, self.writeMethod,\
						self.hdistSpecific, self.regSpecific, excluded=self.excluded)
						print "runTest() - time:", time.clock()-tti
						print "#"*75
						
						
		print "runAll() - total time:", time.clock()-ti
	
	def fileExists(self, datasetname, tt, modelid, motiflength, sfid):
		
		filename = OBJECTFOLDER+datasetname+SPLIT+tt+SPLIT+modelid+SPLIT+'%(l)02d'%{'l':motiflength}+SPLIT+sfid+".res"
		
		r = True
		try:
			file(filename)
		except:
			r = False
		return r


class Basic(Super):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		
		#tm = "basic"
		tm = BASIC
		
		#Skip if: result file exists and should not be overwritten
		#or: the motiflength exceeds the possible motiflength of the dataset
		if motiflength == -1:
			motiflength = dataset.getMotifWindows()[0][2]
		
		print "Max motiflength:",dataset.getMaxMotifLength()
		print "-"*50
		
		if(self.fileExists(dataset._filename, tm, model.id, motiflength, scorefunction.id)\
			and not writeMethod & FORCE_RUN):
			print writeMethod
			print "skipping: file exists", dataset._filename, model.id, motiflength, scorefunction.id
			return
		if dataset.getMaxMotifLength() < motiflength:
			print "skipping: maximum motiflength exceeded", dataset._filename, model.id, motiflength, scorefunction.id
			return
		
		
		print dataset._filename, model.id, motiflength, scorefunction.id
		
		p = None
		if model.id == IUPACID:
			
			brsf = regSpecific
			if brsf == None:
				if self.fileExists(dataset._filename, tm, model.id, motiflength-1, scorefunction.id):
					r = Result()
					r.read(dataset._filename+SPLIT+tm+SPLIT+model.id+SPLIT+'%(l)02d'%{'l':motiflength-1}+SPLIT+scorefunction.id+".res")
					brsf = r.m.score-0.04
					print "IUPAC: using "+ str(brsf) + " as input score"
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, brsf=brsf, excluded=excluded)
		elif model.id == HDISTID:
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, minmut=hdistSpecific[0], maxmut=hdistSpecific[1], excluded=excluded)
		else:
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, excluded=excluded)
		
		
		print p
		p[0].search(dataset, scorefunction, excluded=excluded)
		
		
		result = Result(p[0], tm)
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:

			result.write()
		
		return result

class CenteredAlignment(Super):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		
		#tm = "basic"
		tm = BASIC
		
		#Skip if: result file exists and should not be overwritten
		#or: the motiflength exceeds the possible motiflength of the dataset
		if motiflength == -1:
			motiflength = dataset.getMotifWindows()[0][2]
		
		print "Max motiflength:",dataset.getMaxMotifLength()
		print "-"*50
		
		if(self.fileExists(dataset._filename, tm, model.id, motiflength, scorefunction.id)\
			and not writeMethod & FORCE_RUN):
			print "skipping: file exists", dataset._filename, model.id, motiflength, scorefunction.id
			return
		if dataset.getMaxMotifLength() < motiflength:
			print "skipping: maximum motiflength exceeded", dataset._filename, model.id, motiflength, scorefunction.id
			return
		
		
		print dataset._filename, model.id, motiflength, scorefunction.id
		
		p = None
		if model.id == IUPACID:
			
			brsf = regSpecific
			if brsf == None:
				if self.fileExists(dataset._filename, tm, model.id, motiflength-1, scorefunction.id):
					r = Result()
					r.read(dataset._filename+SPLIT+tm+SPLIT+model.id+SPLIT+'%(l)02d'%{'l':motiflength-1}+SPLIT+scorefunction.id+".res")
					brsf = r.m.score-0.04
					print "IUPAC: using "+ str(brsf) + " as input score"
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, brsf=brsf, excluded=excluded)
		elif model.id == HDISTID:
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, minmut=hdistSpecific[0], maxmut=hdistSpecific[1], excluded=excluded)
		else:
			p = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, excluded=excluded)
		
		
		print p
		p[0].search(dataset, scorefunction, excluded=excluded)
		
		
		result = Result(p[0], tm)
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:

			result.write()
		
		return result

				
class CVBasic(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		if motiflength == -1:
			motiflength = dataset.getMotifWindows()[0][2]
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		type = -1
		
		
		mi = dataset.getMotifIndexes()
		totalMotifs = len(mi)
		motifsFound = 0
		
		
		
		sequences = dataset.getSequences()
		self.cvnum = 1
		for i in range(0, len(mi), self.cvnum):
			ci = mi[i:i+self.cvnum]
			currentTP = len(ci)
			
			d = getEmptySet()
			d._filename = dataset._filename
			mk = motif_known()
			
			for l in range(len(ci)):
				j, k = ci[l]
				d._sequence_list.append(sequence(l, sequences[j][k:k+motiflength]))
				mk.add_hitlist_entry(l, 0, motiflength)
			
			d._motif_list.append(mk)
			
			
			
			
			#print ci
			for j in range(len(ci)):
				l, m = ci[j]
				for k in range(m-motiflength+1, m)+range(m+1, m+motiflength): ci += [(l, k)]
			
			#print ci
			p = b.runTest(model, dataset, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific, excluded=ci)
			p = p.m
			
			
			
			
			#print p
			#print "."*50
			p.search(d, scorefunction)
			print "."*50
			motifsFound += p.TP
			
		
		f = file(dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id), "w")
		f.write(str(totalMotifs)+","+str(motifsFound)+","+str(float(motifsFound)/totalMotifs))
		f.close()
		"motifs found:", motifsFound
		
	

class CVSBasic(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		print "CVS"
		sequences = dataset.getSequences()
		if len(sequences) == 1:
			cvg = CVGBlocks(None, None, None, None, cvnum=self.cvnum, excluded=[])
			cvg.runTest(model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific, regSpecific, excluded)
			return
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		
		
		if motiflength == -1:
			motiflength = dataset.getMotifWindows()[0][2]
		print "motiflength:", motiflength
				
		#get the positive and a negative set
		motifpos = dataset.getMotifIndexes()
		
		cvgroups = []
		for i in range(len(sequences)):
			pos, neg = [], []
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: pos.append(seq[j:j+motiflength])
				else: neg.append(seq[j:j+motiflength])
			cvgroups.append((pos, neg))
		for line in cvgroups:
			print line[0]
		
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		for i in range(len(cvgroups)):
			
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			d._filename = dataset._filename
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			
			print "motifindexes:", d.getMotifIndexes(), "mk:", len(pos)
			
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m
			
			
			
			
			
			
			pos, neg = validationset
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFP, tFN, tTN = m.search(validd, scorefunction)
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FP, FN, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvs"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		
class CVSBasicTest(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		tm = CVBASIC
		print "CVS"
		sequences = dataset.getSequences()
		if len(sequences) == 1:
			print "Error: CVGBASICTEST NOT IMPLEMENTED YET"
			import sys
			sys.exit(1)
			#cvg = CVGBasicTest(None, None, None, None, cvnum=self.cvnum, excluded=[])
			#cvg.runTest(model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific, regSpecific, excluded)
			#return
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		#Align motifwindows
		mw = dataset.getMotifWindows()
		#end align windows
		
		#get the positive and a negative set
		r = range(len(sequences))
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		totaltimeintest = 0
		totalHitlist = []
		m = None
		motifsUsed = []
		for i in r:
			temptime = time.clock()
			
			
			tempmw = []
			inversetmw = []
			for j, k, l in mw:
				if i!=j: tempmw.append((j,k,l))
				else: inversetmw.append((j, k, l))
			

			
			
			
			dataset.setMotifHitlist(tempmw)
			tempseq = dataset._sequence_list[i].seq
			dataset._sequence_list[i].seq = ""
			m = b.runTest(model, dataset, scorefunction, motiflength,FORCE_RUN, hdistSpecific, regSpecific)
			m = m.m
			motifsUsed.append(getPatternString(m))
			
			dataset._sequence_list[i].seq = tempseq
			
			templist = []
			for j in r[0:i] + r[i+1:len(sequences)]:
				templist.append(dataset._sequence_list[j].seq)
				dataset._sequence_list[j].seq = ""
			
			dataset.setMotifHitlist(inversetmw)
			tscore, tTP, tFN, tFP, tTN = m.search(dataset, scorefunction)
			totalHitlist += m.hitlist
			
			for j, k in enumerate(r[0:i] + r[i+1:len(sequences)]):
				dataset._sequence_list[k].seq = templist[j]
				
				
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(r)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
		score/=len(r)
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		print result
		
		
		#reset hitlist
		dataset.setMotifHitlist(mw)#used to be resethitlist, but seems better to leave alignments
		#end reset hitlist
		
		m.hitlist = totalHitlist
		m.TP = TP
		m.FP = FP
		m.FN = FN
		m.TN = TN
		m.score = scorefunction.fitness(TP, FN, FP, TN)
		
		m.CVpatterns = motifsUsed
		
		m.dataset = dataset
		
		result = Result(m, tm)
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:

			result.write()
		
		return result
	
	
class CVGBasic(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		print "CVG"
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		"""
		#Align motifwindows
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		"""
		
		"""
		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)
		dataset.setMotifHitlist(mw)
		"""
		#end align windows
		#get the positive and a negative set
		if motiflength == -1:
			motiflength = dataset.getMotifWindows()[0][2]
		
		print "motiflength:", motiflength
		
		
		positive, negative = [], []
		motifpos = dataset.getMotifIndexes()
		sequences = dataset.getSequences()
		for i in range(len(sequences)):
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: positive.append(seq[j:j+motiflength])
				else: negative.append(seq[j:j+motiflength])
		
		
		from math import ceil, floor
		
		ngroups = int(ceil(float(len(positive))/self.cvnum))
		nratio = (len(negative)/ngroups)
		
		
		state = 0
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		cvgroups = []

		for i in range(0, len(positive), self.cvnum):
			pos = positive[i:i+self.cvnum]
			neg = []
			if i+self.cvnum >= len(positive): neg = negative[state:]
			else: neg = negative[state:state+nratio]
			state += nratio
			cvgroups.append((pos, neg))
			
		
		

		for i in range(len(cvgroups)):
			
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m
			
			
			
			
			
			
			pos, neg = validationset
			
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFP, tFN, tTN = m.search(validd, scorefunction)
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FP, FN, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvg"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		#reset hitlist
		#dataset.setMotifHitlist(resetlist)
		#end reset hitlist		
		
		
		
			

class Blocks(Super):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		#tm = "blocks"
		tm = BLOCKS
		
		
		
		
			
		
		
		print "-"*50
		
		print dataset.getMaxMotifLength()
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
			
		else:
			seq = dataset.getSequences()
			input = []

			
			for seqid, pos, length in mw:
				window = seq[seqid][pos:pos+length].upper()
				#All windows must at least be of length 8
				i=0
				while len(window) < 8:
					window = window+"X"
					i+=1
				print window, seqid, pos, length
				input.append(window)
			
			mlen, alignment = getBlocksAlignment(input)
			
			
			c.set("length", dataset._filename, mlen)
			c.set("indexes", dataset._filename, alignment)
			
			
			
			f = file("blocksindexes.conf", "w")
			c.write(f)
			f.close()
		
		
		if motiflength == -1:
			motiflength = mlen
		if(self.fileExists(dataset._filename, tm, model.id, motiflength, scorefunction.id)\
			and not writeMethod & FORCE_RUN):

			print "skipping: file exists", dataset._filename, model.id, motiflength, scorefunction.id
			return
		
		
		
		
		print dataset._filename, model.id, motiflength, scorefunction.id
		
		seq = dataset.getSequences()
		
		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			#print seq[seqid][pos:pos+length].upper(), seqid, pos, length
			
			mw[i] = (seqid, pos+alignment[i], mlen)
			
			seqid, pos, length = mw[i]
			print seq[seqid][pos:pos+length].upper(), seqid, pos, length
			#print seq[mw[i][0]][mw[i][1]:mw[i][1]+mw[i][2]],
		#print
		dataset.setMotifHitlist(mw)
		
		if dataset.getMaxMotifLength() < motiflength:
			print "skipping: maximum motiflength exceeded", dataset._filename, model.id, motiflength, scorefunction.id
			return

		res = None
		if model.id == IUPACID:
			brsf = regSpecific
			if brsf == None:
				if self.fileExists(dataset._filename, tm, model.id, motiflength-1, scorefunction.id):
					r = Result()
					r.read(dataset._filename+SPLIT+tm+SPLIT+model.id+SPLIT+'%(l)02d'%{'l':motiflength-1}+SPLIT+scorefunction.id+".res")
					brsf = r.m.score-0.04
					print "IUPAC: using "+ str(brsf) + " as input score"
			res = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, brsf=brsf, excluded=excluded)
		
		elif model.id == HDISTID:
			res = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, minmut=hdistSpecific[0], maxmut=hdistSpecific[1], excluded=excluded)
		
		else:
			res = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength, excluded=excluded)
		
		print res
		res[0].search(dataset, scorefunction)
		
		
		result = Result(res[0], tm)
		
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:
			result.write()
		
		dataset.setMotifHitlist(resetlist)
		
		return result


class CVSBlocks(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		print "CVS"
		sequences = dataset.getSequences()
		if len(sequences) == 1:
			cvg = CVGBlocks(None, None, None, None, cvnum=self.cvnum, excluded=[])
			cvg.runTest(model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific, regSpecific, excluded)
			return
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		#Align motifwindows
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		
		if motiflength == -1:
			motiflength = mlen
		print "motiflength:", motiflength
		

		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)

		dataset.setMotifHitlist(mw)
		#end align windows
		
		#get the positive and a negative set
		
		motifpos = dataset.getMotifIndexes()
		
		cvgroups = []
		for i in range(len(sequences)):
			pos, neg = [], []
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: pos.append(seq[j:j+motiflength])
				else: neg.append(seq[j:j+motiflength])
			cvgroups.append((pos, neg))
		for line in cvgroups:
			print line[0]
		
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		
		totaltimeintest = 0
		for i in range(len(cvgroups)):
			temptime = time.clock()
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			d._filename = dataset._filename
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			
			print "motifindexes:", d.getMotifIndexes(), "mk:", len(pos)
			
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m

			
			
			
			pos, neg = validationset
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFN, tFP, tTN = m.search(validd, scorefunction)
			
			#print "#"*40
			#print tscore, tTP, tFN, tFP, tTN
			#print "#"*40
			
			
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(cvgroups)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		
		print result
		
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvs"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		#reset hitlist
		dataset.setMotifHitlist(resetlist)
		#end reset hitlist

class CVSBlocksTest(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		tm = CVBLOCKS
		print "CVS"
		sequences = dataset.getSequences()
		if len(sequences) == 1:
			cvg = CVGBlocksTest(None, None, None, None, cvnum=self.cvnum, excluded=[])
			cvg.runTest(model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific, regSpecific, excluded)
			return
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		#Align motifwindows
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		
		if motiflength == -1:
			motiflength = mlen
		print "motiflength:", motiflength
		

		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)

		dataset.setMotifHitlist(mw)
		#end align windows
		
		#get the positive and a negative set
		r = range(len(sequences))
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		totaltimeintest = 0
		totalHitlist = []
		m = None
		motifsUsed = []
		for i in r:
			temptime = time.clock()
			
			
			tempmw = []
			inversetmw = []
			for j, k, l in mw:
				if i!=j: tempmw.append((j,k,l))
				else: inversetmw.append((j, k, l))
			

			
			
			
			dataset.setMotifHitlist(tempmw)
			tempseq = dataset._sequence_list[i].seq
			dataset._sequence_list[i].seq = ""
			m = b.runTest(model, dataset, scorefunction, motiflength,FORCE_RUN, hdistSpecific, regSpecific)
			m = m.m
			motifsUsed.append(getPatternString(m))
			
			dataset._sequence_list[i].seq = tempseq
			
			templist = []
			for j in r[0:i] + r[i+1:len(sequences)]:
				templist.append(dataset._sequence_list[j].seq)
				dataset._sequence_list[j].seq = ""
			
			dataset.setMotifHitlist(inversetmw)
			tscore, tTP, tFN, tFP, tTN = m.search(dataset, scorefunction)
			totalHitlist += m.hitlist
			
			for j, k in enumerate(r[0:i] + r[i+1:len(sequences)]):
				dataset._sequence_list[k].seq = templist[j]
				
				
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(r)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
		score/=len(r)
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		print result
		
		
		#reset hitlist
		dataset.setMotifHitlist(mw)#used to be resethitlist, but seems better to leave alignments
		#end reset hitlist
		
		m.hitlist = totalHitlist
		m.TP = TP
		m.FP = FP
		m.FN = FN
		m.TN = TN
		m.score = scorefunction.fitness(TP, FN, FP, TN)
		
		m.CVpatterns = motifsUsed
		
		m.dataset = dataset
		
		result = Result(m, tm)
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:

			result.write()
		
		return result
		"""f
		for i in range(len(sequences)):
			pos, neg = [], []
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: pos.append(seq[j:j+motiflength])
				else: neg.append(seq[j:j+motiflength])
			cvgroups.append((pos, neg))
		for line in cvgroups:
			print line[0]
		
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		
		totaltimeintest = 0
		for i in range(len(cvgroups)):
			temptime = time.clock()
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			d._filename = dataset._filename
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			
			print "motifindexes:", d.getMotifIndexes(), "mk:", len(pos)
			
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m
			
			
			
			
			
			
			pos, neg = validationset
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFN, tFP, tTN = m.search(validd, scorefunction)
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(cvgroups)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvs"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		
		#reset hitlist
		dataset.setMotifHitlist(resetlist)
		#end reset hitlist
		
		"""
		
		
		


class CVGBlocks(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		print "CVG"
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		#Align motifwindows
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		
		if motiflength == -1:
			motiflength = mlen
		print "motiflength:", motiflength
		
		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)
		dataset.setMotifHitlist(mw)
		#end align windows
		#get the positive and a negative set
		positive, negative = [], []
		motifpos = dataset.getMotifIndexes()
		sequences = dataset.getSequences()
		for i in range(len(sequences)):
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: positive.append(seq[j:j+motiflength])
				else: negative.append(seq[j:j+motiflength])
		
		
		
		
		ngroups = int(ceil(float(len(positive))/self.cvnum))
		nratio = (len(negative)/ngroups)
		
		
		state = 0
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		cvgroups = []

		for i in range(0, len(positive), self.cvnum):
			pos = positive[i:i+self.cvnum]
			neg = []
			if i+self.cvnum >= len(positive): neg = negative[state:]
			else: neg = negative[state:state+nratio]
			state += nratio
			cvgroups.append((pos, neg))
			
		
		
		totaltimeintest = 0
		for i in range(len(cvgroups)):
			temptime = time.clock()
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m
			
			
			
			
			
			
			pos, neg = validationset
			
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFN, tFP, tTN = m.search(validd, scorefunction)
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(cvgroups)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
			
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		
		print result
		
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvg"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		#reset hitlist
		dataset.setMotifHitlist(resetlist)
		#end reset hitlist

class CVGBlocksTest(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		tm = CVBLOCKS
		print "CVG"
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		
		#Align motifwindows
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		
		if motiflength == -1:
			motiflength = mlen
		print "motiflength:", motiflength
		
		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)
		dataset.setMotifHitlist(mw)
		#end align windows
		
		
		
		#get the positive and a negative set
		positive, negative = [], []
		motifpos = dataset.getMotifIndexes()
		sequences = dataset.getSequences()
		mapping = {}
		negstate = len(motifpos)
		posstate = 0
		#print motifpos
		for i in range(len(sequences)):
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				tmp = seq[j:j+motiflength]
				if (i, j) in motifpos:
					mapping[posstate] = (i, j)
					posstate += 1
					positive.append(tmp)
				else:
					mapping[negstate] = (i, j)
					negstate += 1
					negative.append(tmp)
		seq = positive + negative
		# # # #
		
		
		
		d = getEmptySet()
		mk = motif_known()
		for j in range(len(positive)):mk.add_hitlist_entry(j, 0, motiflength)
		for line in seq: d._sequence_list.append(sequence(-1, line))
		d._motif_list.append(mk)
		resetSequencelist = deepcopy(d._sequence_list)
		
		
		
		
		
		
		# # # #
		#self.cvnum=2
		#positive = range(10)
		#negative = range(21)
		
		ngroups = int(ceil(float(len(positive))/self.cvnum))
		nratio = int(ceil(float(len(negative))/ngroups))
		

		if nratio*(ngroups-1) >= len(negative):
			nratio -= 1 #Prevents the potential problem of the last group not containing a negative set
		
		
		
		
		
		posstate, negstate = 0, 0
		posindex = range(len(positive))
		negindex = range(len(positive), len(negative) + len(positive))
		
		print len(negative), len(positive)
		
		tposi, tnegi = None, None
		
		score, TP, TN, FP, FN = 0, 0, 0, 0, 0
		totalHitlist = []
		totaltimeintest = 0
		m = None
		motifsUsed = []
		for i in range(ngroups):
			temptime = time.clock()
			if i == ngroups-1:
				tposi = posindex[posstate:]
				tnegi = negindex[negstate:]
			else:
				tposi = posindex[posstate:posstate+self.cvnum]
				tnegi = negindex[negstate:negstate+nratio]
			print "negative", len(tnegi)
			#m = b.runTest(model, d, scorefunction, motiflength,\
			#0, hdistSpecific, regSpecific)
			#m = m.m
			
			tempmw, tinvmw = [], []
			
			for j in range(len(positive)):
				if j in tposi:
					tinvmw.append((j, 0, motiflength))
				else: tempmw.append((j, 0, motiflength))
			
			
			
			
			for j in tposi + tnegi:	d._sequence_list[j].seq = ""

			
			d.setMotifHitlist(tempmw)
			
			m = b.runTest(model, d, scorefunction, motiflength, FORCE_RUN, hdistSpecific, regSpecific)
			
			m = m.m
			motifsUsed.append(getPatternString(m))
			d._sequence_list = deepcopy(resetSequencelist)
			
			d.setMotifHitlist(tinvmw)
			for j in range(len(seq)):
				if j not in tposi + tnegi: d._sequence_list[j].seq = ""
			
			
			tscore, tTP, tFN, tFP, tTN = m.search(d, scorefunction)
			d._sequence_list = deepcopy(resetSequencelist)
			
			
			totalHitlist += m.hitlist
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			

			posstate += self.cvnum
			negstate += nratio
			
			percent = float((i+1))/ngroups
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
			
			
			
		#reset hitlist
		dataset.setMotifHitlist(mw)#used to be resethitlist, but seems better to leave alignments
		#end reset hitlist
		
		
		final = []
		for i, j in totalHitlist: final.append(mapping[i])
		final.sort()
		m.hitlist = final
		m.TP = TP
		m.FP = FP
		m.FN = FN
		m.TN = TN
		m.score = scorefunction.fitness(TP, FN, FP, TN)
		
		m.CVpatterns = motifsUsed
		
		m.dataset = dataset
		
		result = Result(m, tm)
		if writeMethod & SCREEN:
			print result
		if writeMethod & FILE:

			result.write()
		
		return result
		
		"""
		positive, negative = [], []
		motifpos = dataset.getMotifIndexes()
		print motifpos
		print
		sequences = dataset.getSequences()
		for i in range(len(sequences)):
			seq = sequences[i]
			for j in range(len(seq)-motiflength+1):
				if (i, j) in motifpos: positive.append(seq[j:j+motiflength])
				else: negative.append(seq[j:j+motiflength])
		
		
		
		
		ngroups = int(ceil(float(len(positive))/self.cvnum))
		nratio = (len(negative)/ngroups)
		
		
		state = 0
		score, TP, FP, FN, TN = 0, 0, 0, 0, 0
		cvgroups = []

		for i in range(0, len(positive), self.cvnum):
			pos = positive[i:i+self.cvnum]
			neg = []
			if i+self.cvnum >= len(positive): neg = negative[state:]
			else: neg = negative[state:state+nratio]
			state += nratio
			cvgroups.append((pos, neg))
			
		
		
		totaltimeintest = 0
		for i in range(len(cvgroups)):
			temptime = time.clock()
			trainingset = cvgroups[:i]+cvgroups[i+1:]
			validationset = cvgroups[i]
			
			
			pos, neg = [], []
			for p, n in trainingset:
				pos += p
				neg += n
			
			
			tempseq = pos + neg
			d = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: d._sequence_list.append(sequence(-1, line))
			d._motif_list.append(mk)
			m = b.runTest(model, d, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific)
			m = m.m
			
			
			
			
			
			
			pos, neg = validationset
			
			tempseq = pos + neg
			validd = getEmptySet()
			mk = motif_known()
			for j in range(len(pos)):mk.add_hitlist_entry(j, 0, motiflength)
			for line in tempseq: validd._sequence_list.append(sequence(-1, line))
			validd._motif_list.append(mk)
			tscore, tTP, tFN, tFP, tTN = m.search(validd, scorefunction)
			
			score += tscore
			TP += tTP
			FP += tFP
			FN += tFN
			TN += tTN
			percent = float((i+1))/len(cvgroups)
			totaltimeintest += time.clock()-temptime
			print "percent done:", str(100*percent)+"%"
			print "time left:", str((totaltimeintest/percent)-totaltimeintest)
			
		score/=len(cvgroups)
		
		
		
		

		
		
		result = "filename:"+dataset._filename, "model:"+str(model.id), "motiflength:"+str(motiflength),\
			"scorefunction:"+str(scorefunction.id), "cvtype:"+str(self.cvnum)+"-fold",\
			"score:"+str(scorefunction.fitness(TP, FN, FP, TN)), "TP:"+str(TP), "FP:"+str(FP),\
			"FN:"+str(FN), "TN:"+str(TN), "avgscore:"+str(score)
		result = "\n".join(result)
		print result
		
		f = file("results/dip/obj/"+dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id)+"_-_cvg"+str(self.cvnum)+".txt", "w")
		f.write(result)
		f.close()
		#reset hitlist
		dataset.setMotifHitlist(resetlist)
		#end reset hitlist
		"""
"""
class CVBlocks(CVSuper):
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None, regSpecific=None, excluded=[]):
		
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		mw = dataset.getMotifWindows()
		resetlist = dataset.getMotifWindows()
		mw.sort()
		
		
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):alignment[i] = int(alignment[i])
		else: 
			print "No blocks entry found for dataset. Run a standard Blocks-test first"
			return
		
		if motiflength == -1:
			motiflength = mlen
		print "motiflength:", motiflength
		
		print mw
		for i in range(len(mw)):
			seqid, pos, length = mw[i]
			mw[i] = (seqid, pos+alignment[i], mlen)
		print alignment
		print mw
			
		dataset.setMotifHitlist(mw)
		
		if dataset.getMaxMotifLength() < motiflength:
			print "skipping: maximum motiflength exceeded", dataset._filename, model.id, motiflength, scorefunction.id
			return
		
		
		
		b = Basic(None, None, None, None, None,	None, None, excluded=excluded)
		type = -1
		
		
		mi = dataset.getMotifIndexes()
		totalMotifs = len(mi)
		motifsFound = 0
		
		
		
		sequences = dataset.getSequences()
		self.cvnum = 1
		for i in range(0, len(mi), self.cvnum):
			ci = mi[i:i+self.cvnum]
			nci = mi[:i]+mi[i+self.cvnum:] #Positions found in nci should not be in ci
			
			
			currentTP = len(ci)
			
			d = getEmptySet()
			mk = motif_known()
			
			for l in range(len(ci)):
				j, k = ci[l]
				d._sequence_list.append(sequence(l, sequences[j][k:k+motiflength]))
				mk.add_hitlist_entry(l, 0, motiflength)
			
			d._motif_list.append(mk)
			
			
			
					
			for j in range(len(ci)):
				l, m = ci[j]
				for k in range(m-motiflength+1, m)+range(m+1, m+motiflength):
					if (l, k) not in nci: ci.append((l, k) )
			#print ci
			p = b.runTest(model, dataset, scorefunction, motiflength,\
				0, hdistSpecific, regSpecific, excluded=ci)
			p = p.m
			print p
			
			
			
			#print p
			#print "."*50
			p.search(d, scorefunction)
			print "."*50
			motifsFound += p.TP
			bah = []
			for alle, blabber in nci: bah.append(sequences[alle][blabber:blabber+motiflength])
			print d.getSequences(), bah
			print p
		
		f = file(dataset._filename+"_-_"+str(model.id)+"_-_"+str(motiflength)+"_-_"+str(scorefunction.id), "w")
		f.write(str(totalMotifs)+","+str(motifsFound)+","+str(float(motifsFound)/totalMotifs))
		f.close()
		"motifs found:", motifsFound
		
		dataset.setMotifHitlist(resetlist)

"""
class RandomAlignPositive(Super):
	#Warning: this class ignores the motiflength-parameter, as it is always set to the size of the smallest window for alignment purposes
	def runTest(self, model, dataset, scorefunction, motiflength, writeMethod, hdistSpecific=None):
		#tm = "randap"
		tm = RAP
		
		
		c = ConfigParser()
		f = file("blocksindexes.conf")
		c.readfp(f)
		f.close()
		mlen, alignment = None, None
		if c.has_option("length", dataset._filename):
			mlen = c.getint("length", dataset._filename)
			alignment = c.get("indexes", dataset._filename)[1:][:-1].split(",")
			for i in range(len(alignment)):
				alignment[i] = int(alignment[i])
			
			
		else:
			print "No entry found for dataset, test dataset with Batchtests.Blocks first"
			return

		
		
		
		#Fetch windowlist
		wi = dataset.getMotifWindows()
		wi.sort()
		
		#Create a new dataset, and use the windows from the original dataset as sequences
		self.d = getEmptySet()
		mk = motif_known()
		self.d._motif_list.append(mk)
		seqs = dataset.getSequences()
		i=0
		
		maxAllowedMotiflength = dataset.getMaxMotifLength()
		for line in wi:
			self.d._sequence_list.append( sequence(i, seqs[line[0]][line[1]:line[1]+line[2]]))
			i += 1
			if maxAllowedMotiflength > line[2]: maxAllowedMotiflength = line[2]

		if maxAllowedMotiflength>mlen:
			motiflength = mlen
		else:
			motiflength = maxAllowedMotiflength
			
		print motiflength
		
		
		#Hitlist to use in recursive search
		self.temp = range(len(wi))
		#Current best search result (Pattern, (score, TP, FN, FP, TN))
		self.result = [0, [scorefunction.valueRange[0]]]
		
		self.hls = []
		self.numPossibilities = 1.0
		for line in wi:
			self.numPossibilities *= line[2]-motiflength+1
		
		self.progress = 0.0
		
		self.start, self.t = time.clock(), time.clock()
		self.rec(wi, 0, motiflength, model, scorefunction)
		
	
	def rec(self, l, i, motiflength, model, scorefunction):

		for j in range(l[i][2]-motiflength+1):
			self.temp[i] = (i, j, motiflength)
			if i == len(l)-1:
				
				self.progress += 1
				if time.clock()-self.t>10:
					self.t = time.clock()
					print "progress:", 100*self.progress/self.numPossibilities, "%"
					print "time:", time.clock()-self.start
					print "approx. time left:", (time.clock()-self.start)/((self.progress/self.numPossibilities)) / 60
					
				
				
				self.d.setMotifHitlist(self.temp)
				res = model.train(dataset = self.d, cl = scorefunction, motifLength = motiflength)
				if res[1][0] > self.result[1][0]:
					
					self.result = res
					self.hls = [self.d.getMotifIndexes()]
					print "check=", res[1][0]
				elif res[1][0] == self.result[1][0]:
					
					self.hls.append(self.d.getMotifIndexes())
					
					
				
			elif i == 0:
				#print i, self.hls

				self.rec(l, i+1, motiflength, model, scorefunction)

			else:
				self.rec(l, i+1, motiflength, model, scorefunction)
				
""""""
		
		
		
class RandomAlignAll(Super):
	def runTest(self, model, dataset, scorefunction, motiflength, hdistSpecific=None):
		tm = RAA
		
		
		wi = dataset.getMotifWindows()
		wi.sort()


		self.temp = range(len(wi))
		self.i = 0
		self.result = [0, [0]]
		self.hls = []
		self.rec(wi, 0, motiflength, model, dataset, scorefunction)
		print self.hls
	
	def rec(self, l, i, motiflength, model, dataset, scorefunction):
		for j in range(l[i][1], l[i][1]+l[i][2]-motiflength+1):
			self.temp[i] = (l[i][0], j, motiflength)
			if i == len(l)-1:
				
				dataset._motif_list[0].set_hitlist(self.temp)
				res = model.train(dataset = dataset, cl = scorefunction, motifLength = motiflength)
				if res[1][0] > self.result[1][0]:
					self.result = res
					self.hls = [dataset.getMotifIndexes()]
					print "check=", res[1][0]
				elif res[1][0] == self.result[1][0]:
					
					self.hls.append(dataset.getMotifIndexes())
				
					
				
			elif i == 3:
				print self.i, self.hls
				self.i += 1
				self.rec(l, i+1, motiflength, model, dataset, scorefunction)

			else:
				self.rec(l, i+1, motiflength, model,dataset, scorefunction)






if __name__ == "__main__":
	import main
	main.main()