import Ini
Ini.versionInit()
from DB_constants import HDISTID, IUPACID, PWMID, PBMID, OPTIMALID
VERBOSE = True


import SearchUtils
import copy
import pickle
from math import sqrt, log
import ScoreFunc
import BC
import time
#from pylab import *
import os

from sets import Set
import Patterns

HDISTMODULE=True
"Whether or not the Hamming distance module from C++ exists for this OS"

try:
	import hdist
except:
	HDISTMODULE=False




class Model:
	"The superclass of every model algorithm class."
	def __init__(self):
		pass
	def train(self):
		"The train-function trains a pattern model by the given attributes"
		pass
	def search(self):
		"A deprecated search function. Pattern searching is now done by the classes in the L{Patterns} class."
		pass
	def setScoringFunction(self, sf):
		"""Sets the model instance's scoring function, which is used when training.
		@param sf: The scoring function to set
		@type sf: L{ScoreFunc}
		@return: None
		"""
		self.scoringFunction = sf
		"The current scoring function of the model."
	
	def setDataset(self, dataset):
		"""Sets the model instance's training dataset.
		@param dataset: The dataset 
		@return: None
		"""
		self.dataset = dataset
		"The current scoring function of the model."


class Mismatch_cmodule(Model):
	"Hamming distance model using a fast Hamming distance algorithm written in C++"
	
	
	#: Short model description
	description = "Hamming Distance model"
	def __init__(self):
		
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		self.motifseeds = []
		self.result = [0]
		self.maxmut = 0
		self.trainingName = ""
		
		self.mismatchMethod = SearchUtils.mismatch
		
		
	
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		self.conpositions = []
		self.consequences = ""
		self.endpositions = []
		curpos = 0
		posimap = {}
		for i in range(len(self.sequences)):
			line = self.sequences[i]
			if len(line)==0:continue
			posimap[i] = curpos
			curpos += len(line)
			for j in range(curpos-self.motiflength+1, curpos):
				
				self.endpositions.append(j)
			
			self.consequences += line
		
		for i, j in self.positions:
			self.conpositions.append(posimap[i]+j)
		
		
		
		for i, j in self.positions:
			motif = self.sequences[i][j:j+self.motiflength]
		
		self.totalPositions = 0
		for line in self.sequences:
			if len(line) > 0: self.totalPositions += len(line) - self.motiflength + 1
		self.hdistobject = hdist.new_hdist(self.consequences, len(self.consequences), len(self.endpositions))
		self.hitsArray = hdist.getHits(self.hdistobject)
		for i in range(len(self.endpositions)):
			hdist.addExcludedIndex(self.hdistobject, self.endpositions[i], i)
			

	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		if VERBOSE: print "excluded:", self.excluded
		self.__preprocess(dataset)
				
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
			
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			if VERBOSE: print "training with distance", i
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
			
			
	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)
		
		self.setMotifSeeds()
		
		
		self.best = 0
		self.bestresult = None
				
		#Step 1:
		for motifseed in self.motifseeds:
			if VERBOSE: print "\tcurrent motif:", motifseed
			self.currentMotif = motifseed

			#counter += 1
			for i in range((self.maxmut+1)):
				
				if i == 0:
					#Step 5: The special case where the number of mutations is 0
					hits = Set(self.chdist(self.currentMotif))
					indexSet =Set(self.conpositions)
					TP = len(hits&indexSet)
					FP = len(hits-indexSet)
					FN = len(indexSet-hits)
					TN = self.totalPositions - TP - FP - FN

					
					if self.scoringFunction.fitness(TP, FN, FP, TN) > self.best:
						self.bestresult = (self.currentMotif, (self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN))
						self.best=self.bestresult[1][0]
					
					
					
					
				else:
					self.currentMut = i
					self.mutLetters = range(0, self.currentMut)
					self.indexes = range(0, self.currentMut)
					#Step 2
					self.genIndexes()
			
		
		best = self.scoringFunction.valueRange[0]
		
		return(self.bestresult,  self.maxmut)
		
	def chdist(self, pattern):
		"hamming distance"
		
		hitcount = hdist.search(self.hdistobject, pattern, len(pattern), self.maxmut)
		ret = []
		for i in range(hitcount):
			index = hdist.getIndex(self.hitsArray, i)
			ret.append(index)
		return ret
		
		
	
	
	#Step 2
	def genIndexes(self):
		"""Generates all possible letter positions in a string
		
		Usage: genIndexes()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genInd(0,0)
	
	
	def __genInd(self, lvl, prev):
		"""Should not be called directly, but via genIndexes()"""
		for i in range(prev, (self.motiflength-(self.currentMut-lvl-1))):
			self.indexes[lvl]=i
			if lvl < (self.currentMut-1):
				self.__genInd(lvl+1, i+1)
			else:
				#Step 3
				self.genLetters()
				
	
	#Step 3    
	def genLetters(self):
		"""Generates from 0 to Mo_mismatch_basic.maxmut lettercombos
		
		Usage: genLetters()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genLet(0)
	
	def __genLet(self, lvl):
		"""Should not be called directly, but via genLetters()"""
		for i in self.letters[self.currentMotif[self.indexes[lvl]]]:
			self.mutLetters[lvl] = i;
			if lvl<(self.currentMut-1):
				self.__genLet(lvl+1);
			else:
				#Step 4
				tempMotif = list(self.currentMotif)
				for i, index in enumerate(self.indexes):
					tempMotif[index] = self.mutLetters[i]
				tempMotif = "".join(tempMotif)
				
				
				#Step 5
				
				hits = Set(self.chdist(tempMotif))
				indexSet =Set(self.conpositions)
				TP = len(hits&indexSet)
				FP = len(hits-indexSet)
				FN = len(indexSet-hits)
				TN = self.totalPositions - TP - FP - FN - len(self.excluded)
				
				if self.scoringFunction.fitness(TP, FN, FP, TN) > self.best:
					self.bestresult = (tempMotif, (self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN))
					self.best=self.bestresult[1][0]
										
				
					
	def setMotifSeeds(self):
		motifs = []
		for i in self.conpositions:
			motifs.append(self.consequences[i:i+self.motiflength])
		
		self.motifseeds = list(Set(motifs))
		
		
					
class Mismatch_lessexhaustive(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.trainingName = ""
		self.mismatchMethod = SearchUtils.mismatch
		self.alphabet = "acgt"
	
	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		
		self.duh = []
		for i in range(self.motiflength): self.duh.append(set())

		
		for i, j in self.positions:
			motif = self.sequences[i][j:j+self.motiflength]
			for k, letter in enumerate(motif):
				self.duh[k].add(letter)
		for i in range(len(self.duh)):
			self.duh[i] = list(self.duh[i])
		print self.duh
		
		
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.totalPositions = 0
		self.best = self.scoringFunction.valueRange[0]
		for line in self.sequences:
			self.totalPositions += len(line)-self.motiflength+1
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)
		
		
		for i in r:
			print "rall", i
			self.maxmut = i
			temp = self.__train()
			
			
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
	def combinations(self, level, length, current):
		
		if level == length:
			pattern =  "".join(current)
			hits = Set(self.mismatchMethod(self.sequences, pattern, self.maxmut)) - Set(self.excluded)
			indexSet = Set(self.positions) - Set(self.excluded)
			TP = len(hits&indexSet)
			FP = len(hits-indexSet)
			FN = len(indexSet-hits)
			TN = self.totalPositions - TP - FP - FN - len(self.excluded)
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			if check > self.best: 
				self.best = check
				self.motif = pattern, self.maxmut
				self.result = check, TP, FN, FP, TN
			#self.summary[self.currentMotif] = , TP, FN, FP, TN
			
		else:
			for i in self.duh[level]:
				current[level] = i
				self.combinations(level+1, length, current)
			
	def __train(self):
		#Private function used by train()
		
		self.combinations(0, self.motiflength, [0]*self.motiflength)
		self.best = self.scoringFunction.valueRange[0]
		self.bestresult = None
		
	

class Mismatch_exhaustive(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.trainingName = ""
		self.mismatchMethod = SearchUtils.mismatch
		self.alphabet = "acgt"
	
	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.totalPositions = 0
		self.best = self.scoringFunction.valueRange[0]
		for line in self.sequences:
			self.totalPositions += len(line)-self.motiflength+1
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)
		
		
		for i in r:
			print "rall", i
			self.maxmut = i
			temp = self.__train()
			
			
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
	def combinations(self, level, length, current):
		
		if level == length:
			pattern =  "".join(current)
			hits = Set(self.mismatchMethod(self.sequences, pattern, self.maxmut)) - Set(self.excluded)
			indexSet = Set(self.positions) - Set(self.excluded)
			TP = len(hits&indexSet)
			FP = len(hits-indexSet)
			FN = len(indexSet-hits)
			TN = self.totalPositions - TP - FP - FN - len(self.excluded)
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			if check > self.best: 
				self.best = check
				self.motif = pattern, self.maxmut
				self.result = check, TP, FN, FP, TN
			#self.summary[self.currentMotif] = , TP, FN, FP, TN
			
		else:
			for i in self.alphabet:
				current[level] = i
				self.combinations(level+1, length, current)
			
	def __train(self):
		#Private function used by train()
		
		self.combinations(0, self.motiflength, [0]*self.motiflength)
		self.best = self.scoringFunction.valueRange[0]
		self.bestresult = None
		
		
		"""
		hits = Set(self.mismatchMethod(self.sequences, self.currentMotif, self.maxmut)) - Set(self.excluded)
		indexSet = Set(self.positions) - Set(self.excluded)
		TP = len(hits&indexSet)
		FP = len(hits-indexSet)
		FN = len(indexSet-hits)
		TN = self.totalPositions - TP - FP - FN - len(self.excluded)
		
		self.summary[self.currentMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
		"""			
						
						
					
					
					

		
		

	







class Mismatch_basic(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		
		self.result = [0]
		self.maxmut = 2
		self.trainingName = ""
		self.mismatchMethod = SearchUtils.mismatch
		
		
	
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		
		

		
		for i, j in self.positions:
			motif = self.sequences[i][j:j+self.motiflength]
	
		self.totalPositions = 0
		for line in self.sequences:
			if len(line)> 0: self.totalPositions += len(line) - self.motiflength + 1
		

	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		if VERBOSE: print "excluded:", self.excluded
		self.__preprocess(dataset)
		
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
		
		
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			if VERBOSE: print "training with distance", i
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result

	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)

		self.setMotifSeeds()
		
		self.best = 0
		self.bestresult = None

		#Step 1:

		for motifseed in self.motifseeds:
			self.currentMotif = motifseed
			if VERBOSE: print "current motif:", motifseed
			for i in range((self.maxmut+1)):
				if i == 0:
					#Step 5: The special case where the number of mutations is 0
					hits = Set(self.mismatchMethod(self.sequences, self.currentMotif, self.maxmut)) - Set(self.excluded)
					indexSet = Set(self.positions) - Set(self.excluded)
					TP = len(hits&indexSet)
					FP = len(hits-indexSet)
					FN = len(indexSet-hits)
					TN = self.totalPositions - TP - FP - FN - len(self.excluded)
					if self.scoringFunction.fitness(TP, FN, FP, TN) > self.best:
						self.bestresult = (self.currentMotif, (self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN))
						self.best=self.bestresult[1][0]
						
				else:
					self.currentMut = i
					self.mutLetters = range(0, self.currentMut)
					self.indexes = range(0, self.currentMut)
					#Step 2
					self.genIndexes()
			
		
		return(self.bestresult,  self.maxmut)
	
	#Step 2
	def genIndexes(self):
		"""Generates all possible letter positions in a string
		
		Usage: genIndexes()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genInd(0,0)
	
	
	def __genInd(self, lvl, prev):
		"""Should not be called directly, but via genIndexes()"""
		for i in range(prev, (self.motiflength-(self.currentMut-lvl-1))):
			self.indexes[lvl]=i
			if lvl < (self.currentMut-1):
				self.__genInd(lvl+1, i+1)
			else:
				#Step 3
				self.genLetters()
				
	
	#Step 3    
	def genLetters(self):
		"""Generates from 0 to Mo_mismatch_basic.maxmut lettercombos
		
		Usage: genLetters()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genLet(0)
	
	def __genLet(self, lvl):
		"""Should not be called directly, but via genLetters()"""
		
		
		for i in self.letters[self.currentMotif[self.indexes[lvl]]]:
		
			self.mutLetters[lvl] = i;
			
			if lvl<(self.currentMut-1):
				self.__genLet(lvl+1);
			else:
				#Step 4
				tempMotif = list(self.currentMotif)
				for i, index in enumerate(self.indexes):
					tempMotif[index] = self.mutLetters[i]
				tempMotif = "".join(tempMotif)
				
				#Step 5
				hits = Set(self.mismatchMethod(self.sequences, tempMotif, self.maxmut)) - Set(self.excluded)
				indexSet = Set(self.positions) - Set(self.excluded)
				TP = len(hits&indexSet)
				FP = len(hits-indexSet)
				FN = len(indexSet-hits)
				TN = self.totalPositions - TP - FP - FN - len(self.excluded)
				
				
				
				if self.scoringFunction.fitness(TP, FN, FP, TN) > self.best:
					self.bestresult = (tempMotif, (self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN))
					self.best=self.bestresult[1][0]
				
	def setMotifSeeds(self):
		motifs = []
		for i, j in self.positions:
			motifs.append(self.sequences[i][j:j+self.motiflength])
		
		self.motifseeds = list(Set(motifs))
		


class Mismatch_qgram_filter(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		self.qlength = 3
		self.result = [0]
		self.maxmut = 2
		self.trainingName = ""
		#self.mismatchMethod = SearchUtils.mismatchExperimental
		self.mismatchMethod = SearchUtils.mismatch
		
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		self.setpositions = Set(self.positions)
		self.qg = []
		
		
		
		for i in range(len(self.sequences)):
			sequence = self.sequences[i]
			seqq = {}
			for j in range(len(sequence)-self.qlength+1):
				qgram = sequence[j:j+self.qlength]
				if qgram not in seqq: seqq[qgram] = [j]
				else: seqq[qgram].append(j)
			self.qg.append(seqq)

		
		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1
		
	
	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		
		self.__preprocess(dataset)
		
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
		
		
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			print "rall", i
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			

			
			
			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
			
			
	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)
		
		self.qthreshold = self.motiflength - self.qlength + 1 - (self.maxmut*self.qlength)
		print "threshold:", self.qthreshold
		self.setMotifSeeds()
		
		self.best = 0
		self.bestresult = None
		self.summary = dict()
		
		
		#Step 1:

		for motifseed in self.motifseeds:
			print "ball"
			
			self.currentMotif = motifseed


			#counter += 1
			for i in range((self.maxmut+1)):
				
				if i == 0:
					#Step 5: The special case where the number of mutations is 0
					if not self.summary.has_key(self.currentMotif):
						tempmotif = self.currentMotif
						#print tempmotif, self.positions
						hits = Set()
						for k in range(len(self.sequences)):
							sequence = self.sequences[k]
							
							qgramhits = []
							for j in range(self.motiflength - self.qlength + 1):
								tqgram = tempmotif[j:j+self.qlength]
								if tqgram in self.qg[k]:
									qgramhits +=  self.qg[k][tqgram]
							
							qgramhits.sort()
							#print qgramhits
							for j in range(len(qgramhits)-self.qthreshold+1):
								minwin, maxwin = qgramhits[j], qgramhits[j+self.qthreshold-1]
								width = maxwin - minwin
								if  width <= self.motiflength-self.qlength+1:
									slack = self.motiflength - width
									minslack, maxslack = minwin - slack, maxwin + slack
									if minslack < 0: minslack = 0
									if maxslack > len(sequence): maxslack = len(sequence)
									for m in range(minslack, maxslack+1):
										if SearchUtils.hdistance(sequence[m:m+self.motiflength], tempmotif, self.maxmut):
											hits.add((k, m))
						
						indexSet = self.setpositions
						TP = len(hits&indexSet)
						FP = len(hits-indexSet)
						FN = len(indexSet-hits)
						TN = self.totalPositions - TP - FP - FN
							
						
							
							
							
							
							
							
						#self.mismatchMethod(, tempMotif, self.maxmut)
						
						
						
						#TP, FP = SearchUtils.mismatchQgram(tempmotif, self.maxmut, self.positive, self.negative)
						#FN = self.totpos - TP
						#TN = self.totneg - FP
						

						self.summary[self.currentMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
						
						
						
					
					
					
				else:
					self.currentMut = i
					self.mutLetters = range(0, self.currentMut)
					self.indexes = range(0, self.currentMut)
					#Step 2
					self.genIndexes()
			#self.currentMotif = self.dataset.getNextMotif()
		
		best = self.scoringFunction.valueRange[0]
		r = None
		for key, value in  self.summary.iteritems():
			if value[0] > best:
				r = (key, value)
				best = value[0]
		if self.summary.has_key("cgggt"):print "rallrall", self.summary["cgggt"]
		return(r,  self.maxmut)
		
		#return(self.bestresult, self.maxmut)
		
	
	#Step 2
	def genIndexes(self):
		"""Generates all possible letter positions in a string
		
		Usage: genIndexes()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genInd(0,0)
	
	
	def __genInd(self, lvl, prev):
		"""Should not be called directly, but via genIndexes()"""
		for i in range(prev, (self.motiflength-(self.currentMut-lvl-1))):
			self.indexes[lvl]=i
			if lvl < (self.currentMut-1):
				self.__genInd(lvl+1, i+1)
			else:
				#Step 3
				self.genLetters()
				
	
	#Step 3    
	def genLetters(self):
		"""Generates from 0 to Mo_mismatch_basic.maxmut lettercombos
		
		Usage: genLetters()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genLet(0)
	
	def __genLet(self, lvl):
		"""Should not be called directly, but via genLetters()"""
		
		
		for i in self.letters[self.currentMotif[self.indexes[lvl]]]:
		#for i in self.duh[self.indexes[lvl]]:
			self.mutLetters[lvl] = i;
			if lvl<(self.currentMut-1):
				self.__genLet(lvl+1);
			else:
				#Step 4
				tempMotif = list(self.currentMotif)
				for i, index in enumerate(self.indexes):
					tempMotif[index] = self.mutLetters[i]
				tempMotif = "".join(tempMotif)
				
				
				#counter = 2
				#for line in self.posset:
				#	if SearchUtils.hdistance(line, tempMotif, self.currentMut): counter += 1
				
				
				
				#Step 5
				if not self.summary.has_key(tempMotif):
					
					
					
					
					
					tempmotif = tempMotif
					#TP, FP = SearchUtils.mismatchQgram(tempmotif, self.maxmut, self.positive, self.negative)
					#FN = self.totpos - TP
					#TN = self.totneg - FP
					#print tempmotif, self.positions
					hits = Set()
					for k in range(len(self.sequences)):
						sequence = self.sequences[k]
						
						qgramhits = []
						for j in range(self.motiflength - self.qlength + 1):
							tqgram = tempmotif[j:j+self.qlength]
							if tqgram in self.qg[k]:
								qgramhits +=  self.qg[k][tqgram]
						
						qgramhits.sort()
						#print qgramhits
						for j in range(len(qgramhits)-self.qthreshold+1):
							minwin, maxwin = qgramhits[j], qgramhits[j+self.qthreshold-1]
							width = maxwin - minwin
							if  width <= self.motiflength-self.qlength+1:
								slack = self.motiflength - width
								minslack, maxslack = minwin - slack, maxwin + slack
								if minslack < 0: minslack = 0
								if maxslack > len(sequence): maxslack = len(sequence)
								for m in range(minslack, maxslack+1):
									0
									if SearchUtils.hdistance(sequence[m:m+self.motiflength], tempmotif, self.maxmut):
										hits.add((k, m))
					
					indexSet = self.setpositions
					TP = len(hits&indexSet)
					FP = len(hits-indexSet)
					FN = len(indexSet-hits)
					TN = self.totalPositions - TP - FP - FN
					
					
					
					self.summary[tempMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
				
	def setMotifSeeds(self):
		motifs = []
		for i, j in self.positions:
			motifs.append(self.sequences[i][j:j+self.motiflength])
		
		self.motifseeds = list(Set(motifs))
		"rall"



class Mismatch_qgram(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		
		self.result = [0]
		self.maxmut = 2
		self.trainingName = ""
		#self.mismatchMethod = SearchUtils.mismatchExperimental
		self.mismatchMethod = SearchUtils.mismatch
		
		
	
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		self.positive = {}
		self.negative = {}
		self.bspositive = {}
		self.bsnegative = {}
		#self.nmpositive = [0 for i in range(4*motiflength)]
		
		
		
		bitpos = {"A":0, "a":0, "C":1, "c":1, "G":2, "g":2, "T":3, "t":3}
		
		self.totpos = 0
		self.totneg = 0
		
		for i in range(len(self.sequences)):
			sequence = self.sequences[i]
			for j in range(len(sequence)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				sub = sequence[j:j+self.motiflength]
				if (i, j) in self.positions:
					if self.positive.has_key(sub): self.positive[sub]+=1
					else: self.positive[sub] = 1
					self.totpos+=1
				else:
					if self.negative.has_key(sub): self.negative[sub]+=1
					else: self.negative[sub] = 1
					self.totneg+=1
		
		
		
		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1
		self.duh = []
		for i in range(self.motiflength): self.duh.append(set())

		
		for i, j in self.positions:
			motif = self.sequences[i][j:j+self.motiflength]
			for k, letter in enumerate(motif):
				self.duh[k].add(letter)
		for i in range(len(self.duh)):
			self.duh[i] = list(self.duh[i])
	
	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		print "excluded:", self.excluded
		self.__preprocess(dataset)
		#self.dataset = dataset
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
		
		
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			print "rall", i
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			

			
			
			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		print self.motif[0], self.motif[1], self.result
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
			
			
	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)
		
		
		self.setMotifSeeds()
		
		self.best = 0
		self.bestresult = None
		self.summary = dict()
		
		
		#Step 1:

		for motifseed in self.motifseeds:

			
			self.currentMotif = motifseed


			#counter += 1
			for i in range((self.maxmut+1)):
				
				if i == 0:
					#Step 5: The special case where the number of mutations is 0
					if not self.summary.has_key(self.currentMotif):
						tempmotif = self.currentMotif
						TP, FP = SearchUtils.mismatchQgram(tempmotif, self.maxmut, self.positive, self.negative)
						
						FN = self.totpos - TP
						TN = self.totneg - FP
						

						self.summary[self.currentMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
						
						
						
					
					
					
				else:
					self.currentMut = i
					self.mutLetters = range(0, self.currentMut)
					self.indexes = range(0, self.currentMut)
					#Step 2
					self.genIndexes()
			#self.currentMotif = self.dataset.getNextMotif()
		
		best = self.scoringFunction.valueRange[0]
		r = None
		for key, value in  self.summary.iteritems():
			if value[0] > best:
				r = (key, value)
				best = value[0]
		if self.summary.has_key("cgggt"):print "rallrall", self.summary["cgggt"]
		return(r,  self.maxmut)
		
		#return(self.bestresult, self.maxmut)
		
	
	#Step 2
	def genIndexes(self):
		"""Generates all possible letter positions in a string
		
		Usage: genIndexes()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genInd(0,0)
	
	
	def __genInd(self, lvl, prev):
		"""Should not be called directly, but via genIndexes()"""
		for i in range(prev, (self.motiflength-(self.currentMut-lvl-1))):
			self.indexes[lvl]=i
			if lvl < (self.currentMut-1):
				self.__genInd(lvl+1, i+1)
			else:
				#Step 3
				self.genLetters()
				
	
	#Step 3    
	def genLetters(self):
		"""Generates from 0 to Mo_mismatch_basic.maxmut lettercombos
		
		Usage: genLetters()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genLet(0)
	
	def __genLet(self, lvl):
		"""Should not be called directly, but via genLetters()"""
		
		
		for i in self.letters[self.currentMotif[self.indexes[lvl]]]:
		#for i in self.duh[self.indexes[lvl]]:
			self.mutLetters[lvl] = i;
			if lvl<(self.currentMut-1):
				self.__genLet(lvl+1);
			else:
				#Step 4
				tempMotif = list(self.currentMotif)
				for i, index in enumerate(self.indexes):
					tempMotif[index] = self.mutLetters[i]
				tempMotif = "".join(tempMotif)
				
				
				#counter = 2
				#for line in self.posset:
				#	if SearchUtils.hdistance(line, tempMotif, self.currentMut): counter += 1
				
				
				
				#Step 5
				if not self.summary.has_key(tempMotif):
					
					
					
					
					
					tempmotif = tempMotif
					TP, FP = SearchUtils.mismatchQgram(tempmotif, self.maxmut, self.positive, self.negative)
					FN = self.totpos - TP
					TN = self.totneg - FP
					
					
					
					
					self.summary[tempMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
				
	def setMotifSeeds(self):
		motifs = []
		for i, j in self.positions:
			motifs.append(self.sequences[i][j:j+self.motiflength])
		
		self.motifseeds = list(Set(motifs))
		"rall"
				
"""
................................................................................
Mo_mismatch_tree, pattern-driven Hamming distance model
................................................................................
"""
class Mismatch_tree2(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		
		self.result = [0]
		self.maxmut = 2
		self.trainingName = ""
		#self.mismatchMethod = SearchUtils.mismatchExperimental
		self.mismatchMethod = SearchUtils.mismatch
		"rall"
		"""
		f = file("c:\\testetest")
		table = []
		line = f.readline()
		while line:
			table.append(line.strip())
			line = f.readline()
		f.close()
		self.profile = dict(zip(table, [0]*len(table)))
		"""
	
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		self.positive = []
		self.negative = []
		self.posMatrix = []
		self.negMatrix = []
		self.lettergroup = "acgt"
		self.totpos = 0
		self.totneg = 0
		
		
		
		
		for i in range(len(self.sequences)):
			sequence = self.sequences[i]
			for j in range(len(sequence)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				sub = sequence[j:j+self.motiflength]
				if (i, j) in self.positions:
					self.positive.append(sub)
					self.totpos+=1
				else:
					self.negative.append(sub)
					self.totneg+=1
		
		self.posstring = BC.bitstring(0, 0)
		self.negstring = BC.bitstring(0, 0)
		for i in range(self.totpos): self.posstring.setBit(i)
		for i in range(self.totneg): self.negstring.setBit(i)
		
		
		
		
		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1

		for i in range(self.motiflength):
			tmppos = {}
			tmpneg = {}
			for l in self.lettergroup:
				tmpbs = BC.bitstring(0, 0)
				tmpbsnot = BC.bitstring(0, 0)
				for j in range(len(self.positive)):
					if self.positive[j][i] == l: tmpbs.setBit(j)
					else: tmpbsnot.setBit(j)
				tmppos[l] = tmpbs
				tmppos["~"+l] = tmpbsnot
				
				
				
				tmpbs = BC.bitstring(0, 0)
				tmpbsnot = BC.bitstring(0, 0)
				for j in range(len(self.negative)):
					if self.negative[j][i] == l: tmpbs.setBit(j)
					else: tmpbsnot.setBit(j)
				tmpneg[l] = tmpbs
				tmpneg["~"+l] = tmpbsnot
				
				
			self.posMatrix.append(tmppos)
			self.negMatrix.append(tmpneg)
		

	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		print "excluded:", self.excluded
		self.__preprocess(dataset)
		#self.dataset = dataset
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
		
		
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			

			
			
			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
			
			
	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)
		
		
		
		
		self.best = 0
		self.bestresult = None
		self.summary = dict()
		self.summary[self.currentMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
		best = self.scoringFunction.valueRange[0]
		r = None
		for key, value in  self.summary.iteritems():
			if value[0] > best:
				r = (key, value)
				best = value[0]
		return(r,  self.maxmut)
		#Step 1:

			
	def msearch(self):
		self.tempTP = 0
		self.tempFP = 0
		self.recsearch(0, 0, pattern, self.posstring, self.negstring)
		return self.tempTP, self.tempFP
		
	def recsearch(self, pos, dist, pattern, posString, negString):
		if dist > self.maxmut: return
		if pos == self.motiflength - 1:
			#print dist
			
			matchPosString = posString.andStrings(self.posMatrix[pos][pattern[pos]])
			
			matchNegString = negString.andStrings(self.negMatrix[pos][pattern[pos]])
			
			self.tempTP += matchPosString.countBits() 
			self.tempFP += matchNegString.countBits() 
			
			if dist < self.maxmut:
				mismatchPosString = posString.andStrings(self.posMatrix[pos]["~"+pattern[pos]])
				mismatchNegString = negString.andStrings(self.negMatrix[pos]["~"+pattern[pos]])
				self.tempTP += mismatchPosString.countBits()
				self.tempFP += mismatchNegString.countBits()
			#print dist
			#print matchPosString
			#print mismatchPosString
		else:
			matchPosString = posString.andStrings(self.posMatrix[pos][pattern[pos]])
			matchNegString = negString.andStrings(self.negMatrix[pos][pattern[pos]])
			
			
					
			
			
			
			self.recsearch(pos+1,dist, pattern, matchPosString, matchNegString)
			
			if dist < self.maxmut:
				mismatchPosString = posString.andStrings(self.posMatrix[pos]["~"+pattern[pos]])
				mismatchNegString = negString.andStrings(self.negMatrix[pos]["~"+pattern[pos]])
				
				
				self.recsearch(pos+1, dist+1, pattern, mismatchPosString, mismatchNegString)
			
			
		
		

	
	


"""
................................................................................
Mo_mismatch_tree, pattern-driven Hamming distance model
................................................................................
"""
class Mismatch_tree(Model):
	"""A pattern-driven Hamming distance model""" 
	def __init__(self):
		self.description = "Hamming Distance model"
		self.id = HDISTID
		self.letters = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
						"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}
		

		
		self.result = [0]
		self.maxmut = 2
		self.trainingName = ""
		#self.mismatchMethod = SearchUtils.mismatchExperimental
		self.mismatchMethod = SearchUtils.mismatch
		"rall"
		"""
		f = file("c:\\testetest")
		table = []
		line = f.readline()
		while line:
			table.append(line.strip())
			line = f.readline()
		f.close()
		self.profile = dict(zip(table, [0]*len(table)))
		"""
	
	def __preprocess(self, dataset):
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()
		self.positive = []
		self.negative = []
		self.posMatrix = []
		self.negMatrix = []
		self.lettergroup = "acgt"
		self.totpos = 0
		self.totneg = 0
		
		
		
		
		for i in range(len(self.sequences)):
			sequence = self.sequences[i]
			for j in range(len(sequence)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				sub = sequence[j:j+self.motiflength]
				if (i, j) in self.positions:
					self.positive.append(sub)
					self.totpos+=1
				else:
					self.negative.append(sub)
					self.totneg+=1
		
		self.posstring = BC.bitstring(0, 0)
		self.negstring = BC.bitstring(0, 0)
		for i in range(self.totpos): self.posstring.setBit(i)
		for i in range(self.totneg): self.negstring.setBit(i)
		
		
		
		
		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1

		for i in range(self.motiflength):
			tmppos = {}
			tmpneg = {}
			for l in self.lettergroup:
				tmpbs = BC.bitstring(0, 0)
				tmpbsnot = BC.bitstring(0, 0)
				for j in range(len(self.positive)):
					if self.positive[j][i] == l: tmpbs.setBit(j)
					else: tmpbsnot.setBit(j)
				tmppos[l] = tmpbs
				tmppos["~"+l] = tmpbsnot
				
				
				
				tmpbs = BC.bitstring(0, 0)
				tmpbsnot = BC.bitstring(0, 0)
				for j in range(len(self.negative)):
					if self.negative[j][i] == l: tmpbs.setBit(j)
					else: tmpbsnot.setBit(j)
				tmpneg[l] = tmpbs
				tmpneg["~"+l] = tmpbsnot
				
				
			self.posMatrix.append(tmppos)
			self.negMatrix.append(tmpneg)
		self.duh = []
		for i in range(self.motiflength): self.duh.append(set())

		
		for i, j in self.positions:
			motif = self.sequences[i][j:j+self.motiflength]
			for k, letter in enumerate(motif):
				self.duh[k].add(letter)
		for i in range(len(self.duh)):
			self.duh[i] = list(self.duh[i])

	
	def train(self, dataset = None, cl=None, motifLength = 4, minmut=1, maxmut=2, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		#Common trainingvariables, used by all models
		self.motiflength = motifLength
		self.scoringFunction = cl
		self.excluded = excluded
		print "excluded:", self.excluded
		self.__preprocess(dataset)
		#self.dataset = dataset
		
		#Unique trainingvariables, used only by this model
		self.minmut = minmut
		self.maxmut = maxmut
		self.motif = None
		self.result = [0]
		self.posset = []
		for sequence_number, motifindex in self.positions:
			self.posset.append(self.sequences[sequence_number][motifindex:motifindex+self.motiflength])
		
		
		
		
		r = []
		
		if self.maxmut >= self.motiflength: r = range(self.minmut, self.motiflength)
		else: r = range(self.minmut, self.maxmut+1)

		
		for i in r:
			
			self.maxmut = i
			temp = self.__train(cl)
			
			result = temp[0][1]

			

			
			
			if result[0] > self.result[0]:
				self.result = result
				self.motif = [temp[0][0], temp[1]]
			
		
		return Patterns.Mismatch(self.motif[0], self.motif[1]), self.result
			
			
			
			
	def __train(self, cl=None):
		"""Private function used by train()"""
		
		
		if cl != None: self.setScoringFunction(cl)
		
		
		
		
		self.best = 0
		self.bestresult = None
		self.summary = dict()
		
		
		#Step 1:


		
		for sequence_number, motifindex in self.positions:

			self.currentMotif= self.sequences[sequence_number][motifindex:motifindex+self.motiflength]


			#counter += 1
			for i in range((self.maxmut+1)):
				
				if i == 0:
					#Step 5: The special case where the number of mutations is 0
					if not self.summary.has_key(self.currentMotif):
						
						TP, FP = self.msearch(self.currentMotif)
						
						FN = self.totpos - TP
						TN = self.totneg - FP
						self.summary[self.currentMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
						#hits = Set(self.mismatchMethod(self.sequences, self.currentMotif, self.maxmut)) - Set(self.excluded)
						#indexSet = Set(self.positions) - Set(self.excluded)
						#TP = len(hits&indexSet)
						#FP = len(hits-indexSet)
						#FN = len(indexSet-hits)
						#TN = self.totalPositions - TP - FP - FN - len(self.excluded)
						
						
						
					
					
					
				else:
					self.currentMut = i
					self.mutLetters = range(0, self.currentMut)
					self.indexes = range(0, self.currentMut)
					#Step 2
					self.genIndexes()
			#self.currentMotif = self.dataset.getNextMotif()
		
		best = self.scoringFunction.valueRange[0]
		r = None
		for key, value in  self.summary.iteritems():
			if value[0] > best:
				r = (key, value)
				best = value[0]
		return(r,  self.maxmut)
		
		#return(self.bestresult, self.maxmut)
		
	
	#Step 2
	def genIndexes(self):
		"""Generates all possible letter positions in a string
		
		Usage: genIndexes()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genInd(0,0)
	
	
	def __genInd(self, lvl, prev):
		"""Should not be called directly, but via genIndexes()"""
		for i in range(prev, (self.motiflength-(self.currentMut-lvl-1))):
			self.indexes[lvl]=i
			if lvl < (self.currentMut-1):
				self.__genInd(lvl+1, i+1)
			else:
				#Step 3
				self.genLetters()
				
	
	#Step 3    
	def genLetters(self):
		"""Generates from 0 to Mo_mismatch_basic.maxmut lettercombos
		
		Usage: genLetters()
		This function is part of the training algorithm for the
		Mo_mismatch_basic - model, and it is not necessary nor recommended to
		call it directly. For more details see Mo_mismatch_basic.train()
			"""
		self.__genLet(0)
	
	def __genLet(self, lvl):
		"""Should not be called directly, but via genLetters()"""
		
		
		for i in self.letters[self.currentMotif[self.indexes[lvl]]]:
		#for i in self.duh[self.indexes[lvl]]:
			self.mutLetters[lvl] = i;
			if lvl<(self.currentMut-1):
				self.__genLet(lvl+1);
			else:
				#Step 4
				tempMotif = list(self.currentMotif)
				for i, index in enumerate(self.indexes):
					tempMotif[index] = self.mutLetters[i]
				tempMotif = "".join(tempMotif)
				
				
				
				#Step 5
				if not self.summary.has_key(tempMotif):
					TP, FP = self.msearch(tempMotif)
					
					FN = self.totpos - TP
					TN = self.totneg - FP
					self.summary[tempMotif] = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
		
					
	def msearch(self, pattern):
		self.tempTP = 0
		self.tempFP = 0
		BC.bitstring(0, 0)
		
		
		
		self.recsearch(0, 0, pattern, self.posstring, self.negstring)
		return self.tempTP, self.tempFP
		
	def recsearch(self, pos, dist, pattern, posString, negString):
		if dist > self.maxmut: return
		if pos == self.motiflength - 1:
			#print dist
			
			matchPosString = posString.andStrings(self.posMatrix[pos][pattern[pos]])
			
			matchNegString = negString.andStrings(self.negMatrix[pos][pattern[pos]])
			
			self.tempTP += matchPosString.countBits() 
			self.tempFP += matchNegString.countBits() 
			
			if dist < self.maxmut:
				mismatchPosString = posString.andStrings(self.posMatrix[pos]["~"+pattern[pos]])
				mismatchNegString = negString.andStrings(self.negMatrix[pos]["~"+pattern[pos]])
				self.tempTP += mismatchPosString.countBits()
				self.tempFP += mismatchNegString.countBits()
			#print dist
			#print matchPosString
			#print mismatchPosString
		else:
			matchPosString = posString.andStrings(self.posMatrix[pos][pattern[pos]])
			matchNegString = negString.andStrings(self.negMatrix[pos][pattern[pos]])
			
			
					
			
			
			
			self.recsearch(pos+1,dist, pattern, matchPosString, matchNegString)
			
			if dist < self.maxmut:
				mismatchPosString = posString.andStrings(self.posMatrix[pos]["~"+pattern[pos]])
				mismatchNegString = negString.andStrings(self.negMatrix[pos]["~"+pattern[pos]])
				
				
				self.recsearch(pos+1, dist+1, pattern, mismatchPosString, mismatchNegString)
			
"""
................................................................................
Mo_regexp_basic_newbs, optimized IUPAC model
................................................................................
"""
LETA, LETC, LETG, LETT = 1, 2, 4, 8

class Regexp(Model):
	"""Optimized IUPAC model"""
	def __init__(self):
		self.description = "Regexp model"
		self.id = IUPACID
		
		self.letterGroup = "acgt"
		
		self.letterIndexes = {LETA:[0], LETC:[1], LETG:[2], LETT:[3], LETA|LETC:[0, 1, 4], LETA|LETG:[0, 2, 5],\
		LETA|LETT:[0, 3, 6], LETC|LETG:[1, 2, 7], LETC|LETT:[1, 3, 8], LETG|LETT:[2, 3, 9], LETA|LETC|LETG:[0, 1, 2, 4, 5, 7, 10],\
		LETA|LETC|LETT:[0, 1, 3, 4, 6, 8, 11], LETA|LETG|LETT:[0, 2, 3, 5, 6, 9, 12], LETC|LETG|LETT:[1, 2,3, 7, 8, 9, 13],\
		LETA|LETC|LETG|LETT:[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]}
		

		
	
	def train(self, dataset=None, cl=None, motifLength=4, brsf = None, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		if brsf == None:
			self.testbest = self.scoringFunction.valueRange[0]
		else:
			self.testbest = brsf
		
		self.motiflength = motifLength
		self.summary = dict()
		self.motif = None
		self.trainingName = ""
		self.excluded = excluded
		self.bsv = BC.bitstringVariables()
		
		self.__preprocess(dataset)

		#Initialize starting bit vectors
		posBitString = BC.bitstring(self.bsv, self.posStringLength)
		for i in range(self.posStringLength):
			posBitString.setBit(i)
		negBitString = BC.bitstring(self.bsv, self.negStringLength)
		for i in range(self.negStringLength):
			negBitString.setBit(i)
		
		for i, line in enumerate(self.positionMatrix):
			for letter in line.keys():
				line["."] = [[posBitString, self.posStringLength], [negBitString, self.negStringLength]]
		
		self.best = self.testbest#brsf

		self.pruned = [0]*self.motiflength
		self.motifList = []

		self.recSearch(["."]*self.motiflength, 0, None, posBitString, negBitString)
		if VERBOSE: print "number of skipped branches at each level:"
		if VERBOSE: print self.pruned
		del self.positionMatrix
		return Patterns.Regexp(self.motifList[0], self.motiflength), self.result

	def recSearch(self, pattern, position, fil, posBitString, negBitString):
		"""The recursive search function, used by train"""
		
		
		recposition = position
		#if position == 3:
		#	print pattern
		#	print self.pruned
			
		#If at bottom of tree (last letter in pattern)
		if position == self.motiflength-1:
			position = self.positionOrder[position]
			letterstring = 0
			_a = posBitString.andStrings(self.positionMatrix[position]["a"][0][0])
			TPa = _a.countBits()
			if TPa > 0: 
				letterstring |=LETA
			
			_c = posBitString.andStrings(self.positionMatrix[position]["c"][0][0])
			TPc = _c.countBits()
			if TPc > 0:
				letterstring |=LETC
			
			
			_g = posBitString.andStrings(self.positionMatrix[position]["g"][0][0])
			TPg = _g.countBits()
			if TPg > 0:
				letterstring |=LETG
			
			
			_t = posBitString.andStrings(self.positionMatrix[position]["t"][0][0])
			TPt = _t.countBits()
			if TPt > 0:
				letterstring |=LETT
			
			del _a, _c, _g, _t
					
			
			TPac = TPa + TPc
			TPag = TPa + TPg
			TPat = TPa + TPt
			TPcg = TPc + TPg
			TPct = TPc + TPt
			TPgt = TPg + TPt
			TPacg = TPac + TPg
			TPact = TPac + TPt
			TPagt = TPag + TPt
			TPcgt = TPcg + TPt
			TPw = TPacg + TPt
			

			FPa, FPc, FPg, FPt, _a, _c, _t, _g = 0, 0, 0, 0, None, None, None, None
			tempList = []
			
			pruned = 0
			if letterstring & LETA:
				_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
				FPa = _a.countBits()
				del _a
				checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
				tempList.append([checka, "a", TPa, FPa])
			else: pruned += 1
			
			if letterstring & LETC:
				_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
				FPc = _c.countBits()
				del _c
				checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
				tempList.append([checkc, "c", TPc, FPc])
				if letterstring & LETA:
					FPac = FPa + FPc
					checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
					tempList.append([checkac, "[ac]", TPac, FPac])
				else: pruned += 1
			else: pruned += 2
			
			if letterstring & LETG:
				_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
				FPg = _g.countBits()
				del _g
				checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
				tempList.append([checkg, "g", TPg, FPg])
				if letterstring & LETA:
					FPag = FPa + FPg
					checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
					tempList.append([checkag, "[ag]", TPag, FPag])
					if letterstring & LETC:
						FPacg = FPag + FPc
						checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
						tempList.append([checkacg, "[acg]", TPacg, FPacg])
					else: pruned += 1
				else: pruned += 2
				if letterstring & LETC:
					FPcg = FPc + FPg
					checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
					tempList.append([checkcg, "[cg]", TPcg, FPcg])
				else: pruned += 1
			else: pruned += 4
			
			if letterstring & LETT:
				_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
				FPt = _t.countBits()
				del _t
				checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
				tempList.append([checkt, "t", TPt, FPt])
				if letterstring & LETA:
					FPat = FPa + FPt
					checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
					tempList.append([checkat, "[at]", TPat, FPat])
					if letterstring & LETC:
						FPact = FPac + FPt
						checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
						tempList.append([checkact, "[act]", TPact, FPact])
						if letterstring & LETG:
							FPw = FPacg + FPt
							checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
							tempList.append([checkw, ".", TPw, FPw])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETG:
						FPagt = FPag + FPt
						checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
						tempList.append([checkagt, "[agt]", TPagt, FPagt])
					else: pruned += 1
				else: pruned += 4
				if letterstring & LETC:
					FPct = FPc + FPt
					checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
					tempList.append([checkct, "[ct]", TPct, FPct])
					if letterstring & LETG:
						FPcgt = FPcg + FPt
						checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
						tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
					else: pruned += 1
				else: pruned += 2
				if letterstring & LETG:
					FPgt = FPg + FPt
					checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
					tempList.append([checkgt, "[gt]", TPgt, FPgt])
				else:pruned += 1
			else: pruned += 8
					
					
			self.pruned[position] += pruned
			
			for line in tempList:
				if line[0] > self.best:
					pattern[position] = line[1]
					self.motifList = ["".join(pattern)]
					self.best = line[0]
					self.result = line[0], line[2], self.posStringLength-line[2], line[3], self.negStringLength-line[3]
				
				
				#Comment the below block if only one of the best motifs should be included
				#Pattern found equally good
				elif line[0] == self.best:
					pattern[position] = line[1]
					self.motifList.append("".join(pattern))

		#If not last letter position:
		else:
			position = self.positionOrder[position]
			letterString = 0
			_a = posBitString.andStrings(self.positionMatrix[position]["a"][0][0])
			TPa = _a.countBits()
			if TPa > 0:
				letterString |=LETA
			_c = posBitString.andStrings(self.positionMatrix[position]["c"][0][0])
			TPc = _c.countBits()
			if TPc>0:
				letterString |=LETC
			_g = posBitString.andStrings(self.positionMatrix[position]["g"][0][0])
			TPg = _g.countBits()
			if TPg>0:
				letterString |=LETG
			_t = posBitString.andStrings(self.positionMatrix[position]["t"][0][0])
			TPt = _t.countBits()
			if TPt>0:
				letterString |=LETT
			del _a, _c, _g, _t
			
			TPac = TPa + TPc
			TPag = TPa + TPg
			TPat = TPa + TPt
			TPcg = TPc + TPg
			TPct = TPc + TPt
			TPgt = TPg + TPt
			TPacg = TPac + TPg
			TPact = TPac + TPt
			TPagt = TPag + TPt
			TPcgt = TPcg + TPt
			TPw = TPacg + TPt



			tList = [[TPa, "a"], [TPc, "c"],\
			[ TPg, "g"], [TPt, "t"],\
			[TPac, "[ac]"], [TPag, "[ag]"],\
			[TPat, "[at]"],	[TPcg, "[cg]"],\
			[TPct, "[ct]"], [TPgt, "[gt]"],\
			[TPacg, "[acg]"], [TPact, "[act]"],\
			[TPagt, "[agt]"], [TPcgt, "[cgt]"],\
			[TPw, "."]]
			
			tempList = []
			for ind in self.letterIndexes[letterString]:
				tempList+=[tList[ind]]
			
			
			for line in tempList:
				check = self.luCheck[line[0]]
				if check > self.best:
					pattern[position] = line[1]
					self.recSearch(pattern, recposition+1, fil, posBitString.andStrings(self.positionMatrix[position][line[1]][0][0]),\
						negBitString.andStrings(self.positionMatrix[position][line[1]][1][0]))
				else:

					self.pruned[position] += 1

	def __preprocess(self, dataset):
		"""Creates bit vectors used by recsearch()"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()

		self.totalPositions = 0
		for line in self.sequences:
			if len(line) > 0: self.totalPositions += len(line) - self.motiflength + 1
		
		
		self.positiveSet = []
		self.negativeSet = []
		self.motifProfile = []
		posList = [0] * 4
		for i in range(self.motiflength): self.motifProfile.append(posList[:])

		
		
		#Fill positiveSet and negativeSet
		fa = self.sequences
		
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				if (i, j) in self.positions: 
					self.positiveSet.append(segment[j:j+self.motiflength])
					for l, k in enumerate(range(j, j+self.motiflength)):
						if segment[k]  == "a": self.motifProfile[l][0] += 1
						elif segment[k]== "c": self.motifProfile[l][1] += 1
						elif segment[k]== "g": self.motifProfile[l][2] += 1
						elif segment[k]== "t": self.motifProfile[l][3] += 1
				else: self.negativeSet.append(segment[j:j+self.motiflength])
		
		
		self.posStringLength = len(self.positiveSet)
		self.negStringLength = len(self.negativeSet)
		
		
		searchPattern = ""
		self.lettergroup = []
		#lists that contain positions with one, two, three or four letterpossibilities, respectively
		#used to find a better order in which to search the tree
		onechar, twochar, threechar, fourchar = [], [], [], []
		
		for i in range(len(self.motifProfile)):
			tempPattern = ""
			for j, k in enumerate(self.motifProfile[i]):
				if k != 0:
					if j==0:tempPattern+="a"
					elif j==1:tempPattern+="c"
					elif j==2:tempPattern+="g"
					elif j==3:tempPattern+="t"
			if len(tempPattern) == 1: onechar.append(i)
			elif len(tempPattern) == 2: twochar.append(i)
			elif len(tempPattern) == 3: threechar.append(i)
			else: fourchar.append(i)
			
			self.lettergroup.append(tempPattern)
			if len(tempPattern)>1: tempPattern = "["+tempPattern+"]"
			
			searchPattern += tempPattern
		#set the order in which to traverse the tree
		self.positionOrder = onechar + twochar + threechar + fourchar
		if VERBOSE:
			print "simplest pattern representing all positive instances:"
			print "\t", searchPattern
			print "position order:"
			print "\t", self.positionOrder
			print
		
		hits = Set(SearchUtils.reghit(self.sequences, searchPattern, self.motiflength)) - Set(self.excluded)
		#indexSet = Set(self.dataset.getIndexList())		
		indexSet = Set(self.positions) - Set(self.excluded)
		TP = len(hits&indexSet)
		FP = len(hits-indexSet)
		FN = len(indexSet-hits)
		TN = self.totalPositions - TP - FP - FN - len(self.excluded)
		
		if self.testbest < self.scoringFunction.fitness(TP, FN, FP, TN):
			self.testbest = self.scoringFunction.fitness(TP, FN, FP, TN) - 0.04
			
		
		#Generate a matrix of letters
		col = []
		self.__genInd2(0, 0, self.letterGroup, 1, [""], col)

		col = dict(col)
		
		letterMatrix = []
		for i in range(self.motiflength):
			letterMatrix.append(copy.copy(col))
		del col
		#Fill the letterMatrix with bitvectors describing hits and misses for each letter of each position of 
		#each entry in the positive and negative sets
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				bitString = BC.bitstring(self.bsv, self.posStringLength)

				counter = 0
				for k in range(len(self.positiveSet)):
					if self.positiveSet[k][i] == j:
						counter += 1
						bitString.setBit(k)

				letterMatrix[i][j] = [[bitString, counter], 0]
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				bitString = BC.bitstring(self.bsv, self.negStringLength)
				counter = 0
				for k in range(len(self.negativeSet)-1, -1, -1):
					if self.negativeSet[k][i] == j:
						counter += 1
						bitString.setBit(k)

				letterMatrix[i][j][1] = [bitString, counter]

		
		#The inputsegments are no longer necessary
		#del self.positiveSet, self.negativeSet, counter

		#Expand the lettermatrix into a positionmatrix, using the bitvectors from the lettermatrix as
		#building blocks
		self.positionMatrix = self.genTable(self.letterGroup, letterMatrix)
		del letterMatrix
		self.luCheck = []
		
		for i in range(self.posStringLength+1):
			self.luCheck.append(self.scoringFunction.fitness(i, self.posStringLength-i, 0, self.negStringLength))
		
				
	def genTable(self, letters, letterMatrix):
		"""Used by preprocess()"""
		matrix = []
		
		for i in range(self.motiflength):

			column = dict(self.genLetterCombos(letters, 3, letterMatrix[i]))
			matrix.append(column)
		return matrix
		
	
	def genLetterCombos(self, letters, letterCount, letterColumn):
		"""Used by genTable()"""
		res = []
		for i in range(1, letterCount+1):
			self.__genInd(0, 0, letters, i, [""]*i, res, letterColumn)
		return res

	def __genInd(self, lvl, prev, letters, letterCount, letterCombo, res, letterColumn):
		"""Used by genLetterCombos()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res, letterColumn)
			else:
				posBitString = BC.bitstring(self.bsv, self.posStringLength)
				negBitString = BC.bitstring(self.bsv, self.negStringLength)
				posCounter = 0
				negCounter = 0
				for letter in letterCombo:
					posBitString = posBitString.orStrings(letterColumn[letter][0][0])
					posCounter += letterColumn[letter][0][1]
					negBitString = negBitString.orStrings(letterColumn[letter][1][0])
					negCounter += letterColumn[letter][1][1]
				
				if letterCount == 1:     
					res.append(("".join(letterCombo), [[posBitString, posCounter], [negBitString, negCounter]]))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), [[posBitString, posCounter], [negBitString, negCounter]]))
	
	
	def __genInd2(self, lvl, prev, letters, letterCount, letterCombo, res):
		"""Used by preprocess()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res)
			else:
				if letterCount == 1:     
					res.append(("".join(letterCombo), 0))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), 0))
	
class Regexp_branch(Model):
	"""Optimized IUPAC model"""
	def __init__(self):
		self.description = "Regexp model"
		self.id = IUPACID
		
		self.letterGroup = "acgt"
		
		self.letterIndexes = {LETA:[0], LETC:[1], LETG:[2], LETT:[3], LETA|LETC:[0, 1, 4], LETA|LETG:[0, 2, 5],\
		LETA|LETT:[0, 3, 6], LETC|LETG:[1, 2, 7], LETC|LETT:[1, 3, 8], LETG|LETT:[2, 3, 9], LETA|LETC|LETG:[0, 1, 2, 4, 5, 7, 10],\
		LETA|LETC|LETT:[0, 1, 3, 4, 6, 8, 11], LETA|LETG|LETT:[0, 2, 3, 5, 6, 9, 12], LETC|LETG|LETT:[1, 2,3, 7, 8, 9, 13],\
		LETA|LETC|LETG|LETT:[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]}
		
		self.optimize1 = False #better starting score: finding basic score based on conserved positions
		self.optimize2 = True #reordering positions
		self.optimize3 = True #4 bitcounts instead of 15 in nodes
		self.optimize4 = True #omit non-needed groups in conserved positions
		self.optimize5 = False #lookuptables for bnb-scores (not much effect)
		
	
		
	
	def train(self, dataset=None, cl=None, motifLength=4, brsf = None, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		if brsf == None:
			self.testbest = self.scoringFunction.valueRange[0]
		else:
			self.testbest = brsf
		
		self.motiflength = motifLength
		self.summary = dict()
		self.motif = None
		self.trainingName = ""
		self.excluded = excluded

		
		self.bsv = BC.bitstringVariables()
		
		self.__preprocess(dataset)
		
		posBitString = BC.bitstring(self.bsv, self.posStringLength)
		for i in range(self.posStringLength):
			posBitString.setBit(i)
		
		negBitString = BC.bitstring(self.bsv, self.negStringLength)
		for i in range(self.negStringLength):
			negBitString.setBit(i)
		
		for i, line in enumerate(self.positionMatrix):
			for letter in line.keys():
				line["."] = [[posBitString, self.posStringLength], [negBitString, self.negStringLength]]
		
		print [posBitString]
		
		
		self.best = self.testbest#brsf

		self.pruned = [0]*self.motiflength
		self.motifList = []

		self.recSearch(["."]*self.motiflength, 0, None, posBitString, negBitString)
		print self.pruned
		
		del self.positionMatrix
		print "motiflist length:", len(self.motifList)
		
		return Patterns.Regexp(self.motifList[0], self.motiflength), self.result
		
		
	

	def recSearch(self, pattern, position, fil, posBitString, negBitString):
		"""The recursive search function, used by train"""
		
		recposition = position
		if position == 2:
			print pattern
			#print self.pruned
			0
		
		#If at bottom of tree (last letter in pattern)
		if position == self.motiflength-1:
			position = self.positionOrder[position]
			letterstring = 0
			
			
			
			_a = posBitString.andStrings(self.positionMatrix[position]["a"][0][0])
			TPa = _a.countBits()
			if TPa > 0: 
				letterstring |=LETA
			_c = posBitString.andStrings(self.positionMatrix[position]["c"][0][0])
			TPc = _c.countBits()
			if TPc > 0:
				letterstring |=LETC
			_g = posBitString.andStrings(self.positionMatrix[position]["g"][0][0])
			TPg = _g.countBits()
			if TPg > 0:
				letterstring |=LETG
			_t = posBitString.andStrings(self.positionMatrix[position]["t"][0][0])
			TPt = _t.countBits()
			if TPt > 0:
				letterstring |=LETT
			
			del _a, _c, _g, _t
			
			
			TPac,TPag,TPat,TPcg,TPct,TPgt,TPacg,TPact,TPagt,TPcgt,TPw = [0]*11
			if self.optimize3:
				TPac = TPa + TPc
				TPag = TPa + TPg
				TPat = TPa + TPt
				TPcg = TPc + TPg
				TPct = TPc + TPt
				TPgt = TPg + TPt
				TPacg = TPac + TPg
				TPact = TPac + TPt
				TPagt = TPag + TPt
				TPcgt = TPcg + TPt
				TPw = TPacg + TPt
			else:
				_ac = posBitString.andStrings(self.positionMatrix[position]["[ac]"][0][0])
				TPac = _ac.countBits()
				_ag = posBitString.andStrings(self.positionMatrix[position]["[ag]"][0][0])
				TPag = _ag.countBits()
				_at = posBitString.andStrings(self.positionMatrix[position]["[at]"][0][0])
				TPat = _at.countBits()
				_cg = posBitString.andStrings(self.positionMatrix[position]["[cg]"][0][0])
				TPcg = _cg.countBits()
				_ct = posBitString.andStrings(self.positionMatrix[position]["[ct]"][0][0])
				TPct = _ct.countBits()
				_gt = posBitString.andStrings(self.positionMatrix[position]["[gt]"][0][0])
				TPgt = _gt.countBits()
				_acg = posBitString.andStrings(self.positionMatrix[position]["[acg]"][0][0])
				TPacg = _acg.countBits()
				_act = posBitString.andStrings(self.positionMatrix[position]["[act]"][0][0])
				TPact = _act.countBits()
				_agt = posBitString.andStrings(self.positionMatrix[position]["[agt]"][0][0])
				TPagt = _agt.countBits()
				_cgt = posBitString.andStrings(self.positionMatrix[position]["[cgt]"][0][0])
				TPcgt = _cgt.countBits()
				_w = posBitString.andStrings(self.positionMatrix[position]["."][0][0])
				TPw = _w.countBits()	
			
			
			FPa, FPc, FPg, FPt, _a, _c, _t, _g = 0, 0, 0, 0, None, None, None, None
			
			
			tempList = []
			if self.optimize4 and self.optimize3: #omit letter groups, 4 calculations
				pruned = 0
				if letterstring & LETA:
					_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
					FPa = _a.countBits()
					del _a
					checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
					tempList.append([checka, "a", TPa, FPa])
				else: pruned += 1
				
				if letterstring & LETC:
					_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
					FPc = _c.countBits()
					del _c
					checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
					tempList.append([checkc, "c", TPc, FPc])
					if letterstring & LETA:
						FPac = FPa + FPc
						checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
						tempList.append([checkac, "[ac]", TPac, FPac])
					else: pruned += 1
				else: pruned += 2
				
				if letterstring & LETG:
					_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
					FPg = _g.countBits()
					del _g
					checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
					tempList.append([checkg, "g", TPg, FPg])
					if letterstring & LETA:
						FPag = FPa + FPg
						checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
						tempList.append([checkag, "[ag]", TPag, FPag])
						if letterstring & LETC:
							FPacg = FPag + FPc
							checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
							tempList.append([checkacg, "[acg]", TPacg, FPacg])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETC:
						FPcg = FPc + FPg
						checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
						tempList.append([checkcg, "[cg]", TPcg, FPcg])
					else: pruned += 1
				else: pruned += 4
				
				if letterstring & LETT:
					_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
					FPt = _t.countBits()
					del _t
					checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
					tempList.append([checkt, "t", TPt, FPt])
					if letterstring & LETA:
						FPat = FPa + FPt
						checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
						tempList.append([checkat, "[at]", TPat, FPat])
						if letterstring & LETC:
							FPact = FPac + FPt
							checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
							tempList.append([checkact, "[act]", TPact, FPact])
							if letterstring & LETG:
								FPw = FPacg + FPt
								checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
								tempList.append([checkw, ".", TPw, FPw])
							else: pruned += 1
						else: pruned += 2
						if letterstring & LETG:
							FPagt = FPag + FPt
							checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
							tempList.append([checkagt, "[agt]", TPagt, FPagt])
						else: pruned += 1
					else: pruned += 4
					if letterstring & LETC:
						FPct = FPc + FPt
						checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
						tempList.append([checkct, "[ct]", TPct, FPct])
						if letterstring & LETG:
							FPcgt = FPcg + FPt
							checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
							tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETG:
						FPgt = FPg + FPt
						checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
						tempList.append([checkgt, "[gt]", TPgt, FPgt])
					else:pruned += 1
				else: pruned += 8
						
						
				self.pruned[position] += pruned
			
			
			elif self.optimize3: # 4 calculations, do not omit letter groups
				_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
				FPa = _a.countBits()
				
				_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
				FPc = _c.countBits()
				
				_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
				FPg = _g.countBits()
				
				_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
				FPt = _t.countBits()
							
				del _a, _c, _g, _t
								
				FPac = FPa + FPc
				FPag = FPa + FPg
				FPat = FPa + FPt
				FPcg = FPc + FPg
				FPct = FPc + FPt
				FPgt = FPg + FPt
				FPacg = FPac + FPg
				FPact = FPac + FPt
				FPagt = FPag + FPt
				FPcgt = FPcg + FPt
				FPw = FPacg + FPt
				
				checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
				tempList.append([checka, "a", TPa, FPa])
				
				checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
				tempList.append([checkc, "c", TPc, FPc])
	
				checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
				tempList.append([checkac, "[ac]", TPac, FPac])
	
				checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
				tempList.append([checkg, "g", TPg, FPg])
	
				checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
				tempList.append([checkag, "[ag]", TPag, FPag])
	
				checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
				tempList.append([checkacg, "[acg]", TPacg, FPacg])
	
				checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
				tempList.append([checkcg, "[cg]", TPcg, FPcg])
				
				checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
				tempList.append([checkt, "t", TPt, FPt])
		
				checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
				tempList.append([checkat, "[at]", TPat, FPat])
				
				checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
				tempList.append([checkact, "[act]", TPact, FPact])
	
				checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
				tempList.append([checkw, ".", TPw, FPw])
	
				checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
				tempList.append([checkagt, "[agt]", TPagt, FPagt])
	
				checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
				tempList.append([checkct, "[ct]", TPct, FPct])
	
				checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
				tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
	
				checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
				tempList.append([checkgt, "[gt]", TPgt, FPgt])
			
			elif self.optimize4: #all calculations, but omit letter groups
				pruned = 0
				if letterstring & LETA:
					_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
					FPa = _a.countBits()
					del _a
					checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
					tempList.append([checka, "a", TPa, FPa])
				else: pruned += 1
				
				if letterstring & LETC:
					_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
					FPc = _c.countBits()
					del _c
					checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
					tempList.append([checkc, "c", TPc, FPc])
					if letterstring & LETA:
						_ac = negBitString.andStrings(self.positionMatrix[position]["[ac]"][1][0])
						FPac = _ac.countBits()
						del _ac
						checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
						tempList.append([checkac, "[ac]", TPac, FPac])
					else: pruned += 1
				else: pruned += 2
				
				if letterstring & LETG:
					_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
					FPg = _g.countBits()
					del _g
					checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
					tempList.append([checkg, "g", TPg, FPg])
					if letterstring & LETA:
						_ag = negBitString.andStrings(self.positionMatrix[position]["[ag]"][1][0])
						FPag = _ag.countBits()
						del _ag
						checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
						tempList.append([checkag, "[ag]", TPag, FPag])
						if letterstring & LETC:
							_acg = negBitString.andStrings(self.positionMatrix[position]["[acg]"][1][0])
							FPacg = _acg.countBits()
							del _acg
							checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
							tempList.append([checkacg, "[acg]", TPacg, FPacg])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETC:
						_cg = negBitString.andStrings(self.positionMatrix[position]["[cg]"][1][0])
						FPcg = _cg.countBits()
						del _cg
						checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
						tempList.append([checkcg, "[cg]", TPcg, FPcg])
					else: pruned += 1
				else: pruned += 4
				
				if letterstring & LETT:
					_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
					FPt = _t.countBits()
					del _t
					checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
					tempList.append([checkt, "t", TPt, FPt])
					if letterstring & LETA:
						_at = negBitString.andStrings(self.positionMatrix[position]["[at]"][1][0])
						FPat = _at.countBits()
						del _at
						checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
						tempList.append([checkat, "[at]", TPat, FPat])
						if letterstring & LETC:
							_act = negBitString.andStrings(self.positionMatrix[position]["[act]"][1][0])
							FPact = _act.countBits()
							del _act
							checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
							tempList.append([checkact, "[act]", TPact, FPact])
							if letterstring & LETG:
								_w = negBitString.andStrings(self.positionMatrix[position]["."][1][0])
								FPw = _w.countBits()
								del _w
								checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
								tempList.append([checkw, ".", TPw, FPw])
							else: pruned += 1
						else: pruned += 2
						if letterstring & LETG:
							_agt = negBitString.andStrings(self.positionMatrix[position]["[agt]"][1][0])
							FPagt = _agt.countBits()
							del _agt
							checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
							tempList.append([checkagt, "[agt]", TPagt, FPagt])
						else: pruned += 1
					else: pruned += 4
					if letterstring & LETC:
						_ct = negBitString.andStrings(self.positionMatrix[position]["[ct]"][1][0])
						FPct = _ct.countBits()
						del _ct
						checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
						tempList.append([checkct, "[ct]", TPct, FPct])
						if letterstring & LETG:
							_cgt = negBitString.andStrings(self.positionMatrix[position]["[cgt]"][1][0])
							FPcgt = _cgt.countBits()
							del _cgt
							checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
							tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETG:
						_gt = negBitString.andStrings(self.positionMatrix[position]["[gt]"][1][0])
						FPgt = _gt.countBits()
						del _gt
						checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
						tempList.append([checkgt, "[gt]", TPgt, FPgt])
					else:pruned += 1
				else: pruned += 8
						
						
				self.pruned[position] += pruned
			
			else:#do not omit letter groups, do not reduce calculations to 4
				_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
				FPa = _a.countBits()
				_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
				FPc = _c.countBits()
				_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
				FPg = _g.countBits()
				_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
				FPt = _t.countBits()
				_ac = negBitString.andStrings(self.positionMatrix[position]["[ac]"][1][0])
				FPac = _ac.countBits()
				_ag = negBitString.andStrings(self.positionMatrix[position]["[ag]"][1][0])
				FPag = _ag.countBits()
				_at = negBitString.andStrings(self.positionMatrix[position]["[at]"][1][0])
				FPat = _at.countBits()
				_cg = negBitString.andStrings(self.positionMatrix[position]["[cg]"][1][0])
				FPcg = _cg.countBits()
				_ct = negBitString.andStrings(self.positionMatrix[position]["[ct]"][1][0])
				FPct = _ct.countBits()
				_gt = negBitString.andStrings(self.positionMatrix[position]["[gt]"][1][0])
				FPgt = _gt.countBits()
				_acg = negBitString.andStrings(self.positionMatrix[position]["[acg]"][1][0])
				FPacg = _acg.countBits()
				_act = negBitString.andStrings(self.positionMatrix[position]["[act]"][1][0])
				FPact = _act.countBits()
				_agt = negBitString.andStrings(self.positionMatrix[position]["[agt]"][1][0])
				FPagt = _agt.countBits()
				_cgt = negBitString.andStrings(self.positionMatrix[position]["[cgt]"][1][0])
				FPcgt = _cgt.countBits()
				_w = negBitString.andStrings(self.positionMatrix[position]["."][1][0])
				FPw = _w.countBits()
				
				del _a, _c, _g, _t, _ac, _ag, _at, _cg, _ct, _gt, _acg, _act, _agt, _cgt, _w
				
				checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
				tempList.append([checka, "a", TPa, FPa])
				
				checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
				tempList.append([checkc, "c", TPc, FPc])
	
				checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
				tempList.append([checkac, "[ac]", TPac, FPac])
	
				checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
				tempList.append([checkg, "g", TPg, FPg])
	
				checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
				tempList.append([checkag, "[ag]", TPag, FPag])
	
				checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
				tempList.append([checkacg, "[acg]", TPacg, FPacg])
	
				checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
				tempList.append([checkcg, "[cg]", TPcg, FPcg])
				
				checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
				tempList.append([checkt, "t", TPt, FPt])
		
				checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
				tempList.append([checkat, "[at]", TPat, FPat])
				
				checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
				tempList.append([checkact, "[act]", TPact, FPact])
	
				checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
				tempList.append([checkw, ".", TPw, FPw])
	
				checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
				tempList.append([checkagt, "[agt]", TPagt, FPagt])
	
				checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
				tempList.append([checkct, "[ct]", TPct, FPct])
	
				checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
				tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
	
				checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
				tempList.append([checkgt, "[gt]", TPgt, FPgt])
				
			for line in tempList:
				if line[0] > self.best:
					pattern[position] = line[1]
					self.motifList = ["".join(pattern)]
					self.best = line[0]
					self.result = line[0], line[2], self.posStringLength-line[2], line[3], self.negStringLength-line[3]
				
				
				#Uncomment this if all best found motifs should be included
				#Pattern found equally good
				elif line[0] == self.best:
					pattern[position] = line[1]
					self.motifList.append("".join(pattern))
		
		#If not last letter position:
		else:
			position = self.positionOrder[position]
			letterString = 0
			TPa, TPc, TPg, TPt = 0, 0, 0, 0
			
			_a = posBitString.andStrings(self.positionMatrix[position]["a"][0][0])
			TPa = _a.countBits()
			if TPa > 0:
				letterString |=LETA
			_c = posBitString.andStrings(self.positionMatrix[position]["c"][0][0])
			TPc = _c.countBits()
			if TPc>0:
				letterString |=LETC
			_g = posBitString.andStrings(self.positionMatrix[position]["g"][0][0])
			TPg = _g.countBits()
			if TPg>0:
				letterString |=LETG
			_t = posBitString.andStrings(self.positionMatrix[position]["t"][0][0])
			TPt = _t.countBits()
			if TPt>0:
				letterString |=LETT
			del _a, _c, _g, _t
			
			
			
			TPac,TPag,TPat,TPcg,TPct,TPgt,TPacg,TPact,TPagt,TPcgt,TPw = [0]*11
			
			if self.optimize3:
				TPac = TPa + TPc
				TPag = TPa + TPg
				TPat = TPa + TPt
				TPcg = TPc + TPg
				TPct = TPc + TPt
				TPgt = TPg + TPt
				TPacg = TPac + TPg
				TPact = TPac + TPt
				TPagt = TPag + TPt
				TPcgt = TPcg + TPt
				TPw = TPacg + TPt
			else:
				_ac = posBitString.andStrings(self.positionMatrix[position]["[ac]"][0][0])
				TPac = _ac.countBits()
				_ag = posBitString.andStrings(self.positionMatrix[position]["[ag]"][0][0])
				TPag = _ag.countBits()
				_at = posBitString.andStrings(self.positionMatrix[position]["[at]"][0][0])
				TPat = _at.countBits()
				_cg = posBitString.andStrings(self.positionMatrix[position]["[cg]"][0][0])
				TPcg = _cg.countBits()
				_ct = posBitString.andStrings(self.positionMatrix[position]["[ct]"][0][0])
				TPct = _ct.countBits()
				_gt = posBitString.andStrings(self.positionMatrix[position]["[gt]"][0][0])
				TPgt = _gt.countBits()
				_acg = posBitString.andStrings(self.positionMatrix[position]["[acg]"][0][0])
				TPacg = _acg.countBits()
				_act = posBitString.andStrings(self.positionMatrix[position]["[act]"][0][0])
				TPact = _act.countBits()
				_agt = posBitString.andStrings(self.positionMatrix[position]["[agt]"][0][0])
				TPagt = _agt.countBits()
				_cgt = posBitString.andStrings(self.positionMatrix[position]["[cgt]"][0][0])
				TPcgt = _cgt.countBits()
				_w = posBitString.andStrings(self.positionMatrix[position]["."][0][0])
				TPw = _w.countBits()
			
			tList = [[TPa, "a"], [TPc, "c"],\
			[ TPg, "g"], [TPt, "t"],\
			[TPac, "[ac]"], [TPag, "[ag]"],\
			[TPat, "[at]"],	[TPcg, "[cg]"],\
			[TPct, "[ct]"], [TPgt, "[gt]"],\
			[TPacg, "[acg]"], [TPact, "[act]"],\
			[TPagt, "[agt]"], [TPcgt, "[cgt]"],\
			[TPw, "."]]
			
			tempList = []
			
			if self.optimize4:
				for ind in self.letterIndexes[letterString]:
					tempList+=[tList[ind]]
			else:
				tempList = tList
			
			for line in tempList:
				check = 0
				if self.optimize5: check = self.luCheck[line[0]]
				else: check = self.scoringFunction.fitness(line[0], self.posStringLength-line[0], 0, self.negStringLength)
				if check > self.best:
					pattern[position] = line[1]
					self.recSearch(pattern, recposition+1, fil, posBitString.andStrings(self.positionMatrix[position][line[1]][0][0]),\
						negBitString.andStrings(self.positionMatrix[position][line[1]][1][0]))
				else:

					self.pruned[position] += 1
			
		
	
	
	
	def __preprocess(self, dataset):
		"""Creates bit vectors used by recsearch()"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()

		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1
		self.positiveSet = []
		self.negativeSet = []
		self.motifProfile = []
		posList = [0] * 4
		for i in range(self.motiflength): self.motifProfile.append(posList[:])

		fa = self.sequences
		
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				if (i, j) in self.positions: 
					self.positiveSet.append(segment[j:j+self.motiflength])
					for l, k in enumerate(range(j, j+self.motiflength)):
						if segment[k]  == "a": self.motifProfile[l][0] += 1
						elif segment[k]== "c": self.motifProfile[l][1] += 1
						elif segment[k]== "g": self.motifProfile[l][2] += 1
						elif segment[k]== "t": self.motifProfile[l][3] += 1
				else: self.negativeSet.append(segment[j:j+self.motiflength])
		
		
		self.posStringLength = len(self.positiveSet)
		self.negStringLength = len(self.negativeSet)
		
		
		searchPattern = ""
		self.lettergroup = []
		#lists that contain positions with one, two, three or four letterpossibilities, respectively
		#used to find a better order in which to search the tree
		onechar, twochar, threechar, fourchar = [], [], [], []
		
		for i in range(len(self.motifProfile)):
			tempPattern = ""
			for j, k in enumerate(self.motifProfile[i]):
				if k != 0:
					if j==0:tempPattern+="a"
					elif j==1:tempPattern+="c"
					elif j==2:tempPattern+="g"
					elif j==3:tempPattern+="t"
			if len(tempPattern) == 1: onechar.append(i)
			elif len(tempPattern) == 2: twochar.append(i)
			elif len(tempPattern) == 3: threechar.append(i)
			else: fourchar.append(i)
			
			self.lettergroup.append(tempPattern)
			if len(tempPattern)>1: tempPattern = "["+tempPattern+"]"
			
			searchPattern += tempPattern
		
		self.conservedPositionsOpt = []
		for l in self.lettergroup:
			tempg = 0
			if "a" in l: tempg |= LETA
			if "c" in l: tempg |= LETC
			if "g" in l: tempg |= LETG
			if "t" in l: tempg |= LETT
			self.conservedPositionsOpt.append(tempg)
		
		
		
		print searchPattern
		if self.optimize2: self.positionOrder = onechar + twochar + threechar + fourchar
		else: self.positionOrder = range(self.motiflength)
		print "positionorder", self.positionOrder
		
		print self.positionOrder
		
		hits = Set(SearchUtils.reghit(self.sequences, searchPattern, self.motiflength)) - Set(self.excluded)
		
		indexSet = Set(self.positions) - Set(self.excluded)
		TP = len(hits&indexSet)
		FP = len(hits-indexSet)
		FN = len(indexSet-hits)
		TN = self.totalPositions - TP - FP - FN - len(self.excluded)
		
		print self.scoringFunction.fitness(TP, FN, FP, TN)
		if self.optimize1 and self.testbest < self.scoringFunction.fitness(TP, FN, FP, TN):
			self.testbest = self.scoringFunction.fitness(TP, FN, FP, TN) - 0.04
		
		#Generate a matrix of letters
		col = []
		self.__genInd2(0, 0, self.letterGroup, 1, [""], col)

		col = dict(col)
		
		letterMatrix = []
		for i in range(self.motiflength):
			letterMatrix.append(copy.copy(col))
		del col
		#Fill the letterMatrix with bitvectors describing hits and misses for each letter of each position of 
		#each entry in the positive and negative sets
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				
				bitString = BC.bitstring(self.bsv, self.posStringLength)

				counter = 0
				for k in range(len(self.positiveSet)):
					if self.positiveSet[k][i] == j:
						counter += 1
				
						bitString.setBit(k)

				letterMatrix[i][j] = [[bitString, counter], 0]
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				
				bitString = BC.bitstring(self.bsv, self.negStringLength)
				counter = 0
				for k in range(len(self.negativeSet)-1, -1, -1):
					if self.negativeSet[k][i] == j:
						counter += 1
					
						bitString.setBit(k)

				letterMatrix[i][j][1] = [bitString, counter]

		
		#The inputsegments are no longer necessary
		#del self.positiveSet, self.negativeSet, counter

		#Expand the lettermatrix into a positionmatrix, using the bitvectors from the lettermatrix as
		#building blocks
		
		self.positionMatrix = self.genTable(self.letterGroup, letterMatrix)
		del letterMatrix
		self.luCheck = []
		for i in range(self.posStringLength+1):
			self.luCheck.append(self.scoringFunction.fitness(i, self.posStringLength-i, 0, self.negStringLength))
	def genTable(self, letters, letterMatrix):
		"""Used by preprocess()"""
		matrix = []
		
		for i in range(self.motiflength):

			column = dict(self.genLetterCombos(letters, 3, letterMatrix[i]))
			matrix.append(column)
		return matrix
		
	
	def genLetterCombos(self, letters, letterCount, letterColumn):
		"""Used by genTable()"""
		res = []
		for i in range(1, letterCount+1):
			self.__genInd(0, 0, letters, i, [""]*i, res, letterColumn)
		return res

	def __genInd(self, lvl, prev, letters, letterCount, letterCombo, res, letterColumn):
		"""Used by genLetterCombos()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res, letterColumn)
			else:
				posBitString = BC.bitstring(self.bsv, self.posStringLength)
				negBitString = BC.bitstring(self.bsv, self.negStringLength)
				posCounter = 0
				negCounter = 0
				for letter in letterCombo:
					posBitString = posBitString.orStrings(letterColumn[letter][0][0])
					posCounter += letterColumn[letter][0][1]
					negBitString = negBitString.orStrings(letterColumn[letter][1][0])
					negCounter += letterColumn[letter][1][1]
				
				if letterCount == 1:     
					res.append(("".join(letterCombo), [[posBitString, posCounter], [negBitString, negCounter]]))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), [[posBitString, posCounter], [negBitString, negCounter]]))
	
	
	def __genInd2(self, lvl, prev, letters, letterCount, letterCombo, res):
		"""Used by preprocess()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res)
			else:
				if letterCount == 1:     
					res.append(("".join(letterCombo), 0))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), 0))
	

class Regexp_basic(Model):
	"""Optimized IUPAC model"""
	def __init__(self):
		self.description = "Regexp model"
		self.id = IUPACID
		
		self.letterGroup = "acgt"
		
		self.letterIndexes = {LETA:[0], LETC:[1], LETG:[2], LETT:[3], LETA|LETC:[0, 1, 4], LETA|LETG:[0, 2, 5],\
		LETA|LETT:[0, 3, 6], LETC|LETG:[1, 2, 7], LETC|LETT:[1, 3, 8], LETG|LETT:[2, 3, 9], LETA|LETC|LETG:[0, 1, 2, 4, 5, 7, 10],\
		LETA|LETC|LETT:[0, 1, 3, 4, 6, 8, 11], LETA|LETG|LETT:[0, 2, 3, 5, 6, 9, 12], LETC|LETG|LETT:[1, 2,3, 7, 8, 9, 13],\
		LETA|LETC|LETG|LETT:[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]}
		
		self.optimize1 = True #Conserved letter positions
		self.optimize2 = False #Change position order
		self.optimize3 = True #Only 4 bit counts per set in the last level, only count where TP not 0
	
	def train(self, dataset=None, cl=None, motifLength=4, brsf = None, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		if brsf == None:
			self.testbest = self.scoringFunction.valueRange[0]
		else:
			self.testbest = brsf
		
		self.motiflength = motifLength
		self.summary = dict()
		self.motif = None
		self.trainingName = ""
		self.excluded = excluded

		
		
		self.bsv = BC.bitstringVariables()
		
		self.__preprocess(dataset)
		
		posBitString = BC.bitstring(self.bsv, self.posStringLength)
		for i in range(self.posStringLength):
			posBitString.setBit(i)

		negBitString = BC.bitstring(self.bsv, self.negStringLength)
		for i in range(self.negStringLength):
			negBitString.setBit(i)
		
		
		for i, line in enumerate(self.positionMatrix):
			for letter in line.keys():
				line["."] = [[posBitString, self.posStringLength], [negBitString, self.negStringLength]]
		
		self.best = self.testbest#brsf

		self.pruned = [0]*self.motiflength
		self.motifList = []
		self.t = time.clock()
		self.recSearch(["."]*self.motiflength, 0, None, posBitString, negBitString)
		print self.pruned

		del self.positionMatrix
		print "motiflist length:", len(self.motifList)
		
		return Patterns.Regexp(self.motifList[0], self.motiflength), self.result
		
		
	

	def recSearch(self, pattern, position, fil, posBitString, negBitString):
		"""The recursive search function, used by train"""
		#If at bottom of tree (last letter in pattern)
		
		recposition = position
		if position == 2:
			print self.t-time.clock()
			self.t = time.clock()
			
			print pattern
			print self.pruned
		if position == self.motiflength-1:
			position = self.positionOrder[position]
			letterstring = 0

			_a = posBitString.andStrings(self.positionMatrix[position]["a"][0][0])
			TPa = _a.countBits()
			if TPa > 0: 
				letterstring |=LETA
			_c = posBitString.andStrings(self.positionMatrix[position]["c"][0][0])
			TPc = _c.countBits()
			if TPc > 0:
				letterstring |=LETC
			_g = posBitString.andStrings(self.positionMatrix[position]["g"][0][0])
			TPg = _g.countBits()
			if TPg > 0:
				letterstring |=LETG
			_t = posBitString.andStrings(self.positionMatrix[position]["t"][0][0])
			TPt = _t.countBits()
			if TPt > 0:
				letterstring |=LETT
			
			del _a, _c, _g, _t
			
			
			TPac,TPag,TPat,TPcg,TPct,TPgt,TPacg,TPact,TPagt,TPcgt,TPw = [0]*11
			if self.optimize3:
				TPac = TPa + TPc
				TPag = TPa + TPg
				TPat = TPa + TPt
				TPcg = TPc + TPg
				TPct = TPc + TPt
				TPgt = TPg + TPt
				TPacg = TPac + TPg
				TPact = TPac + TPt
				TPagt = TPag + TPt
				TPcgt = TPcg + TPt
				TPw = TPacg + TPt
			else:
				_ac = posBitString.andStrings(self.positionMatrix[position]["[ac]"][0][0])
				TPac = _ac.countBits()
				_ag = posBitString.andStrings(self.positionMatrix[position]["[ag]"][0][0])
				TPag = _ag.countBits()
				_at = posBitString.andStrings(self.positionMatrix[position]["[at]"][0][0])
				TPat = _at.countBits()
				_cg = posBitString.andStrings(self.positionMatrix[position]["[cg]"][0][0])
				TPcg = _cg.countBits()
				_ct = posBitString.andStrings(self.positionMatrix[position]["[ct]"][0][0])
				TPct = _ct.countBits()
				_gt = posBitString.andStrings(self.positionMatrix[position]["[gt]"][0][0])
				TPgt = _gt.countBits()
				_acg = posBitString.andStrings(self.positionMatrix[position]["[acg]"][0][0])
				TPacg = _acg.countBits()
				_act = posBitString.andStrings(self.positionMatrix[position]["[act]"][0][0])
				TPact = _act.countBits()
				_agt = posBitString.andStrings(self.positionMatrix[position]["[agt]"][0][0])
				TPagt = _agt.countBits()
				_cgt = posBitString.andStrings(self.positionMatrix[position]["[cgt]"][0][0])
				TPcgt = _cgt.countBits()
				_w = posBitString.andStrings(self.positionMatrix[position]["."][0][0])
				TPw = _w.countBits()	
			
			
			FPa, FPc, FPg, FPt, _a, _c, _t, _g = 0, 0, 0, 0, None, None, None, None
			tempList = []
			if self.optimize1 and self.optimize3: #omit letter groups, 4 calculations
				pruned = 0
				if letterstring & LETA:
					_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
					FPa = _a.countBits()
					del _a
					checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
					tempList.append([checka, "a", TPa, FPa])
				else: pruned += 1
				
				if letterstring & LETC:
					_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
					FPc = _c.countBits()
					del _c
					checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
					tempList.append([checkc, "c", TPc, FPc])
					if letterstring & LETA:
						FPac = FPa + FPc
						checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
						tempList.append([checkac, "[ac]", TPac, FPac])
					else: pruned += 1
				else: pruned += 2
				
				if letterstring & LETG:
					_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
					FPg = _g.countBits()
					del _g
					checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
					tempList.append([checkg, "g", TPg, FPg])
					if letterstring & LETA:
						FPag = FPa + FPg
						checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
						tempList.append([checkag, "[ag]", TPag, FPag])
						if letterstring & LETC:
							FPacg = FPag + FPc
							checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
							tempList.append([checkacg, "[acg]", TPacg, FPacg])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETC:
						FPcg = FPc + FPg
						checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
						tempList.append([checkcg, "[cg]", TPcg, FPcg])
					else: pruned += 1
				else: pruned += 4
				
				if letterstring & LETT:
					_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
					FPt = _t.countBits()
					del _t
					checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
					tempList.append([checkt, "t", TPt, FPt])
					if letterstring & LETA:
						FPat = FPa + FPt
						checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
						tempList.append([checkat, "[at]", TPat, FPat])
						if letterstring & LETC:
							FPact = FPac + FPt
							checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
							tempList.append([checkact, "[act]", TPact, FPact])
							if letterstring & LETG:
								FPw = FPacg + FPt
								checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
								tempList.append([checkw, ".", TPw, FPw])
							else: pruned += 1
						else: pruned += 2
						if letterstring & LETG:
							FPagt = FPag + FPt
							checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
							tempList.append([checkagt, "[agt]", TPagt, FPagt])
						else: pruned += 1
					else: pruned += 4
					if letterstring & LETC:
						FPct = FPc + FPt
						checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
						tempList.append([checkct, "[ct]", TPct, FPct])
						if letterstring & LETG:
							FPcgt = FPcg + FPt
							checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
							tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETG:
						FPgt = FPg + FPt
						checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
						tempList.append([checkgt, "[gt]", TPgt, FPgt])
					else:pruned += 1
				else: pruned += 8
						
						
				self.pruned[position] += pruned
			
			
			elif self.optimize3: # 4 calculations, do not omit letter groups
				_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
				FPa = _a.countBits()
				
				_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
				FPc = _c.countBits()
				
				_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
				FPg = _g.countBits()
				
				_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
				FPt = _t.countBits()
							
				del _a, _c, _g, _t
								
				FPac = FPa + FPc
				FPag = FPa + FPg
				FPat = FPa + FPt
				FPcg = FPc + FPg
				FPct = FPc + FPt
				FPgt = FPg + FPt
				FPacg = FPac + FPg
				FPact = FPac + FPt
				FPagt = FPag + FPt
				FPcgt = FPcg + FPt
				FPw = FPacg + FPt
				
				checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
				tempList.append([checka, "a", TPa, FPa])
				
				checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
				tempList.append([checkc, "c", TPc, FPc])
	
				checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
				tempList.append([checkac, "[ac]", TPac, FPac])
	
				checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
				tempList.append([checkg, "g", TPg, FPg])
	
				checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
				tempList.append([checkag, "[ag]", TPag, FPag])
	
				checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
				tempList.append([checkacg, "[acg]", TPacg, FPacg])
	
				checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
				tempList.append([checkcg, "[cg]", TPcg, FPcg])
				
				checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
				tempList.append([checkt, "t", TPt, FPt])
		
				checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
				tempList.append([checkat, "[at]", TPat, FPat])
				
				checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
				tempList.append([checkact, "[act]", TPact, FPact])
	
				checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
				tempList.append([checkw, ".", TPw, FPw])
	
				checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
				tempList.append([checkagt, "[agt]", TPagt, FPagt])
	
				checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
				tempList.append([checkct, "[ct]", TPct, FPct])
	
				checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
				tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
	
				checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
				tempList.append([checkgt, "[gt]", TPgt, FPgt])
			
			elif self.optimize1: #all calculations, but omit letter groups
				pruned = 0
				if letterstring & LETA:
					_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
					FPa = _a.countBits()
					del _a
					checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
					tempList.append([checka, "a", TPa, FPa])
				else: pruned += 1
				
				if letterstring & LETC:
					_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
					FPc = _c.countBits()
					del _c
					checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
					tempList.append([checkc, "c", TPc, FPc])
					if letterstring & LETA:
						_ac = negBitString.andStrings(self.positionMatrix[position]["[ac]"][1][0])
						FPac = _ac.countBits()
						del _ac
						checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
						tempList.append([checkac, "[ac]", TPac, FPac])
					else: pruned += 1
				else: pruned += 2
				
				if letterstring & LETG:
					_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
					FPg = _g.countBits()
					del _g
					checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
					tempList.append([checkg, "g", TPg, FPg])
					if letterstring & LETA:
						_ag = negBitString.andStrings(self.positionMatrix[position]["[ag]"][1][0])
						FPag = _ag.countBits()
						del _ag
						checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
						tempList.append([checkag, "[ag]", TPag, FPag])
						if letterstring & LETC:
							_acg = negBitString.andStrings(self.positionMatrix[position]["[acg]"][1][0])
							FPacg = _acg.countBits()
							del _acg
							checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
							tempList.append([checkacg, "[acg]", TPacg, FPacg])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETC:
						_cg = negBitString.andStrings(self.positionMatrix[position]["[cg]"][1][0])
						FPcg = _cg.countBits()
						del _cg
						checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
						tempList.append([checkcg, "[cg]", TPcg, FPcg])
					else: pruned += 1
				else: pruned += 4
				
				if letterstring & LETT:
					_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
					FPt = _t.countBits()
					del _t
					checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
					tempList.append([checkt, "t", TPt, FPt])
					if letterstring & LETA:
						_at = negBitString.andStrings(self.positionMatrix[position]["[at]"][1][0])
						FPat = _at.countBits()
						del _at
						checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
						tempList.append([checkat, "[at]", TPat, FPat])
						if letterstring & LETC:
							_act = negBitString.andStrings(self.positionMatrix[position]["[act]"][1][0])
							FPact = _act.countBits()
							del _act
							checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
							tempList.append([checkact, "[act]", TPact, FPact])
							if letterstring & LETG:
								_w = negBitString.andStrings(self.positionMatrix[position]["."][1][0])
								FPw = _w.countBits()
								del _w
								checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
								tempList.append([checkw, ".", TPw, FPw])
							else: pruned += 1
						else: pruned += 2
						if letterstring & LETG:
							_agt = negBitString.andStrings(self.positionMatrix[position]["[agt]"][1][0])
							FPagt = _agt.countBits()
							del _agt
							checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
							tempList.append([checkagt, "[agt]", TPagt, FPagt])
						else: pruned += 1
					else: pruned += 4
					if letterstring & LETC:
						_ct = negBitString.andStrings(self.positionMatrix[position]["[ct]"][1][0])
						FPct = _ct.countBits()
						del _ct
						checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
						tempList.append([checkct, "[ct]", TPct, FPct])
						if letterstring & LETG:
							_cgt = negBitString.andStrings(self.positionMatrix[position]["[cgt]"][1][0])
							FPcgt = _cgt.countBits()
							del _cgt
							checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
							tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
						else: pruned += 1
					else: pruned += 2
					if letterstring & LETG:
						_gt = negBitString.andStrings(self.positionMatrix[position]["[gt]"][1][0])
						FPgt = _gt.countBits()
						del _gt
						checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
						tempList.append([checkgt, "[gt]", TPgt, FPgt])
					else:pruned += 1
				else: pruned += 8
						
						
				self.pruned[position] += pruned
			
			else:#do not omit letter groups, do not reduce calculations to 4
				_a = negBitString.andStrings(self.positionMatrix[position]["a"][1][0])
				FPa = _a.countBits()
				_c = negBitString.andStrings(self.positionMatrix[position]["c"][1][0])
				FPc = _c.countBits()
				_g = negBitString.andStrings(self.positionMatrix[position]["g"][1][0])
				FPg = _g.countBits()
				_t = negBitString.andStrings(self.positionMatrix[position]["t"][1][0])
				FPt = _t.countBits()
				_ac = negBitString.andStrings(self.positionMatrix[position]["[ac]"][1][0])
				FPac = _ac.countBits()
				_ag = negBitString.andStrings(self.positionMatrix[position]["[ag]"][1][0])
				FPag = _ag.countBits()
				_at = negBitString.andStrings(self.positionMatrix[position]["[at]"][1][0])
				FPat = _at.countBits()
				_cg = negBitString.andStrings(self.positionMatrix[position]["[cg]"][1][0])
				FPcg = _cg.countBits()
				_ct = negBitString.andStrings(self.positionMatrix[position]["[ct]"][1][0])
				FPct = _ct.countBits()
				_gt = negBitString.andStrings(self.positionMatrix[position]["[gt]"][1][0])
				FPgt = _gt.countBits()
				_acg = negBitString.andStrings(self.positionMatrix[position]["[acg]"][1][0])
				FPacg = _acg.countBits()
				_act = negBitString.andStrings(self.positionMatrix[position]["[act]"][1][0])
				FPact = _act.countBits()
				_agt = negBitString.andStrings(self.positionMatrix[position]["[agt]"][1][0])
				FPagt = _agt.countBits()
				_cgt = negBitString.andStrings(self.positionMatrix[position]["[cgt]"][1][0])
				FPcgt = _cgt.countBits()
				_w = negBitString.andStrings(self.positionMatrix[position]["."][1][0])
				FPw = _w.countBits()
				
				del _a, _c, _g, _t, _ac, _ag, _at, _cg, _ct, _gt, _acg, _act, _agt, _cgt, _w
				
				checka = self.scoringFunction.fitness(TPa, self.posStringLength-TPa, FPa, self.negStringLength-FPa)
				tempList.append([checka, "a", TPa, FPa])
				
				checkc = self.scoringFunction.fitness(TPc, self.posStringLength-TPc, FPc, self.negStringLength-FPc)
				tempList.append([checkc, "c", TPc, FPc])
	
				checkac = self.scoringFunction.fitness(TPac, self.posStringLength-TPac, FPac, self.negStringLength-FPac)
				tempList.append([checkac, "[ac]", TPac, FPac])
	
				checkg = self.scoringFunction.fitness(TPg, self.posStringLength-TPg, FPg, self.negStringLength-FPg)
				tempList.append([checkg, "g", TPg, FPg])
	
				checkag = self.scoringFunction.fitness(TPag, self.posStringLength-TPag, FPag, self.negStringLength-FPag)
				tempList.append([checkag, "[ag]", TPag, FPag])
	
				checkacg = self.scoringFunction.fitness(TPacg, self.posStringLength-TPacg, FPacg, self.negStringLength-FPacg)
				tempList.append([checkacg, "[acg]", TPacg, FPacg])
	
				checkcg = self.scoringFunction.fitness(TPcg, self.posStringLength-TPcg, FPcg, self.negStringLength-FPcg)
				tempList.append([checkcg, "[cg]", TPcg, FPcg])
				
				checkt = self.scoringFunction.fitness(TPt, self.posStringLength-TPt, FPt, self.negStringLength-FPt)
				tempList.append([checkt, "t", TPt, FPt])
		
				checkat = self.scoringFunction.fitness(TPat, self.posStringLength-TPat, FPat, self.negStringLength-FPat)
				tempList.append([checkat, "[at]", TPat, FPat])
				
				checkact = self.scoringFunction.fitness(TPact, self.posStringLength-TPact, FPact, self.negStringLength-FPact)
				tempList.append([checkact, "[act]", TPact, FPact])
	
				checkw = self.scoringFunction.fitness(TPw, self.posStringLength-TPw, FPw, self.negStringLength-FPw)
				tempList.append([checkw, ".", TPw, FPw])
	
				checkagt = self.scoringFunction.fitness(TPagt, self.posStringLength-TPagt, FPagt, self.negStringLength-FPagt)
				tempList.append([checkagt, "[agt]", TPagt, FPagt])
	
				checkct = self.scoringFunction.fitness(TPct, self.posStringLength-TPct, FPct, self.negStringLength-FPct)
				tempList.append([checkct, "[ct]", TPct, FPct])
	
				checkcgt = self.scoringFunction.fitness(TPcgt, self.posStringLength-TPcgt, FPcgt, self.negStringLength-FPcgt)
				tempList.append([checkcgt, "[cgt]", TPcgt, FPcgt])
	
				checkgt = self.scoringFunction.fitness(TPgt, self.posStringLength-TPgt, FPgt, self.negStringLength-FPgt)
				tempList.append([checkgt, "[gt]", TPgt, FPgt])
				
			for line in tempList:
				if line[0] > self.best:
					pattern[position] = line[1]
					self.motifList = ["".join(pattern)]
					self.best = line[0]
					self.result = line[0], line[2], self.posStringLength-line[2], line[3], self.negStringLength-line[3]
				
				
				#Uncomment this if all best found motifs should be included
				#Pattern found equally jolly good
				elif line[0] == self.best:
					pattern[position] = line[1]
					self.motifList.append("".join(pattern))
				
		#If not last letter position:
		else:
			position = self.positionOrder[position]
			
			tList = [["a"], ["c"],\
			[ "g"], ["t"],\
			["[ac]"], [ "[ag]"],\
			[ "[at]"],	[ "[cg]"],\
			["[ct]"], ["[gt]"],\
			[ "[acg]"], [ "[act]"],\
			[ "[agt]"], ["[cgt]"],\
			["."]]
			
			tempList = []
			
			if self.optimize1:			
				for i in self.letterIndexes[self.conservedPositionsOpt[position]]:
					tempList.append(tList[i])
			else: 
				tempList = tList
			
			for [line] in tempList:
				
				pattern[position] = line
				self.recSearch(pattern, recposition+1, fil, posBitString.andStrings(self.positionMatrix[position][line][0][0]),\
					negBitString.andStrings(self.positionMatrix[position][line][1][0]))
		
	
	
	
	def __preprocess(self, dataset):
		"""Creates bit vectors used by recsearch()"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		self.sequences = dataset.getSequences()
		self.positions = dataset.getMotifIndexes()

		
		
		
		
		
		self.totalPositions = 0
		for line in self.sequences:
			self.totalPositions += len(line) - self.motiflength + 1
		
		
		
		self.positiveSet = []
		self.negativeSet = []
		self.motifProfile = []
		posList = [0] * 4
		for i in range(self.motiflength): self.motifProfile.append(posList[:])

		
		
		#Fill positiveSet and negativeSet
		fa = self.sequences
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				if (i, j) in self.positions: 
					self.positiveSet.append(segment[j:j+self.motiflength])
					for l, k in enumerate(range(j, j+self.motiflength)):
						if segment[k]  == "a": self.motifProfile[l][0] += 1
						elif segment[k]== "c": self.motifProfile[l][1] += 1
						elif segment[k]== "g": self.motifProfile[l][2] += 1
						elif segment[k]== "t": self.motifProfile[l][3] += 1
				else: self.negativeSet.append(segment[j:j+self.motiflength])
		
		
		self.posStringLength = len(self.positiveSet)
		self.negStringLength = len(self.negativeSet)
		
		
		searchPattern = ""
		searchPatternList = []
		self.lettergroup = []
		#lists that contain positions with one, two, three or four letterpossibilities, respectively
		#used to find a better order in which to search the tree
		onechar, twochar, threechar, fourchar = [], [], [], []
		
		for i in range(len(self.motifProfile)):
			tempPattern = ""
			for j, k in enumerate(self.motifProfile[i]):
				if k != 0:
					if j==0:tempPattern+="a"
					elif j==1:tempPattern+="c"
					elif j==2:tempPattern+="g"
					elif j==3:tempPattern+="t"
			if len(tempPattern) == 1: onechar.append(i)
			elif len(tempPattern) == 2: twochar.append(i)
			elif len(tempPattern) == 3: threechar.append(i)
			else: fourchar.append(i)
			
			self.lettergroup.append(tempPattern)
			if len(tempPattern)>1: tempPattern = "["+tempPattern+"]"
			
			searchPattern += tempPattern
			searchPatternList.append(tempPattern)
		#set the order in which to traverse the tree
		
		self.conservedPositionsOpt = []
		for l in self.lettergroup:
			tempg = 0
			if "a" in l: tempg |= LETA
			if "c" in l: tempg |= LETC
			if "g" in l: tempg |= LETG
			if "t" in l: tempg |= LETT
			self.conservedPositionsOpt.append(tempg)

		if self.optimize2:self.positionOrder = onechar + twochar + threechar + fourchar
		else: self.positionOrder = range(self.motiflength)
		
		
		
		#Generate a matrix of letters
		col = []
		self.__genInd2(0, 0, self.letterGroup, 1, [""], col)

		col = dict(col)
		
		letterMatrix = []
		for i in range(self.motiflength):
			letterMatrix.append(copy.copy(col))
		del col
		#Fill the letterMatrix with bitvectors describing hits and misses for each letter of each position of 
		#each entry in the positive and negative sets
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				bitString = BC.bitstring(self.bsv, self.posStringLength)

				counter = 0
				for k in range(len(self.positiveSet)):
					if self.positiveSet[k][i] == j:
						counter += 1
						bitString.setBit(k)

				letterMatrix[i][j] = [[bitString, counter], 0]
		
		for i in range(self.motiflength):
			for j in self.letterGroup:
				bitString = BC.bitstring(self.bsv, self.negStringLength)
				counter = 0
				for k in range(len(self.negativeSet)-1, -1, -1):
					if self.negativeSet[k][i] == j:
						counter += 1
						bitString.setBit(k)

				letterMatrix[i][j][1] = [bitString, counter]

		
		#The inputsegments are no longer necessary
		#del self.positiveSet, self.negativeSet, counter

		#Expand the lettermatrix into a positionmatrix, using the bitvectors from the lettermatrix as
		#building blocks
		self.positionMatrix = self.genTable(self.letterGroup, letterMatrix)
		del letterMatrix
		self.luCheck = []
		
		for i in range(self.posStringLength+1):
			self.luCheck.append(self.scoringFunction.fitness(i, self.posStringLength-i, 0, self.negStringLength))
		
	def genTable(self, letters, letterMatrix):
		"""Used by preprocess()"""
		matrix = []
		
		for i in range(self.motiflength):

			column = dict(self.genLetterCombos(letters, 3, letterMatrix[i]))
			matrix.append(column)
		return matrix
		
	
	def genLetterCombos(self, letters, letterCount, letterColumn):
		"""Used by genTable()"""
		res = []
		for i in range(1, letterCount+1):
			self.__genInd(0, 0, letters, i, [""]*i, res, letterColumn)
		return res

	def __genInd(self, lvl, prev, letters, letterCount, letterCombo, res, letterColumn):
		"""Used by genLetterCombos()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res, letterColumn)
			else:
				posBitString = BC.bitstring(self.bsv, self.posStringLength)
				negBitString = BC.bitstring(self.bsv, self.negStringLength)
				posCounter = 0
				negCounter = 0
				for letter in letterCombo:
					posBitString = posBitString.orStrings(letterColumn[letter][0][0])
					posCounter += letterColumn[letter][0][1]
					negBitString = negBitString.orStrings(letterColumn[letter][1][0])
					negCounter += letterColumn[letter][1][1]
				
				if letterCount == 1:     
					res.append(("".join(letterCombo), [[posBitString, posCounter], [negBitString, negCounter]]))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), [[posBitString, posCounter], [negBitString, negCounter]]))
	
	
	def __genInd2(self, lvl, prev, letters, letterCount, letterCombo, res):
		"""Used by preprocess()"""
		for i in range(prev, (len(letters)-(letterCount-lvl-1))):
			letterCombo[lvl] = letters[i]
			if lvl < (letterCount-1):
				self.__genInd(lvl+1, i+1, letters, letterCount, letterCombo, res)
			else:
				if letterCount == 1:     
					res.append(("".join(letterCombo), 0))
				else:
					res.append(("".join(["["]+letterCombo+["]"]), 0))
	
	


class Regexp_combos2(Model):
	"""Optimized IUPAC model"""
	def __init__(self):
		self.description = "Regexp model"
		self.id = IUPACID
		self.alphabet = ["a", "c", "g", "t", "[ac]", "[ag]", "[at]", "[cg]", "[ct]", "[gt]", "[acg]", "[act]", "[agt]", "[cgt]", "[acgt]"]
		self.bases = ["a", "c", "g", "t"]
		
	
	def combinations2(self, level, length, pattern, current):
		
		if level == length:
			tTP, tFP = self.qgram.get("".join(current), (0, 0))
			self.TP += tTP
			self.FP += tFP
			
		else:
			for i in pattern[level]:
				if i in ("[", "]"): continue
				current[level] = i
				self.combinations2(level+1, length, pattern, current)
	
	
	def combinations(self, level, length, current):
		
		if level == length:
			
			self.TP, self.FP = 0, 0
			t=time.clock()
			if t - self.t > 5:
				print current
				self.t = t
			self.combinations2(0, length, current, [0]*self.motiflength)
			check = self.scoringFunction.fitness(self.TP, self.positivePos-self.TP, self.FP, self.negativePos-self.FP)
			if check> self.best:
				self.best = check
				self.pattern = "".join(current)
				
				self.result = check, self.TP, self.positivePos-self.TP, self.FP, self.negativePos-self.FP
				
		else:
			for i in self.alphabet:
				current[level] = i
				self.combinations(level+1, length, current)
	
	def combinations3(self, level, length, current):
		
		if level == length:
			pattern = "".join(current)
			t=time.clock()
			if t - self.t > 5:
				print pattern
				self.t = t
			
			
			hits = Set(SearchUtils.reghit(self.dataset.getSequences(), pattern, length)) - Set(self.excluded)
			indexSet = Set(self.positions) - Set(self.excluded)
			TP = len(hits&indexSet)
			FP = len(hits-indexSet)
			self.qgram[pattern] = [TP, FP]
			
		else:
			for i in self.bases:
				current[level] = i
				self.combinations3(level+1, length, current)
	
	def train(self, dataset=None, cl=None, motifLength=4, brsf = None, excluded = []):
		
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		if brsf == None:
			self.testbest = self.scoringFunction.valueRange[0]
		else:
			self.testbest = brsf
		self.excluded = excluded
		self.positions = self.dataset.getMotifIndexes()
		
		self.motiflength = motifLength
		self.best = self.scoringFunction.valueRange[0]
		self.totalPositions = 0
		for line in self.dataset.getSequences():
			self.totalPositions += len(line) - self.motiflength + 1
		self.positivePos = len(self.positions)
		self.negativePos = self.totalPositions - self.positivePos
		
		
		self.qgram = {}
				
		self.t = time.clock()
		self.combinations3(0, self.motiflength, [0]*self.motiflength)
		
		self.best = self.scoringFunction.valueRange[0]
		
		
		self.t = time.clock()
		self.combinations(0, self.motiflength, [0]*self.motiflength)
		return Patterns.Regexp(self.pattern.replace("[acgt]", "."), self.motiflength), self.result

class Regexp_combos(Model):
	"""Optimized IUPAC model"""
	def __init__(self):
		self.description = "Regexp model"
		self.id = IUPACID
		self.alphabet = ["a", "c", "g", "t", "[ac]", "[ag]", "[at]", "[cg]", "[ct]", "[gt]", "[acg]", "[act]", "[agt]", "[cgt]", "."]
		

	def combinations(self, level, length, current):
		
		if level == length:
			pattern =  "".join(current)
			#print pattern
			hits = Set(SearchUtils.reghit(self.dataset.getSequences(), pattern, length)) - Set(self.excluded)
			indexSet = Set(self.positions) - Set(self.excluded)
			TP = len(hits&indexSet)
			FP = len(hits-indexSet)
			FN = len(indexSet-hits)
			TN = self.totalPositions - TP - FP - FN - len(self.excluded)
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			if check > self.best:
				self.best = check
				self.result = check, TP, FN, FP, TN
				self.pattern = pattern
		else:
			for i in self.alphabet:
				current[level] = i
				self.combinations(level+1, length, current)
	
	def train(self, dataset=None, cl=None, motifLength=4, brsf = None, excluded = []):
		
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		if brsf == None:
			self.testbest = self.scoringFunction.valueRange[0]
		else:
			self.testbest = brsf
		self.excluded = excluded
		self.positions = self.dataset.getMotifIndexes()
		self.motiflength = motifLength
		self.best = self.scoringFunction.valueRange[0]
		self.totalPositions = 0
		for line in self.dataset.getSequences():
			self.totalPositions += len(line) - self.motiflength + 1
		print self.totalPositions
		
		
		self.combinations(0, self.motiflength, [0]*self.motiflength)
		return Patterns.Regexp(self.pattern, self.motiflength), self.result
		
class PWM_BM_new(Model):
	"""PWM model"""
	def __init__(self):
		self.description = "PWM_BM model"
		self.id = PBMID
		self.maxorder = 3

	def __preprocess(self):
		0
		
		
		
	def train(self, dataset=None, cl=None, motifLength = 4, pseudoweigth = 0.0001, order=4, excluded=[]):
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None:
			print "error: train(): no dataset given"
			return None
		if cl != None: self.setScoringFunction(cl)
		
		self.excluded = excluded
		self.motiflength = motifLength
		self.returnbest = self.scoringFunction.valueRange[0]
		self.returnmodel = None
		self.order = order
		
		self.__preprocess()
		
		return self._train(pseudoweigth)
		
		
		
		
	def _train(self, pseudoweigth):
	
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		
		self.pseudow = pseudoweigth
		
		
		self.summary = dict()
		self.motif = []
		self.background = {"a":0, "c":0, "g":0, "t":0}
		self.trainingName = ""
		
		self.best = self.scoringFunction.valueRange[0]
		self.newbackground = {"":{"a":0, "c":0, "g":0, "t":0}, -1:{"a":1, "c":1, "g":1, "t":1}}

		for i in range(self.motiflength):
			#old
			self.motif.append({"a":0, "c":0, "g":0, "t":0})
			#endold
			#self.newmotif.append({"":{"a":0, "c":0, "g":0, "t":0}, -1:{"a":1, "c":1, "g":1, "t":1}})

		#Finding the character counts of all motifs and for the background
		fa = self.dataset.getSequences()
		for i, j in self.dataset.getMotifIndexes():
			if (i, j) in self.excluded: continue
			motifinstance = fa[i][j:j+self.motiflength]
			for k in range(self.motiflength):
				c = motifinstance[k]
				#old
				self.motif[k][c] += 1
				#endold
				#self.newmotif[k][""][c] += 1
				
				
				
				
				#subinstance = motifinstance[:k+1][-self.maxorder-1:-1]
				#print subinstance, k
				#for l in range(len(subinstance)):
					#preceding = subinstance[l:]
					#if not self.newmotif[k].has_key(preceding):self.newmotif[k][preceding] = {"a":0, "c":0, "g":0, "t":0}
					#self.newmotif[k][preceding][c] += 1
		
		
		print self.motif
						
		tmpex = []
		for i, j in self.excluded:
			for k in range(j, j+self.motiflength):
				tmpex.append((i, k))
		
		
		for i in range(len(fa)):
			segment = fa[i]
			j = 0
			while j < len(segment):
				if (i, j) in tmpex:
					j += 1
					continue
				if (i, j) in self.dataset.getMotifIndexes():
					
					j+=self.motiflength-1
				else:
					
					preceding = ""
					if j < 3:
						preceding = segment[0:j]
					else:
						preceding = segment[j-3:j]
					
					
					while True:
						
						if not self.newbackground.has_key(preceding): self.newbackground[preceding] = {"a":0, "c":0, "g":0, "t":0}
						self.newbackground[preceding][segment[j]] += 1
						if preceding == "": break
						preceding = preceding[1:]
					
					self.background[segment[j]] += 1
				j += 1
		
		
		
		#OLDBG
		#Normalizing the background counts to get probabilities
		#bcount = float(sum(self.background.values()))
		#for i in self.background.keys():
			
		#	self.background[i] = float(self.background[i]) / bcount
		#	if self.background[i] == 0: self.background[i] = 0.0000000000001
		#ENDOLDBG
		
		
		#NEWBG
		#self.newbackground[-1] = self.newbackground[""]
		for i in self.newbackground.keys():
			bcount = float(sum(self.newbackground[i].values()))
			for j in self.newbackground[i].keys():
				self.newbackground[i][j] = float(self.newbackground[i][j])/bcount
				if self.newbackground[i][j] == 0: self.newbackground[i][j] = 0.0000000000001 
		
		#ENDNEWBG

		
		
		
		#Creating the log-odds score using the motif letter counts
		for i in range(len(self.motif)):
			
			#old
			bcount = float(sum(self.motif[i].values()))
			
			for j in self.motif[i].keys():
				
				
				self.motif[i][j] = log( ( (float(self.motif[i][j]) + (self.newbackground[""][j] * self.pseudow) )/ (bcount + self.pseudow) ) / self.newbackground[""][j] )
		
			#endold
			#for j in self.newmotif[i].keys():
			#	bcount = float(sum(self.newmotif[i][j].values()))
				
			#	for k in self.newmotif[i][j].keys():
					#oldbg (only line that must be changed
					#self.newmotif[i][j][k] = log( ( (float(self.newmotif[i][j][k]) + (self.background[k] * self.pseudow) )/ (bcount + self.pseudow) ) / self.background[k] )
					#newbg
			#		self.newmotif[i][j][k] = log( ( (float(self.newmotif[i][j][k]) + (self.newbackground[j][k] * self.pseudow) )/ (bcount + self.pseudow) ) / self.newbackground[j][k] )
		
		
		#Count the score of each substring, and add the total score to the corresponding list
		#for maks in range(-1, 4):
		for maks in [self.order]:
			
			self.negativeList = []
			self.positiveList = []
			for i in range(len(fa)):
				segment = fa[i]
				for j in range(len(segment)-self.motiflength+1):
					if (i, j) in self.excluded: continue
					#old
					
					value = 0
					substring = segment[j:j+self.motiflength]
					for k, c in enumerate(substring):
						
						value += self.motif[k][c]
					
					#endold
					
					
					
					#print substring
					for k in range(self.motiflength):
						preceding = ""
						if maks > 0:
							preceding = substring[:k][-maks:]
							while not self.newbackground.has_key(preceding):
								preceding = preceding[1:]
						elif maks == -1:
							preceding = -1
						
						
						value -= self.newbackground[preceding][substring[k]]
					
					
					
					if (i, j) in self.dataset.getMotifIndexes():
						self.positiveList.append(value)
					else:
						self.negativeList.append(value)
			
			self.negativeList.sort()
			self.negativeList.reverse()
			self.positiveList.sort()
			
			
			
			self.avg = 0
			#self.avg = sum(self.positiveList) / len(self.positiveList)
			#print self.avg
			
			
			
	
			
			
			#Find the threshold value yielding the best pattern score
	
			
			for i, threshold in enumerate(self.positiveList):
	
	
				FN = i
				TP = len(self.positiveList) - i
	
				
				FP = 0
				for j in range(len(self.negativeList)):
	
					if threshold > self.negativeList[j]:
						FP = j
						break
					elif j == len(self.negativeList)-1:
						FP = len(self.negativeList)
				
				TN = len(self.negativeList) - FP
				
	
				
				check = self.scoringFunction.fitness(TP, FN, FP, TN)
				
				if self.best < check:
					self.best = check
					self.maks = maks
					self.threshold = threshold
					self.result = check, TP, FN, FP, TN
					self.bestpseudow = self.pseudow
				
			print self.best
			
		return Patterns.PWM_BM_new(self.motif, self.threshold, self.avg, self.maks, self.bestpseudow, self.newbackground), self.result

	

class PWM_new(Model):
	"""PWM model"""
	def __init__(self):
		self.description = "PWM model"
		self.id = PWMID
		self.maxorder = 3


		
		
		
	def __preprocess(self):
		0
		
		
		
	def train(self, dataset=None, cl=None, motifLength = 4, pseudoweigth = 1, order=3, excluded=[]):
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None:
			print "error: train(): no dataset given"
			return None
		if cl != None: self.setScoringFunction(cl)
		
		self.excluded = excluded
		self.motiflength = motifLength
		self.returnbest = self.scoringFunction.valueRange[0]
		self.returnmodel = None
		self.order = order
		
		self.__preprocess()
		#newer
		return self._train(pseudoweigth)
		
		
		
		
	def _train(self, pseudoweigth):
	
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		
		self.pseudow = pseudoweigth
		
		
		self.summary = dict()
		self.motif = []
		self.background = {"a":0, "c":0, "g":0, "t":0}
		self.trainingName = ""
		
		self.best = self.scoringFunction.valueRange[0]
		self.newmotif = []
		self.newbackground = {"":{"a":0, "c":0, "g":0, "t":0}}
		
		
		
		
		
		for i in range(self.motiflength):
			#old
			self.motif.append({"a":0, "c":0, "g":0, "t":0})
			#endold
			self.newmotif.append({"":{"a":0, "c":0, "g":0, "t":0}, -1:{"a":1, "c":1, "g":1, "t":1}})

		#Finding the character counts of all motifs and for the background
		fa = self.dataset.getSequences()
		for i, j in self.dataset.getMotifIndexes():
			if (i, j) in self.excluded: continue
			motifinstance = fa[i][j:j+self.motiflength]
			for k in range(self.motiflength):
				c = motifinstance[k]
				#old
				self.motif[k][c] += 1
				#endold
				self.newmotif[k][""][c] += 1
				
				
				
				
				subinstance = motifinstance[:k+1][-self.maxorder-1:-1]
				#print subinstance, k
				for l in range(len(subinstance)):
					preceding = subinstance[l:]
					if not self.newmotif[k].has_key(preceding):self.newmotif[k][preceding] = {"a":0, "c":0, "g":0, "t":0}
					self.newmotif[k][preceding][c] += 1
		
		
		
						
					

		
		tmpex = []
		for i, j in self.excluded:
			for k in range(j, j+self.motiflength):
				tmpex.append((i, k))
		
		
		for i in range(len(fa)):
			segment = fa[i]
			j = 0
			while j < len(segment):
				if (i, j) in tmpex:
					j += 1
					continue
				if (i, j) in self.dataset.getMotifIndexes():
					
					j+=self.motiflength-1
				else:
					
					preceding = ""
					if j < 3:
						preceding = segment[0:j]
					else:
						preceding = segment[j-3:j]
					
					
					while True:
						
						if not self.newbackground.has_key(preceding): self.newbackground[preceding] = {"a":0, "c":0, "g":0, "t":0}
						self.newbackground[preceding][segment[j]] += 1
						if preceding == "": break
						preceding = preceding[1:]
					
					self.background[segment[j]] += 1
				j += 1

		
		
		#OLDBG
		#Normalizing the background counts to get probabilities
		bcount = float(sum(self.background.values()))
		for i in self.background.keys():
			
			self.background[i] = float(self.background[i]) / bcount
			if self.background[i] == 0: self.background[i] = 0.0000000000001
		#ENDOLDBG
		
		
		#NEWBG
		self.newbackground[-1] = self.newbackground[""]
		for i in self.newbackground.keys():
			bcount = float(sum(self.newbackground[i].values()))
			for j in self.newbackground[i].keys():
				self.newbackground[i][j] = float(self.newbackground[i][j])/bcount
				if self.newbackground[i][j] == 0: self.newbackground[i][j] = 0.0000000000001 
		
		#ENDNEWBG
		
		
		
		
		#Creating the log-odds score using the motif letter counts
		for i in range(len(self.motif)):
			#old
			#bcount = float(sum(self.motif[i].values()))
			#for j in self.motif[i].keys():
			#	self.motif[i][j] = log( ( (float(self.motif[i][j]) + (self.background[j] * self.pseudow) )/ (bcount + self.pseudow) ) / self.background[j] )
			#endold
			for j in self.newmotif[i].keys():
				bcount = float(sum(self.newmotif[i][j].values()))
				
				for k in self.newmotif[i][j].keys():
					#oldbg (only line that must be changed
					#self.newmotif[i][j][k] = log( ( (float(self.newmotif[i][j][k]) + (self.background[k] * self.pseudow) )/ (bcount + self.pseudow) ) / self.background[k] )
					#newbg
					self.newmotif[i][j][k] = log( ( (float(self.newmotif[i][j][k]) + (self.newbackground[j][k] * self.pseudow) )/ (bcount + self.pseudow) ) / self.newbackground[j][k] )
	
		
		#Count the score of each substring, and add the total score to the corresponding list
		#for maks in range(-1, 4):
		for maks in [self.order]:
			
			self.negativeList = []
			self.positiveList = []
			for i in range(len(fa)):
				segment = fa[i]
				for j in range(len(segment)-self.motiflength+1):
					if (i, j) in self.excluded: continue
					#old
					"""
					value = 0
					for k, c in enumerate(segment[j:j+self.motiflength]):
						value += self.motif[k][c]
					"""
					#endold
					
					
					value = 0
					substring = segment[j:j+self.motiflength]
					#print substring
					for k in range(self.motiflength):
						preceding = ""
						if maks > 0:
							preceding = substring[:k][-maks:]
							while not self.newmotif[k].has_key(preceding):
								preceding = preceding[1:]
						elif maks == -1:
							preceding = -1
						
						
						value += self.newmotif[k][preceding][substring[k]]
					
					
					
					if (i, j) in self.dataset.getMotifIndexes():
						self.positiveList.append(value)
					else:
						self.negativeList.append(value)
			
			self.negativeList.sort()
			self.negativeList.reverse()
			self.positiveList.sort()
			
			self.avg = 0
			#self.avg = sum(self.positiveList) / len(self.positiveList)
			#print self.avg
			
			
			
	
			
			
			#Find the threshold value yielding the best pattern score
	
			
			for i, threshold in enumerate(self.positiveList):
	
	
				FN = i
				TP = len(self.positiveList) - i
	
				
				FP = 0
				for j in range(len(self.negativeList)):
	
					if threshold > self.negativeList[j]:
						FP = j
						break
					elif j == len(self.negativeList)-1:
						FP = len(self.negativeList)
				
				TN = len(self.negativeList) - FP
				
	
				
				check = self.scoringFunction.fitness(TP, FN, FP, TN)
				
				if self.best < check:
					self.best = check
					self.maks = maks
					self.threshold = threshold
					self.result = check, TP, FN, FP, TN
					self.bestpseudow = self.pseudow
				
			print self.best
			
		return Patterns.PWM_new(self.newmotif, self.threshold, self.avg, self.maks, self.bestpseudow), self.result
	
	
	




"""
................................................................................
Mo_PWM_basic, probabilistic PWM model
................................................................................
"""

class PWM(Model):
	"""PWM model"""
	def __init__(self):
		self.description = "PWM model"
		self.id = PWMID
		self.pseudow = 1


		
		
		
		
		
	
	
	
	def train(self, dataset=None, cl=None, motifLength = 4, pseudoweigth = 0.0001, excluded=[]):
	
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		self.excluded = excluded
		self.pseudow = pseudoweigth
		self.motiflength = motifLength
		
		self.summary = dict()
		self.motif = []
		self.background = {"a":0, "c":0, "g":0, "t":0}
		self.trainingName = ""
		self.negativeList = []
		self.positiveList = []
		self.best = self.scoringFunction.valueRange[0]
		
		
		
		
		
		
		
		for i in range(self.motiflength):
			self.motif.append({"a":0, "c":0, "g":0, "t":0})
		#self.background = [0] * 4

		#Finding the character counts of all motifs and for the background
		fa = dataset.getSequences()
		for i, j in dataset.getMotifIndexes():
			if (i, j) in self.excluded: continue
			for k, c in enumerate(fa[i][j:j+self.motiflength]):
				self.motif[k][c] += 1
		
		
		tmpex = []
		for i, j in self.excluded:
			for k in range(j, j+self.motiflength):
				tmpex.append((i, k))
		
		for i in range(len(fa)):
			segment = fa[i]
			j = 0
			while j < len(segment):
				if (i, j) in tmpex:
					j += 1
					continue
				if (i, j) in dataset.getMotifIndexes():
					
					j+=self.motiflength-1
				else:
					
					self.background[segment[j]] += 1
				j += 1
			
		print "rall"
		
		#Normalizing the background counts to get probabilities

		bcount = float(sum(self.background.values()))
		for i in self.background.keys():
			
			self.background[i] = float(self.background[i]) / bcount
			if self.background[i] == 0: self.background[i] = 0.0000000000001
			
		
		
		#Creating the log-odds score using the motif letter counts
		for i in range(len(self.motif)):

			bcount = float(sum(self.motif[i].values()))
			for j in self.motif[i].keys():
				if self.background[j] == 0: print "hyl"
				self.motif[i][j] = log( ( (float(self.motif[i][j]) + (self.background[j] * self.pseudow) )/ (bcount + self.pseudow) ) / self.background[j] )
				
		
		
		#Count the score of each substring, and add the total score to the corresponding list
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				value = 0
				for k, c in enumerate(segment[j:j+self.motiflength]):
					"""
					if c == "a": value += self.motif[k][0]
					elif c == "c": value += self.motif[k][1]
					elif c == "g": value += self.motif[k][2]
					elif c == "t": value += self.motif[k][3]
					"""
					value += self.motif[k][c]
					
				if (i, j) in dataset.getMotifIndexes():
					self.positiveList.append(value)
				else:
					self.negativeList.append(value)
		
		self.negativeList.sort()
		self.negativeList.reverse()
		self.positiveList.sort()
		
		self.avg = 0
		#self.avg = sum(self.positiveList) / len(self.positiveList)
		#print self.avg
		
		
		

		
		
		#Find the threshold value yielding the best pattern score

		
		for i, threshold in enumerate(self.positiveList):


			FN = i
			TP = len(self.positiveList) - i

			
			FP = 0
			for j in range(len(self.negativeList)):

				if threshold > self.negativeList[j]:
					FP = j
					break
				elif j == len(self.negativeList)-1:
					FP = len(self.negativeList)
			
			TN = len(self.negativeList) - FP
			

			
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			
			if self.best < check:
				self.best = check
				self.threshold = threshold
				self.result = check, TP, FN, FP, TN
				
		#self.trainingName = self.dataset.fileName
		self.motif = [self.motif, self.threshold]
		return Patterns.PWM(self.motif[0], self.motif[1], self.avg), self.result
	
	
	
	
			

"""
................................................................................
PWM_BM - PWM model with a higher-order background model
................................................................................
"""

class PWM_BM(Model):
	"""PWM model"""
	def __init__(self):
		self.description = "PWM_BM model"
		self.id = PBMID
		self.pseudow = 1


		
		
		
		
		
	
	
	
	def train(self, dataset=None, cl=None, motifLength = 4, backgroundOrder = 3, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		self.excluded = excluded
		print self.excluded
		self.motiflength = motifLength
		
		self.summary = dict()
		self.motif = []
		self.background = dict()
		self.trainingName = ""
		self.negativeList = []
		self.positiveList = []
		self.best = self.scoringFunction.valueRange[0]
		self.backgroundOrder = backgroundOrder
		if self.backgroundOrder > 3: self.backgroundOrder = 3
		elif self.backgroundOrder < 0: self.backgroundOrder = 0
		

		
		
		
		for i in range(self.motiflength):
			self.motif.append({"a":0, "c":0, "g":0, "t":0})
		#self.background = [0] * 4

		
		fa = dataset.getSequences()
		
		excludedIndexes = []
		for i in range(len(fa)): 
			
			excludedIndexes.append(set(range(self.backgroundOrder)))
		#Lettercounts in PWM
		for i, j in dataset.getMotifIndexes():
			if (i, j) in self.excluded: continue
			for k, c in enumerate(fa[i][j:j+self.motiflength]):
				self.motif[k][c] += 1
			
			#(Nothing to do with lettercounts, but also need to iterate motif indexes)
			for k in range(j, j+self.motiflength+self.backgroundOrder):
				excludedIndexes[i].add(k)

			
			
		#Lettercounts in background model
		self.background["*"] = {"a":0, "c":0, "g":0, "t":0}
		for i in range(len(fa)):
			segment = fa[i]
			for j in set(range(len(segment))) - excludedIndexes[i]:
				if (i, j) in self.excluded: continue
				if not self.background.has_key(segment[j-self.backgroundOrder:j]):
					self.background[segment[j-self.backgroundOrder:j]] = {"a":0, "c":0, "g":0, "t":0}
				self.background[segment[j-self.backgroundOrder:j]][segment[j]] += 1
				self.background["*"][segment[j]] += 1

		print 
		print self.background
		print
		#Normalizing the background counts to get probabilities, and calculate the log-value
		for line in self.background.values():
			bcount = float(sum(line.values()))
			for i in line.keys():
				try:
					line[i] = log(float(line[i]) / bcount)
				except OverflowError:
					line[i] = -(10**7)
		
		print 
		print self.background
		print
		#Creating the patterns scores using the motif letter counts

		for i in range(len(self.motif)):

			bcount = float(sum(self.motif[i].values()))
			for j in self.motif[i].keys():
				
				try:
					self.motif[i][j] = log(self.motif[i][j]/bcount )
				except OverflowError:
					self.motif[i][j] = -(10**7)

		
		
		
		#Count the score of each substring, and add the total score to the corresponding list
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				value = 0
				for k, c in enumerate(segment[j:j+self.motiflength]):
					value += self.motif[k][c]
					if self.background.has_key(segment[j-self.backgroundOrder:j]):
						value -= self.background[segment[j-self.backgroundOrder:j]][c]
					else:
						value -= self.background["*"][c]
				
				if (i, j) in dataset.getMotifIndexes():
					self.positiveList.append(value)
				else:
					self.negativeList.append(value)
		
		


		
		#Find the threshold value yielding the best pattern score
		self.negativeList.sort()
		self.negativeList.reverse()
		self.positiveList.sort()
		#for line in self.positiveList:

		for i, threshold in enumerate(self.positiveList):


			FN = i
			TP = len(self.positiveList) - i

			
			FP = 0
			for j in range(len(self.negativeList)):

				if threshold > self.negativeList[j]:
					FP = j
					break
				elif j == len(self.negativeList)-1:
					FP = len(self.negativeList)
			
			TN = len(self.negativeList) - FP
			

			
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			
			if self.best < check:
				self.best = check
				self.threshold = threshold
				self.result = check, TP, FN, FP, TN
				
		#self.trainingName = self.dataset.fileName
		#self.motif = [self.motif, self.backgroundOrder, self.background, self.threshold]
		return Patterns.PWM_BM(self.motif, self.backgroundOrder, self.background, self.threshold), self.result
	
	



	
"""
................................................................................
Mo_optimal, artificial, "optimal" model
................................................................................
"""
	
class Optimal(Model):
	"""Artificial, "Optimal" model"""
	def __init__(self):
		self.description = "Optimal model"
		self.id = "op"

	
	def train(self, dataset=None, cl=None, motifLength=7, excluded = []):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		self.motiflength = motifLength
		
		#Dictionary containing the TP/FP-count of all motif instances:    {motif:[TP, FP]}
		self.positiveSet = dict()
		
		fa = dataset.getSequences()
		for i, j in dataset.getMotifIndexes():
			if self.positiveSet.has_key(fa[i][j:j+self.motiflength]):
				self.positiveSet[fa[i][j:j+self.motiflength]][0]+=1
			else:
				self.positiveSet[fa[i][j:j+self.motiflength]] = [1, 0]
		
		
		#Total number of negative possibilities in the set
		negCount = 0

		#The maximum number of FP
		maxFP = 0
		
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				
				if (i, j) not in dataset.getMotifIndexes():
					negCount += 1
					if self.positiveSet.has_key(segment[j:j+self.motiflength]):
						self.positiveSet[segment[j:j+self.motiflength]][1] += 1
						maxFP += 1
		
		
		check = self.scoringFunction.valueRange[0]
		

		for i in range(len(dataset.getMotifIndexes())+1-1, -1, -1):

			for j in range(maxFP+1-1, -1, -1):
				
				
				#for all variations of TP, FN, FP, TN in the available area
				tTP, tFN, tFP, tTN = i, len(dataset.getMotifIndexes())-i, j, negCount-j
				#calculate fitness
				a = self.scoringFunction.fitness(tTP, tFN, tFP, tTN)
				if a > check:
					
					cTPFP = [0, 0]
					motifs = []
					for motif, TPFP in self.positiveSet.iteritems():
						if not (tTP-TPFP[0]>= 0 and tFP-TPFP[1] >= 0):
							continue
						b = self.scoringFunction.fitness(tTP-TPFP[0], tFN+TPFP[0], tFP-TPFP[1], tTN+TPFP[1])
						if b<=a:
							cTPFP[0] += TPFP[0]
							cTPFP[1] += TPFP[1]
							motifs.append(motif)
					if cTPFP[0] == tTP and cTPFP[1] == tFP:
						check = a
						TP = tTP
						FP = tFP
						self.motif = motifs
				
		FN = len(dataset.getMotifIndexes()) - TP
		TN = negCount - FP
		
		
		self.result = check, TP, FN, FP, TN
		r = ""
		for line in self.motif:
			r+=line+"|"

		return Patterns.Optimal(r[:-1], self.motiflength), self.result

class Optimal_new(Model):
	"""Artificial, "Optimal" model"""
	def __init__(self):
		self.description = "Optimal model"
		self.id = "op"

	
	def train(self, dataset=None, cl=None, motifLength=7, excluded=[]):
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		
		self.motiflength = motifLength
		
		#Dictionary containing the TP/FP-count of all motif instances:    {motif:[TP, FP]}
		self.positiveSet = dict()
		fa = dataset.getSequences()
		for i, j in dataset.getMotifIndexes():
			if self.positiveSet.has_key(fa[i][j:j+self.motiflength]):
				self.positiveSet[fa[i][j:j+self.motiflength]][0]+=1
			else:
				self.positiveSet[fa[i][j:j+self.motiflength]] = [1, 0]
		
		
		#Total number of negative possibilities in the set
		negCount = 0

		#The maximum number of FP
		maxFP = 0
		
		#number of positions
		numpos = 0
		for i in range(len(fa)):
			segment = fa[i]
			numpos += len(segment)-self.motiflength+1
			for j in range(len(segment)-self.motiflength+1):
				
				if (i, j) not in dataset.getMotifIndexes():
					negCount += 1
					if self.positiveSet.has_key(segment[j:j+self.motiflength]):
						self.positiveSet[segment[j:j+self.motiflength]][1] += 1
						maxFP += 1
		
		posCount = numpos - negCount
		
		ratios = []
		guaranteed = []
		
		for k, [TP, FP] in self.positiveSet.iteritems():
			if FP == 0: guaranteed.append([FN+100, TP, FP, k])
			else:
				ratio = ((float(TP)/posCount)/(float(FP)/negCount))
				ratios.append([ratio, TP, FP, k])
		ratios.sort()
		ratios.reverse()
		
		ratios = guaranteed + ratios
		
		self.check = -1
		self.i = 0
		
		gTP, gFP, minstances = 0, 0, []
		self.TP, self.FP = 0, 0
		
		for i in range(len(ratios)):
			Null, TP, FP, k = ratios[i]
			minstances.append(k)
			gTP += TP
			gFP += FP
			c = self.scoringFunction.fitness(gTP, posCount-(gTP), gFP, negCount-(gFP))
			if c > self.check:
				self.check = c
				self.i = i
				self.TP = gTP
				self.FP = gFP
				
		minstances = minstances[:self.i+1]
		TP, FP = self.TP, self.FP
		FN, TN = posCount - TP, negCount - FP
		print TP, FP, TN, FN, self.i

		result = self.scoringFunction.fitness(TP, FN, FP, TN), TP, FN, FP, TN
		return Patterns.Optimal("|".join(minstances), self.motiflength), result



"""
................................................................................
Mo_PWM_basic, probabilistic PWM model
................................................................................
"""

class Negative_PWM(Model):
	"""PWM model"""
	def __init__(self):
		self.description = "PWM model"
		self.id = PWMID
		self.pseudow = 1

	def train(self, dataset=None, cl=None, motifLength = 4, pseudoweigth = 1, excluded=[]):
	
		"""Trains the model
		
		dataset: sets the current dataset of the model to dataset
		cl: sets the current pattern score function to cl
		"""
		if dataset != None: self.setDataset(dataset)
		elif self.dataset == None: return None
		
		if cl != None: self.setScoringFunction(cl)
		self.excluded = excluded
		self.pseudow = pseudoweigth
		self.motiflength = motifLength
		
		self.summary = dict()
		self.motif = []
		self.background = {"a":0, "c":0, "g":0, "t":0}
		self.trainingName = ""
		self.negativeList = []
		self.positiveList = []
		self.best = self.scoringFunction.valueRange[0]

		for i in range(self.motiflength):
			self.motif.append({"a":0, "c":0, "g":0, "t":0})


		#Finding the character counts of all motifs and for the background
		fa = dataset.getSequences()
		for i in range(len(fa)):
			for j in range(len(fa[i])-self.motiflength+1):
				
				if (i, j) in self.excluded+dataset.getMotifIndexes(): continue
				for k, c in enumerate(fa[i][j:j+self.motiflength]):
					self.motif[k][c] += 1
		tmpex = []
		for i, j in self.excluded:
			for k in range(j, j+self.motiflength):
				tmpex.append((i, k))

		for i, j in dataset.getMotifIndexes():
			segment = fa[i]
			
			
			if (i, j) in tmpex:
			
				continue
					
			else:
				for k in range(	j, j+self.motiflength):
					self.background[segment[k]] += 1
			
		
		#Normalizing the background counts to get probabilities
		bcount = float(sum(self.background.values()))
		for i in self.background.keys():
			
			self.background[i] = float(self.background[i]) / bcount
			if self.background[i] == 0: self.background[i] = 0.0000000000001
		
		
		#Creating the log-odds score using the motif letter counts
		for i in range(len(self.motif)):

			bcount = float(sum(self.motif[i].values()))
			for j in self.motif[i].keys():
				if self.background[j] == 0: print "hyl"
				self.motif[i][j] = log( ( (float(self.motif[i][j]) + (self.background[j] * self.pseudow) )/ (bcount + self.pseudow) ) / self.background[j] )
		
		
		
		#Count the score of each substring, and add the total score to the corresponding list
		for i in range(len(fa)):
			segment = fa[i]
			for j in range(len(segment)-self.motiflength+1):
				if (i, j) in self.excluded: continue
				value = 0
				for k, c in enumerate(segment[j:j+self.motiflength]):
					value += self.motif[k][c]
					
				if (i, j) in dataset.getMotifIndexes():
					self.positiveList.append(value)
				else:
					self.negativeList.append(value)
		
		self.negativeList.sort()
		self.negativeList.reverse()
		self.positiveList.sort()
		
		self.avg = 0
				
		#Find the threshold value yielding the best pattern score
		for i, threshold in enumerate(self.positiveList):


			FN = i
			TP = len(self.positiveList) - i
			FP = 0
			for j in range(len(self.negativeList)):

				if threshold > self.negativeList[j]:
					FP = j
					break
				elif j == len(self.negativeList)-1:
					FP = len(self.negativeList)
			
			TN = len(self.negativeList) - FP
			check = self.scoringFunction.fitness(TP, FN, FP, TN)
			
			if self.best < check:
				self.best = check
				self.threshold = threshold
				self.result = check, TP, FN, FP, TN
				
		self.motif = [self.motif, self.threshold]
		return Patterns.PWM(self.motif[0], self.motif[1], self.avg), self.result

Mismatch = Mismatch_basic
if HDISTMODULE:
	print "Using CModule"
	Mismatch = Mismatch_cmodule

if __name__ == '__main__':
	import main
	main.main()

