import os, re
import ScoreFunc as sf
import pos




def extractInfo(fileName):
	
	text = file(fileName).read()
		
	p = text.find("sites sorted by position p-value")
	
	# Move 4 lines further down
	for i in range(4): p = text.find("\n", p+1) + 1
	
	res = {}
	while text[p:p+20] != "-"*20:
		nextp = text.find("\n", p+1) + 1
		fields = text[p:nextp].split()
		if fields[1] in ("+", "-"): #revcomp has been used
			if fields[0] in res: res[fields[0]].append({"pos":int(fields[2])-1, "p-value":float(fields[3]), "motif":fields[5], "strand":fields[1]})
			else: res[fields[0]] = [{"pos":int(fields[2])-1, "p-value":float(fields[3]), "motif":fields[5], "strand":fields[1]}]
		else:
			if fields[0] in res: res[fields[0]].append({"pos":int(fields[1])-1, "p-value":float(fields[2]), "motif":fields[4], "strand":"+"})
			else: res[fields[0]] = [{"pos":int(fields[1])-1, "p-value":float(fields[2]), "motif":fields[4], "strand":"+"}]
		
		p = nextp
	return res


def tf_getAllHits(fileName):
	#Gets all hits in all sequences
	
	fp = file(fileName)
	
	
	seq_id = ""
	seq = ""
	outputList = []
	line = fp.readline()
	res = {}
	sites = []
	sequences = []
	while line:
		if line[0] in ('#','\n'):
			seq += line
			line = fp.readline()
			continue
		if line[0] in ('>'):
			seq += line
			seq_id = line.strip()
			
			line = fp.readline()
			continue
		tmp = ""
		while (line) and (not line[0] == '>'):
			
			tmp += line.strip()
			line = fp.readline()
		
		alignment = seq_id.split("|")[4].lower()
		sites.append(alignment)
		sequences.append((seq_id.split()[0][1:], tmp))
		seq = ""
	sequences = list(set(sequences))
	for seq_id, seq in sequences:
		pos = []
		for a in sites:
			p = seq.find(a)
			while p != -1:
				pos.append(p)
				p = seq.find(a, p+1)
		
		res[seq_id] = {"pos":pos, "len":len(seq), "mlen":len(a)}
		
		
	
	fp.close()
	return res


def tf_getHits(fileName):
	#Gets one hit in each sequence. For all hits use getAllHits()
	
	fp = file(fileName)
	
	
	seq_id = ""
	seq = ""
	outputList = []
	line = fp.readline()
	res = {}
	while line:
		if line[0] in ('#','\n'):
			seq += line
			line = fp.readline()
			continue
		if line[0] in ('>'):
			seq += line
			seq_id = line.strip()
			
			line = fp.readline()
			continue
		tmp = ""
		while (line) and (not line[0] == '>'):
			
			tmp += line.strip()
			line = fp.readline()
		
		alignment = seq_id.split("|")[4].lower()
		
		res[seq_id.split()[0][1:]] = {"pos":[tmp.find(alignment)], "len":len(tmp), "mlen":len(alignment)}
		
		
		seq = ""
	
	fp.close()
	return res

def tf_getScore(datasetFolder, resultFolder, datasetFilter, scoringFunction):
	# Complete hit or complete miss
	if not datasetFolder[-1] in ("\\", "/", os.sep): datasetFolder += os.sep
	if not resultFolder[-1] in ("\\", "/", os.sep): resultFolder += os.sep
	
	
	pattern = re.compile(datasetFilter)
	d = os.listdir(datasetFolder)
	d.sort()
	
	ret = {}
	
	for f in d:
		fn = datasetFolder+f
		rn = resultFolder+f+".txt"
		
		if not pattern.match(f) or os.path.isdir(fn) or not os.path.getsize(fn): continue
		ah = tf_getHits(fn) ## Change according to what method should be used
		fh = extractInfo(rn)
		
		
		T = 0

		for k in ah.keys(): T += ah[k]["len"] - ah[k]["mlen"] + 1
			
		TP, FP, FN = 0, 0, 0
		for k in ah.keys():
			positions = set(ah[k]["pos"])
			hits = set()
			
			if k in fh: hits = set([fh[k][0]["pos"]])
			TP += len(positions & hits)
			FP += len(hits-positions)
			FN += len(positions-hits)
					
		ret[f] = scoringFunction(TP, FP, FN, T-(TP+FP+FN)), TP, FP, FN, T-(TP+FP+FN)
	return ret

def tm_getSeqLen(fileName):
	fp = file(fileName)
	
	seq_id = ""
	seq = ""
	outputList = []
	line = fp.readline()
	res = []
	while line:
		if line[0] in ('#','\n'):
			seq += line
			line = fp.readline()
			continue
		if line[0] in ('>'):
			seq += line
			seq_id = line.strip()
			
			line = fp.readline()
			continue
		tmp = ""
		while (line) and (not line[0] == '>'):
			
			tmp += line.strip()
			line = fp.readline()
		
		res.append(len(tmp))
		
		
		seq = ""
	
	fp.close()

	return res
	
def tm_getScoreNucleotideLevel(datasetFolder, resultFolder, datasetFilter, scoringFunction):
	p = pos.getPositions()
	
	
	if not datasetFolder[-1] in ("\\", "/", os.sep): datasetFolder += os.sep
	if not resultFolder[-1] in ("\\", "/", os.sep): resultFolder += os.sep
	
	pattern = re.compile(datasetFilter)
	d = os.listdir(datasetFolder)
	d.sort()
	
	ret = {}
	for f in d:
		fn = datasetFolder+f
		rn = resultFolder+f+".txt"
		
		if not pattern.match(f) or os.path.isdir(fn) or not os.path.getsize(fn): continue
		
		#Foundhits
		fh = extractInfo(rn)
		
		foundHits = []
		for k in fh.keys():
			seqnum = int(k[4:])
			for d in fh[k]:
				for positions in range(d["pos"], d["pos"]+len(d["motif"])):
					foundHits.append((seqnum, positions))
		foundHits = set(foundHits)
		
		
		#Actualhits
		actualHits = []
		
		sl = tm_getSeqLen(fn)
		
		T = 0
		for k in sl: T += k
		
		
		
		dpos = p[f[:-len("*.fasta")]]
		for seqid, invpos, instance, null in dpos:
			
			for positions in range(sl[seqid] + invpos, sl[seqid] + invpos + len(instance)):
				actualHits.append((seqid, positions))
		actualHits = set(actualHits)
		
		#Calculate
		TP, FP, FN = 0, 0, 0
		TP += len(actualHits & foundHits)
		FP += len(foundHits-actualHits)
		FN += len(actualHits-foundHits)
					
		ret[f] = scoringFunction(TP, FP, FN, T-(TP+FP+FN)), TP, FP, FN, T-(TP+FP+FN)
	return ret
		
def tf_getScoreNucleotideLevel(datasetFolder, resultFolder, datasetFilter, scoringFunction):
	#Accounting for overlap
	if not datasetFolder[-1] in ("\\", "/", os.sep): datasetFolder += os.sep
	if not resultFolder[-1] in ("\\", "/", os.sep): resultFolder += os.sep
	
	
	pattern = re.compile(datasetFilter)
	d = os.listdir(datasetFolder)
	d.sort()
	
	
	
	
	ret = {}
	for f in d:
		fn = datasetFolder+f
		rn = resultFolder+f+".txt"
		
		if not pattern.match(f) or os.path.isdir(fn) or not os.path.getsize(fn): continue
		ah = tf_getHits(fn) ## Change according to what method should be used
		fh = extractInfo(rn)
		
		
		
		
		T = 0

		for k in ah.keys(): T += ah[k]["len"]
			
		TP, FP, FN = 0, 0, 0
		for k in ah.keys():
			positions = ah[k]["pos"]
			hits = []
			if k in fh: hits = [fh[k][0]["pos"]]
			

			tpositions, thits = [], []
			for p in positions:
				for i in range(p, p+ah[k]["mlen"]): tpositions.append(i)
			for h in hits:
				for i in range(h, h+len(fh[k][0]["motif"])): thits.append(i)
			positions, hits = set(tpositions), set(thits)
			
			
			#print positions, hits
			TP += len(positions & hits)
			FP += len(hits-positions)
			FN += len(positions-hits)
					
		ret[f] = scoringFunction(TP, FP, FN, T-(TP+FP+FN)), TP, FP, FN, T-(TP+FP+FN)
	return ret


def temporary_ah():
	import GraphGen as gg
	from ExtractResults import alteredHits
	cc = sf.wcc(0)
	
	
	#TRANSFAC
	
	null, h1 = alteredHits("a", ".+res")
	null, h2 = alteredHits("b", ".+res")
	null, h3 = alteredHits("c", ".+res")
	
	real_oops = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\oops_known_l_out", ".+fas", cc.fitness)
	real_zoops = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\zoops_known_l_out", ".+fas", cc.fitness)
	real_zoops_unk = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\zoops_unknown_l_out", ".+fas", cc.fitness)
	
	mchain_oops = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\oops_known_l_out", ".+fas", cc.fitness)
	mchain_zoops = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\zoops_known_l_out", ".+fas", cc.fitness)
	mchain_zoops_unk = tf_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain",r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\zoops_unknown_l_out", ".+fas", cc.fitness)
	
	r1, r2, r3, m1, m2, m3 = real_oops, real_zoops, real_zoops_unk, mchain_oops, mchain_zoops, mchain_zoops_unk
	
	realkey, mchainkey, realahkey = [], [], []
	
	temp = []
	for k, v in h1.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: realkey.append(k)
	
	temp = []
	for k, v in h2.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: mchainkey.append(k)
	
	temp = []
	for k, v in h3.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: realahkey.append(k)
	
	#END TRANSFAC
	#TOMPA
	null, to1 = alteredHits("added/generic/pwm", ".+res")
	null, to2 = alteredHits("added/mchain/pwm", ".+res")
	null, to3 = alteredHits("added/real/pwm", ".+res")
	
	to_re_kno = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real\mwgiven_out", ".+fasta", cc.fitness)
	to_re_unk = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real\out", ".+fasta", cc.fitness)
	to_re_rev = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real\revcomp", ".+fasta", cc.fitness)
	to_mc_kno = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain\mwgiven_out", ".+fasta", cc.fitness)
	to_mc_unk = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain\out", ".+fasta", cc.fitness)
	to_mc_rev = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\mchain\revcomp", ".+fasta", cc.fitness)
	to_ge_kno = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic\mwgiven_out", ".+fasta", cc.fitness)
	to_ge_unk = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic\out", ".+fasta", cc.fitness)
	to_ge_rev = tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\generic\revcomp", ".+fasta", cc.fitness)
	
	
	to_realkey, to_mchainkey, to_generickey = [], [], []
	
	temp = []
	for k, v in to3.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: to_realkey.append(k)
	
	temp = []
	for k, v in to2.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: to_mchainkey.append(k)
	
	temp = []
	for k, v in to1.iteritems():	temp.append((cc.fitness(*v[0]), k))
	temp.sort()
	for v, k in temp: to_generickey.append(k)
	
	
	import pickle as p
	r = p.load(file("results.dict"))["MEME"]
	
	"""
	tmp = to_ge_kno
	tkey = to_generickey
	
	TP, FP, FN, TN = 0, 0, 0, 0
	for k in tkey:
		
		tTP, tFN, tFP, tTN = tmp[k.split("_-_")[0]][1:]
		TP += tTP
		TN += tTN
		FP += tFP
		FN += tFN
	print cc.fitness(TP, FN, FP, TN)
	print TP+FP+TN+FN
	"""
	
	"""
	TP, FN, FP, TN = 0, 0, 0, 0
	for a, tTP, tFN, tFP, tTN in r:
		TP += tTP
		FP += tFP
		FN += tFN
		TN += tTN
	print cc.fitness(TP, FN, FP, TN)
	"""
	
	"""
	TP, FP, FN, TN = 0, 0, 0, 0
	for a, b, c, d, e in r:
		f = a+".fasta"
		if a[-1] == "g":
			if f in to_ge_unk:
				print f
				tTP, tFN, tFP, tTN = to_ge_unk[f][1:]
				TP += tTP
				FP += tFP
				FN += tFN
				TN += tTN
		if a[-1] == "m":
			if f in to_mc_unk:
				print f
				tTP, tFN, tFP, tTN = to_mc_unk[f][1:]
				TP += tTP
				FP += tFP
				FN += tFN
				TN += tTN
		if a[-1] == "r":
			if f in to_re_rev:
				print f
				tTP, tFN, tFP, tTN = to_re_unk[f][1:]
				TP += tTP
				FP += tFP
				FN += tFN
				TN += tTN
	print cc.fitness(TP, FN, FP, TN)
	"""
	
	
	
	
	a, b = [], []
	for k in to_realkey:
		a.append(cc.fitness(*to3[k][0]))
		b.append(to_re_rev[k.split("_-_")[0]][0])
	
	from pylab import corrcoef
	print corrcoef(a, b)
	
	
	"""
	
	genkeys, mchainkeys, realkeys = to1.keys(), to2.keys(), to3.keys()
	
	TP, FP, FN, TN = 0, 0, 0, 0
	tompaTP, tompaFP, tompaFN, tompaTN = 0, 0, 0, 0
	
	for dset, toTP, toFN, toFP, toTN in r:
		if dset[-1] == "g":
			print dset
			for k in genkeys:
				if k[:len(dset)] == dset:
					if k in to1:
						tTP, tFN, tFP, tTN = to1[k][0]
						TP += tTP
						FP += tFP
						FN += tFN
						TN += tTN
						tompaTP += toTP
						tompaFP += toFP
						tompaFN += toFN
						tompaTN += toTN
						
		elif dset[-1] == "m":
			print dset
			for k in mchainkeys:
				if k[:len(dset)] == dset:
					if k in to2:
						tTP, tFN, tFP, tTN = to2[k][0]
						TP += tTP
						FP += tFP
						FN += tFN
						TN += tTN
						tompaTP += toTP
						tompaFP += toFP
						tompaFN += toFN
						tompaTN += toTN
		elif dset[-1] == "r":
			print dset
			for k in realkeys:
				if k[:len(dset)] == dset:
					if k in to3:
						tTP, tFN, tFP, tTN = to3[k][0]
						TP += tTP
						FP += tFP
						FN += tFN
						TN += tTN
						tompaTP += toTP
						tompaFP += toFP
						tompaFN += toFN
						tompaTN += toTN
	
	print TP+ FN+ FP+ TN
	print tompaTP+ tompaFN+ tompaFP+ tompaTN
	print cc.fitness(TP, FN, FP, TN)
	print cc.fitness(tompaTP, tompaFN, tompaFP, tompaTN)
	"""
	
	
	
	#END TOMPA
	
	
	
	#for k in key:
	#	a.append(cc.fitness(*h1[k][0]))
	#	b.append(cc.fitness(*h2[k][0]))
	#	c.append(h3[k[:10]][0])
	#	d.append(h4[k[:10]][0])
	"""
	a, b = [], []
	for k in mchainkey:
		a.append(cc.fitness(*h2[k][0]))
		b.append(m3[k[:10]][0])
	
	from pylab import corrcoef
	print corrcoef(a, b)
	"""
	
	"""
	tmp = m3
	tkey = mchainkey
	
	
	TP, FP, FN, TN = 0, 0, 0, 0
	for k in tkey[-50:]:
		
		tTP, tFN, tFP, tTN = tmp[k[:10]][1:]
		TP += tTP
		TN += tTN
		FP += tFP
		FN += tFN
	print cc.fitness(TP, FN, FP, TN)
	TP, FP, FN, TN = 0, 0, 0, 0
	for k in tkey[:-50]:
		
		tTP, tFN, tFP, tTN = tmp[k[:10]][1:]
		TP += tTP
		TN += tTN
		FP += tFP
		FN += tFN
	print cc.fitness(TP, FN, FP, TN)
	"""
	
	"""
	input = []
	
	input.append({'x':range(len(a)), 'y':a, 'type':gg.SIMPLE,\
						'lineproperties':{'label':'Real', 'marker':'.', 'linewidth':1, 'color':'b',\
						'markerfacecolor':'b'}})
	input.append({'x':range(len(b)), 'y':b, 'type':gg.SIMPLE,\
						'lineproperties':{'label':'Mchain', 'marker':'.', 'linewidth':1, 'color':'r',\
						'markerfacecolor':'r'}})
	input.append({'x':range(len(c)), 'y':c, 'type':gg.SIMPLE,\
						'lineproperties':{'label':'Real, MEME', 'marker':'.', 'linewidth':1, 'color':'g',\
						'markerfacecolor':'g'}})
	input.append({'x':range(len(d)), 'y':d, 'type':gg.SIMPLE,\
						'lineproperties':{'label':'Mchain, MEME', 'marker':'.', 'linewidth':1, 'color':'y',\
						'markerfacecolor':'y'}})
	"""
						
	#gg.simplePlot(input, title="Meme_vs_exhaustive_relative2", xlabel=" ", ylabel="CC", fname="C:\\temp\\*")
	
def getIDs(filename):
	fp = file(filename)
	line = fp.readline()
	ret = []
	while line:
		if line[0] in ('#','\n'):
			line = fp.readline()
			continue
		if line[0] in ('>'):
			ret.append(line)
			line = fp.readline()
			continue
		while (line) and (not line[0] == '>'):
			line = fp.readline()
	return ret
	

def getMapping(old_id_dir, new_id_dir, filename):
	if old_id_dir[-1] not in ("\\", "/", os.sep):
		old_id_dir += os.sep
	if new_id_dir[-1] not in ("\\", "/", os.sep):
		new_id_dir += os.sep
	
	o = getIDs(old_id_dir + filename)
	n = getIDs(new_id_dir + filename)
	
	for a, b in zip(o, n):
		if "".join(a.split("|")[-1:]) != "".join(b.split("|")[-1:]):
			print "error: not equal"
			sys.exit(0)
	n = [a.split("|")[0][1:] for a in n]
	o = [a[1:] for a in o]
	return zip(o, n)
	
if __name__ == "__main__":
	import os
	old_dirname = r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain"
	new_dirname = r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\newID"
	subdir = r"\zoops_known_l_out" + os.sep
	outputdir = "zoops_known" + os.sep
	path = old_dirname+ subdir +os.sep
	p = re.compile(r".+fas\.txt$")
	if not os.path.exists(path+outputdir): os.makedirs(path+outputdir)
	for f in os.listdir(path):
		if not p.match(f):
			continue
		fname = ".".join(f.split(".")[:-1])
		
		mapping = getMapping(old_dirname, new_dirname, fname)
		info = extractInfo(path + f)
		of = file(path + outputdir + f.split(".")[0]+"_answer.txt", "w")
		for k in info:
			k2 = "rall"
			for o, n in mapping:
				
				if o[:len(k)] == k:
					k2 = n
					break
			if k2 == "rall":
				print "error. exit"
				import sys
				sys.exit(0)
			
			of.write(", ".join([k2, repr(info[k][0]["pos"]+1), repr(info[k][0]["pos"]+1+ len(info[k][0]["motif"])), info[k][0]["motif"]]) +"\n")
			#repr(info[k]["pos"]+1+len(info[k]["motif"])), info[k]["motif"]])+ "\n")
			
		of.close()
	
		
		
		
	

	#for k in extractInfo(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\out\M00441.fas.txt").keys():
	#	print k in getHits(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\M00441.fas").keys()
	#print getHits(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\M00441.fas")
	#print
	#print extractInfo(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\out\M00441.fas.txt")
	#cc = sf.wcc(0)
	#getScore(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\out", ".+fas", cc.fitness)
	#getScore(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\out", ".+fas", cc.fitness)	
	#print len(getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\mchain\out", ".+fas", cc.fitness))
	#getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\transfac\noIUPAC\nc_sequences\noshort\no_d_seq\uniqueids\end_truncated\min_five_seq\out", ".+fas", cc.fitness)
	#temporary_ah()
	#print tm_getScoreNucleotideLevel(r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real", r"C:\Diplom\Prosjekt\framework\trunk\shared\md_datasett\dataset\dna\tompa05assesing\real\mwgiven_out", ".+fasta", cc.fitness)
	
	