import pylab as p
from scipy import arange, interpolate



SIMPLE = 0
SPLINE = 1
TEXT = 2


def simpleBars(bars, groupWidth=1.0, xticks=None, title=None, fname=None, showWindow=False, xlabel=None, ylabel=None, yaxis=None, axis = None):
	w = 0
	
	maxy = 0
	for i in range(len(bars)):
		if not bars[i].has_key('width'): bars[i]['width'] = 1
		if not bars[i].has_key('color'): bars[i]['color'] = 'b'
		if not bars[i].has_key('stdev'): bars[i]['stdev'] = None
		w += bars[i]['width']
		
		if max(bars[i]['values'])> maxy:maxy = max(bars[i]['values'])
		
	
	barw = float(w)
	w += groupWidth
	
	ind = arange(0, w*len(bars[0]['values']), w)
	
	
	p.xticks(arange(barw/2, (barw/2)+(w*len(bars[0]['values'])), w), xticks, fontsize='small')
	
	labels = []
	labelbars = []
	for bar in bars:
		b = p.bar(ind, bar['values'], bar['width'], color=bar['color'], yerr=bar['stdev'])
		ind += bar['width']
		
		if bar.has_key('label'):
			labels.append(bar['label'])
			labelbars.append(b[0])
	
	if len(labels) > 0:
		leg = p.legend( labelbars, labels, loc='best', shadow=True, pad=0.5 )
		ltext  = leg.get_texts()
		
		frame  = leg.get_frame()
		#frame.set_height(frame.height*2)
		p.setp(ltext, fontsize='small')
		
	
	
	if xlabel != None: p.xlabel(xlabel)
	if ylabel != None: p.ylabel(ylabel)
	if title != None: p.title(title)
	
	
	if yaxis == None:
		if axis: p.axis((axis[0], axis[1], axis[2], axis[3]))		
		else: p.axis((0, w*len(bars[0]['values']), 0, maxy*1.25))
			
	else:p.axis((0, w*len(bars[0]['values']), yaxis[0], yaxis[1]))
	if fname:
		ft = ""
		if title != None: ft = title
		fname = fname.replace('*', ft)				

		if fname:
			p.savefig(fname+".png", dpi=200, facecolor='#ffffff', edgecolor='w')
	
	if showWindow: p.show()
	
	
	
def scatter(plots, axis=None, xlabel=None, ylabel=None, title=None, fname="*", showWindow=False):
	p.clf()
	for plot in plots:
		color = "b"
		marker = "o"
		area = [100]*len(plot['x'])
		alpha = 1.
		if "c" in plot: color = plot['c']
		if "m" in plot: marker = plot['m']
		if "a" in plot: area = plot['a']
		if "alpha" in plot: alpha = plot['alpha']
		
		p.scatter(plot['x'], plot['y'], s=area, marker=marker, c=color, alpha=alpha)
		
		if "text" in plot:

			
			for x, y, t,in zip(plot['x'], plot['y'], plot['text']):

				p.text(x+0.01, y+0.01, t)
		
	
	if axis != None: p.axis(axis)
	if xlabel != None: p.xlabel(xlabel)
	if ylabel != None: p.ylabel(ylabel)
	if title != None: p.title(title)	
	
	if fname:
		ft = ""
		if title != None: ft = title
		fname = fname.replace('*', ft)				

		if fname:
			p.savefig(fname+".png", dpi=200, facecolor='#ffffff', edgecolor='w')
	if showWindow: p.show()
	

def simplePlot(plots, axis=None, xlabel=None, ylabel=None, title=None, fname="*", showWindow=False):
	#plots: 
	#	A tuple/list of dictionaries. Each dicitionary describes one plot, and all plots
	#	will be plotted in the same grid.
	#	At a minimum, each dictionary must contain values for keys 'x', 'y' and 'type'
	#	dictionarykeys:
	#	'x': list of x-values
	#	'y': list of y-values
	#		(plot i has coordinates x[i], y[i])
	#	'type': SIMPLELINE, SPLINE or TEXT.
	#		SIMPLE: simple plots
	#		SPLINE: 'softens' the line plot using cubic splines
	#		TEXT: text labels
	#	'lineproperties': A dictionary of line properties to set. Not used if plots['type']==TEXT
	#		see http://matplotlib.sourceforge.net/tutorial.html for details. A small selection of properties:
	#		'label': string describing the plot in the legend. If omitted, the plot is not included in the legend
	#		'linestyle': one of -- : -. -
	#		'marker': one of + , o . s v x > < ^
	#
	#	Only applied for type TEXT:
	#	'font': Dictionary defining a font. See matplotlibdocumentation for details.
	#	'dist': if set, the label is only plotted if it is farther away than dist from any other textlabel.
	# # # #
	#additional parameters:
	#axis: list/tuple of length 4 defining the axis [xmin, xmax, ymin, ymax]
	#xlabel: label for the x-axis
	#ylabel: label for the y-axis
	#title: figure title
	#fname: filename to write the figure to. '*' is replaced with the title parameter
	#	if not fname: no file is written
	#	if fname=='*' and not title: no file is written
	#showWindow: if showWindow==True, graph window is attempted displayed using default backend
	
	p.clf() #Clears current figure (in case the function is called several times in one run)
		
	
	labels = [] #stores labels to be used in the legend
	lines = [] #stores lines to be used in the legend
	edist = None
	if axis: edist = Edist(axis) #stores points that are Eucledian distance-dependent
	else: edist = Edist()
	
	
	for simpleplot in plots:
		if not (simpleplot.has_key('x') and simpleplot.has_key('y') and simpleplot.has_key('type')):
			print "warning: plot-dictionary must contain 'x'-, 'y'- and 'type'-keys. Following plot omitted:"
			print simpleplot
			continue
		x = simpleplot['x']
		y = simpleplot['y']
		
		if simpleplot['type'] == SPLINE:
			#Rounded graph using a cubic spline
			tck = interpolate.splrep(x, y, s=0)
			xspline = arange(x[0], x[-1], float(x[1]-x[0])/100)
			yspline= interpolate.splev(xspline, tck, der=0)
			
			line = p.plot(xspline, yspline)
			
			
			lp = {}
			if simpleplot.has_key('lineproperties'):
				lp = simpleplot['lineproperties']
			
			for k in lp.keys():
				if k.find('marker') != -1: continue
				elif k=='label':
					
					labels.append(lp['label'])
					lines.append(line)
				else:
					p.setp(line, k, lp[k])
			
			if lp.has_key('marker'):
				#print lp['marker']
				m = p.plot(x, y, lp['marker'])
				for k in lp.keys():
					if k.find('line') != -1 or k == 'label' or k=='marker': continue
					p.setp(m, k, lp[k])
				
		elif simpleplot['type'] == SIMPLE:
			line = p.plot(x, y)
			
		
			
			lp = {}
			if simpleplot.has_key('lineproperties'):
				lp = simpleplot['lineproperties']
			for k in lp.keys():
				if k=='label':
					labels.append(lp['label'])
					lines.append(line)
				p.setp(line, k, lp[k])
		elif simpleplot['type'] == TEXT:
			#Get defined distance, if any
			edistance = -1
			if simpleplot.has_key('dist'):
				edistance = simpleplot['dist']
			
			
			#Use font, if specified
			tlfont = {}
			if simpleplot.has_key('font'):
				tlfont = simpleplot['font']
			
			
			tx = [""]*len(x)
			if simpleplot.has_key('text'):
				tx = simpleplot['text']
			

			for i in range(len(x)):
				if not edist.isFarther((x[i], y[i]), edistance): continue
				
				p.text(x[i], y[i], str(tx[i]), tlfont)
				point = (x[i], y[i])
				edist.addPoint(point)
		
			
	


	
	if len(lines) > 0:
		leg = p.legend( lines, labels, loc='lower right', shadow=True, pad = 0.5)
		ltext  = leg.get_texts()
		llines = leg.get_lines()
		frame  = leg.get_frame()
		#print dir(frame), 
		#print frame.__doc__
		frame.set_height(frame.height*2)
		p.setp(ltext, fontsize='small')
		p.setp(llines, linewidth=1.5)
	if axis != None: p.axis(axis)
	if xlabel != None: p.xlabel(xlabel)
	if ylabel != None: p.ylabel(ylabel)
	if title != None: p.title(title)
	


	if fname:
		ft = ""
		if title != None: ft = title
		fname = fname.replace('*', ft)				

		if fname:
			p.savefig(fname+".png", dpi=200, facecolor='#ffffff', edgecolor='w')
	if showWindow: p.show()



# # # # # # # # # # # # # # # # # # #
#Mathemathical functions
def avgdev(values):
	#returns the average and standard deviation as a tuple from a list of values
	avg = float(sum(values))/float(len(values))
	dev = 0
	for value in values: dev += (value-avg)**2
	dev /= len(values)
	dev **= 0.5
	return avg, dev


# # # # # # # # # # # # #
#Eucledian distance class
class Edist:
	#Todo: normalize positions!
	def __init__(self, dimensions=[0, 1, 0, 1]):
		self.l = []
		self.d = dimensions
		
	def isFarther(self, p, mindist):
		
		for q in self.l:
			
			if self.eucledianDistance(p, q) < mindist: return False
		
		return True

	def eucledianDistance(self, pointp, pointq):
		px, py = pointp
		qx, qy = pointq
		return (((px-qx)**2)+((py-qy)**2))**0.5
	
	def addPoint(self, point):
		self.l.append(point)




if __name__=="__main__":
	from random import random
	#examples: 0 is line plot example, 1 is bar example
	example = 1
	
	#Line plot:
	if example == 0:
		
		axis = [0, 10, 0, 5]
		title = "example"
		
		#Create plottingvalues for two graphs
		x1values = x2values = range(axis[0], axis[1]+1)
		y1values = []
		y2values = []
		for l in x1values:
			y1values.append(axis[2]+(random()*axis[3]))
			y2values.append(axis[2]+(random()*axis[3]))
		
		
		
		
		
		input = []
		
		
		
		#Add a spline
		input.append({'x':x1values, 'y':y1values, 'type':SPLINE,\
					'lineproperties':{'label':'Plot 1', 'marker':'x', 'linewidth':2, 'color':'b',\
					'markerfacecolor':'b'}})
		#Add another spline
		input.append({'x':x2values, 'y':y2values, 'type':SPLINE,\
					'lineproperties':{'label':'Plot 2', 'marker':'o', 'linewidth':2, 'color':'r',\
					'markerfacecolor':'r'}})
		
		#Add a dotted line segment between 1values[i] and 2values[i]
		for x1, y1, x2, y2 in zip(x1values, y1values, x2values, y2values):
			input.append({'x':[x1, x2], 'y':[y1, y2], 'type':SIMPLE,\
					'lineproperties':{'linestyle':':', 'linewidth':1, 'color':'k'}})
		
		
		
		
		
		#Add values as text for the first splinegraph
		y1text = []
		y1textpos = []
		for yval in y1values:
			y1textpos.append(yval+0.07)
			y1text.append(str(yval)[:4])
		
		
		input.append({'x':x1values, 'y':y1textpos, 'text':y1text, 'dist':0.02, 'type':TEXT, 'font':{'fontname'   : 'Courier',
			'color'      : 'k',
			'fontweight' : 'bold',
			'fontsize'   : 5}})
		
		simplePlot(input, axis=axis, title=title, xlabel="time", ylabel="temperature", fname="C:\\temp\\*")
	
	#Bars:
	elif example == 1:
		input = []
		input.append({'values':[1, 2, 3, 4, 5], 'color':'r', 'label':'ball'})
		input.append({'values':[5, 4, 3, 2, 1], 'width':0.5, 'color':'b', 'label':'rall'})
		input.append({'values':[10, 4, 2, 7, 8], 'width':1.5, 'color':'#FFFF00', 'stdev':1, 'label':'lall'})
		
		groupLabels = ["Group 1", "Group 2", "Group 3", "Group 4", "Group 5"]
		input.append({'values':[1, 2, 3, 4, 5], 'color':'r', 'label':'ball'})
		
		simpleBars(input, showWindow=True, xticks = groupLabels, title="Bars", xlabel="Groups", fname="c:\\*", ylabel="value")
	
	elif example == 2:
		input = []
		input.append({'x':[1, 2, 3, 4], 'y':[4, 3, 2, 1], 'a':[500, 500, 500, 500], 'c':"r", "m":"^"})
		input.append({'x':[2, 2, 2, 2], 'y':[5, 1, 2, 3], 'c':"g", "m":"s", 'text':["one", "two", "three", "etc"]})
		
		scatter(input, showWindow=True, title="Bars", xlabel="Groups", ylabel="value")