import Ini
Ini.versionInit()

import library.dataset
import SearchUtils
from sets import Set


from DB_constants import HDISTID, IUPACID, PWMID, PBMID, OPTIMALID


class  Pattern:
	def __init__(self):
		pass

class Mismatch(Pattern):
	def __init__(self, hdstring="", maxdist=-1):
		self.id = HDISTID
		self.length = len(hdstring)
		self.hdstring = hdstring
		self.maxdist = maxdist
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [None]*8

		Pattern.__init__(self)
	
	def __str__(self):
		r = "Mismatch pattern\n"
		
		for name, value in vars(self).iteritems():
			if name == "hitlist" : continue
			r += name + " " +str(value)+ "\n"
		return r
		
		#return "Mismatch instance:\nstring: \""+self.hdstring+"\"\tmax. distance: "+str(self.maxdist)
	
	def getPattern(self):
		return [self.hdstring, self.maxdist]
	
	def search(self, dataset, scoringFunction, excluded=[]):
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line)>0: totalPositions += len(line) - len(self.hdstring) + 1
		
		
		self.hitlist = SearchUtils.mismatch(sequences, self.hdstring, self.maxdist)
		print self.hitlist
		
		hits = Set(self.hitlist)
		indexSet = Set(positions)
		
		hits -= Set(excluded)
		indexSet -= Set(excluded)
		
		self.TP = len(hits&indexSet)
		self.FP = len(hits-indexSet)
		self.FN = len(indexSet-hits)
		self.TN = totalPositions - self.TP - self.FP - self.FN - len(excluded)
		print self.TP+self.FP+self.FN+self.TN
		
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)
		


class Regexp(Pattern):
	def __init__(self, rstring="", length = -1):
		self.id = IUPACID
		self.r = rstring
		self.length = length
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [None]*8
		Pattern.__init__(self)
	
	def __str__(self):
		r = "IUPAC\n"

		for name, value in vars(self).iteritems():
			if name == "hitlist" : continue
			r += name + " " +str(value)+ "\n"
		return r
	
	def getPattern(self):
		return [self.rstring, self.length]
	
	def search(self, dataset, scoringFunction, excluded=[]):
		#print "SEARCH, START", self.r
		#for l in dataset._sequence_list[0:5]: print l.seq
		
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line)>0:totalPositions += len(line) - self.length + 1
		
		self.hitlist = Set(SearchUtils.reghit(sequences, self.r, self.length))
		hits = Set(self.hitlist) - Set(excluded)
		indexSet = Set(positions) - Set(excluded)
		self.TP = len(hits&indexSet)
		self.FP = len(hits-indexSet)
		self.FN = len(indexSet-hits)
		self.TN = totalPositions - self.TP - self.FP - self.FN - len(excluded)
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)

class PWM(Pattern):
	def __init__(self, pwm=[], threshold=0, avg=0):
		self.id = PWMID
		self.length = len(pwm)
		self.pwm = pwm
		self.t = threshold
		self.avg = avg
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [None]*8
		Pattern.__init__(self)
	
	#TODO: formatting
	def __str__(self):
		r = "PWM pattern\n"
		for name, value in vars(self).iteritems():
			if name == "hitlist" or name == "pwm": continue
			r += name + " " +str(value)+ "\n"

		r+="\t"
		for i in range(len(self.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in self.pwm[0]:
			r+= str(k)+"\t"
			for i in range(len(self.pwm)):
				r+=str(self.pwm[i][k])[:5]+"\t"
			r+= "\n" 

		return r
	
	def getPattern(self):
		return [self.pwm, self.t]
	
	def search(self, dataset, scoringFunction, excluded = []):
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line)>0:totalPositions += len(line) - len(self.pwm) + 1
		
		self.hitlist = SearchUtils.PWMhit(sequences, self.pwm, self.t)
		hits = Set(self.hitlist) - Set(excluded)
		indexSet = Set(positions) - Set(excluded)
		self.TP = len(hits&indexSet)
		self.FP = len(hits-indexSet)
		self.FN = len(indexSet-hits)
		self.TN = totalPositions - self.TP - self.FP - self.FN - len(excluded)
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)
		
class PWM_BM_new(Pattern):
	def __init__(self, pwm=[], threshold=0, avg=0, maks=-2, bcount=0, background={}):
		self.id = PBMID
		self.length = len(pwm)
		self.pwm = pwm
		self.t = threshold
		self.avg = avg
		self.maks = maks
		self.bcount = bcount
		self.background = background
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [[]] + [None]*7
		Pattern.__init__(self)
	
	#TODO: formatting
	def __str__(self):
		r = "PWM pattern\n"
		for name, value in vars(self).iteritems():
			if name == "hitlist" or name == "pwm" or name == "background": continue
			r += name + " " +str(value)+ "\n"

		
		
		
		r+="\n"
		r+="\t"
		for i in range(len(self.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in self.pwm[0]:
			r+= str(k)+"\t"
			for i in range(len(self.pwm)):
				r+=str(self.pwm[i][k])[:5]+"\t"
			r+= "\n" 
		r+=("."*30)+"\n"
		
		return r
	
	def getPattern(self):
		return [self.pwm, self.background, self.t]
	
	def search(self, dataset, scoringFunction, excluded = []):
		
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		self.TP = 0
		self.FP = 0
		self.FN = 0
		self.TN = 0
		
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line) > 0: totalPositions += len(line) - len(self.pwm) + 1
		
		for i in range(len(sequences)):
			sequence = sequences[i]
			for j in range(len(sequence)-self.length+1):
				if (i, j) in excluded: continue
				
				value = 0
				substring = sequence[j:j+self.length]
				for k, c in enumerate(substring):
					
					value += self.pwm[k][c]
				
				#endold
				
				
				
				
				for k in range(self.length):
					
					preceding = ""
					
					if self.maks > 0:
						preceding = substring[:k][-self.maks:]
						#print "["+preceding+"]"
						
						while not self.background.has_key(preceding):
							preceding = preceding[1:]
					elif self.maks == -1:
						preceding = -1
					
					
					value -= self.background[preceding][substring[k]]
				
				
				
				if (i, j) in self.dataset.getMotifIndexes():
					if value >= self.t:
						self.TP += 1
						self.hitlist.append((i, j))
					else:self.FN += 1
				else:
					if value >= self.t:
						self.FP +=1
						self.hitlist.append((i, j))
					else: self.TN +=1
		
		
		
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		print "#"*10, "\n", self.hitlist, "#"*10
		return (self.score, self.TP, self.FN, self.FP, self.TN)

class PWM_new(Pattern):
	def __init__(self, pwm=[], threshold=0, avg=0, maks=-2, bcount=0):
		self.id = PWMID
		self.length = len(pwm)
		self.pwm = pwm
		self.t = threshold
		self.avg = avg
		self.maks = maks
		self.bcount = bcount
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [None]*8
		Pattern.__init__(self)
	
	#TODO: formatting
	def __str__(self):
		r = "PWM pattern\n"
		for name, value in vars(self).iteritems():
			if name == "hitlist" or name == "pwm": continue
			r += name + " " +str(value)+ "\n"

		
		r+="no  order\n"
		r+="\t"
		for i in range(len(self.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in self.pwm[0][-1]:
			r+= str(k)+"\t"
			for i in range(len(self.pwm)):
				r+=str(self.pwm[i][-1][k])[:5]+"\t"
			r+= "\n" 
		r+="\n"
		
		
		r+="0th  order\n"
		r+="\t"
		for i in range(len(self.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in self.pwm[0][""]:
			r+= str(k)+"\t"
			for i in range(len(self.pwm)):
				r+=str(self.pwm[i][""][k])[:5]+"\t"
			r+= "\n" 
		r+=("."*30)+"\n"
		"""
		#A little too verbose
		r+="1st  order\n"
		
		for i in range(1,len(self.pwm)):
			r+= "position"+str( i )+"\n"
			for k in self.pwm[i].keys():#[""]:
				if type(k)==type(""):
					if len(k)==1:
						for j in self.pwm[i][k]:
							r+= str(j)+"|"+str(k)+":"+str(self.pwm[i][k][j])+", "
			r+="\n"
		r+=("."*30)+"\n"
		r+="2nd  order\n"
		
		for i in range(2,len(self.pwm)):
			r+= "position"+str( i )+"\n"
			for k in self.pwm[i].keys():#[""]:
				if type(k)==type(""):
					if len(k)==2:
						for j in self.pwm[i][k]:
							r+= str(j)+"|"+str(k)+":"+str(self.pwm[i][k][j])+", "
			r+="\n"
		r+=("."*30)+"\n"
		r+="3rd  order\n"
		
		for i in range(3,len(self.pwm)):
			r+= "position"+str( i )+"\n"
			for k in self.pwm[i].keys():#[""]:
				if type(k)==type(""):
					if len(k)==3:
						for j in self.pwm[i][k]:
							r+= str(j)+"|"+str(k)+":"+str(self.pwm[i][k][j])+", "
			r+="\n"
		r+=("."*30)+"\n"
		"""
		return r
	
	def getPattern(self):
		return [self.pwm, self.t]
	
	def search(self, dataset, scoringFunction, excluded = []):
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		self.TP = 0
		self.FP = 0
		self.FN = 0
		self.TN = 0
		
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line)>0:totalPositions += len(line) - len(self.pwm) + 1
		
		for i in range(len(sequences)):
			sequence = sequences[i]
			for j in range(len(sequence)-self.length+1):
				if (i, j) in excluded: continue
				
				value = 0
				substring = sequence[j:j+self.length]
				
				for k in range(self.length):
					preceding = ""
					if self.maks > 0:
						preceding = substring[:k][-self.maks:]
						while not self.pwm[k].has_key(preceding):
							preceding = preceding[1:]
					elif self.maks == -1:
						preceding = -1
					
					
					value += self.pwm[k][preceding][substring[k]]
				
				
				
				if (i, j) in self.dataset.getMotifIndexes():
					if value >= self.t:self.TP += 1
					else:self.FN += 1
				else:
					if value >= self.t:self.FP +=1
					else: self.TN +=1
		
		
		
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)

class PWM_BM(Pattern):
	def __init__(self, pwm=[], order=1, background = {}, threshold=0):
		self.id = PBMID
		self.length = len(pwm)
		self.pwm = pwm
		self.t = threshold
		self.b = background
		self.o = order
		
		self.hitlist, self.scoringFunction, self.dataset, self.score, self.TP, self.FP, self.TN, self.FN = [None]*8
		Pattern.__init__(self)
	
	#TODO: formatting
	def __str__(self):
		r = "PWM w/bgm pattern\n\t"
		for name, value in vars(self).iteritems():
			if name == "hitlist" or name == "pwm": continue
			r += name + " " +str(value)+ "\n"
		r+="\t"
		for i in range(len(self.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in self.pwm[0]:
			r+= str(k)+"\t"
			for i in range(len(self.pwm)):
				r+=str(self.pwm[i][k])[:5]+"\t"
			r+= "\n" 
		

		
		
		return r
	
	def getPattern(self):
		return [self.pwm, self.b, self.t, self.o]
	
	def search(self, dataset, scoringFunction, excluded = []):
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			totalPositions += len(line) - len(self.pwm) + 1
		
		self.hitlist = SearchUtils.PWM_BMhit(sequences, self.pwm, self.b, self.o, self.t)
		hits = Set(self.hitlist) - Set(excluded)
		indexSet = Set(positions) - Set(excluded)
		self.TP = len(hits&indexSet)
		self.FP = len(hits-indexSet)
		self.FN = len(indexSet-hits)
		self.TN = totalPositions - self.TP - self.FP - self.FN - len(excluded)
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)


class Optimal(Pattern):
	def __init__(self, ilist, length):
		self.id = OPTIMALID
		self.ilist = ilist
		self.length = length
		Pattern.__init__(self)
	
	def __str__(self):
		r = "Optimal pattern\n"
		
		for name, value in vars(self).iteritems():
			if name == "hitlist" : continue
			r += name + " " +str(value)+ "\n"
		return r
	
	def search(self, dataset, scoringFunction, excluded=[]):
		self.scoringFunction = scoringFunction.id
		self.dataset = dataset
		sequences = dataset.getSequences()
		positions = dataset.getMotifIndexes()
		totalPositions = 0
		for line in sequences:
			if len(line)>0:totalPositions += len(line) - self.length + 1
		
		self.hitlist = SearchUtils.reghit(sequences, self.ilist, self.length)
		hits = Set(self.hitlist)
		indexSet = Set(positions)
		self.TP = len(hits&indexSet)
		self.FP = len(hits-indexSet)
		self.FN = len(indexSet-hits)
		self.TN = totalPositions - self.TP - self.FP - self.FN
		self.score = scoringFunction.fitness(self.TP, self.FN, self.FP, self.TN)
		return (self.score, self.TP, self.FN, self.FP, self.TN)
	
def getPatternString(pattern):
	
	
	
	if pattern.id == OPTIMALID:
		return pattern.ilist
	elif pattern.id == HDISTID:
		return pattern.hdstring + " distance:"+str(pattern.maxdist)
	elif pattern.id == IUPACID:
		return pattern.r
	elif pattern.id == PWMID:
		r = "threshold:"+str(pattern.t)+"\n"
		variables = vars(pattern).keys()
		if "bcount" in variables: 
			r+= "background count:"+str(pattern.bcount)+"\n"
			r+= "max order:"+str(pattern.maks)+"\n"
			
			r+="no  order\n"
			r+="\t"
			for i in range(len(pattern.pwm)):
				r += str(i)+"\t\t"
			r+="\n"
			for k in pattern.pwm[0][-1]:
				r+= str(k)+"\t"
				for i in range(len(pattern.pwm)):
					r+=str(pattern.pwm[i][-1][k])[:5]+"\t"
				r+= "\n" 
			r+="\n"
			
			
			r+="0th  order\n"
			r+="\t"
			for i in range(len(pattern.pwm)):
				r += str(i)+"\t\t"
			r+="\n"
			for k in pattern.pwm[0][""]:
				r+= str(k)+"\t"
				for i in range(len(pattern.pwm)):
					r+=str(pattern.pwm[i][""][k])[:5]+"\t"
				r+= "\n" 
			r+=("."*30)+"\n"
			r+="1st  order\n"
			
			for i in range(1,len(pattern.pwm)):
				r+= "position"+str( i )+"\n"
				for k in pattern.pwm[i].keys():#[""]:
					if type(k)==type(""):
						if len(k)==1:
							for j in pattern.pwm[i][k]:
								r+= str(j)+"|"+str(k)+":"+str(pattern.pwm[i][k][j])+", "
				r+="\n"
			r+=("."*30)+"\n"
			r+="2nd  order\n"
			
			for i in range(2,len(pattern.pwm)):
				r+= "position"+str( i )+"\n"
				for k in pattern.pwm[i].keys():#[""]:
					if type(k)==type(""):
						if len(k)==2:
							for j in pattern.pwm[i][k]:
								r+= str(j)+"|"+str(k)+":"+str(pattern.pwm[i][k][j])+", "
				r+="\n"
			r+=("."*30)+"\n"
			r+="3rd  order\n"
			
			for i in range(3,len(pattern.pwm)):
				r+= "position"+str( i )+"\n"
				for k in pattern.pwm[i].keys():#[""]:
					if type(k)==type(""):
						if len(k)==3:
							for j in pattern.pwm[i][k]:
								r+= str(j)+"|"+str(k)+":"+str(pattern.pwm[i][k][j])+", "
				r+="\n"
			r+=("."*30)+"\n"
			"""
				r+= str(k)+"\t"
				for i in range(len(pattern.pwm)):
					r+=str(pattern.pwm[i][""][k])[:5]+"\t"
				r+= "\n" 
			"""
		else:
		
			r+="\t"
			for i in range(len(pattern.pwm)):
				r += str(i)+"\t\t"
			r+="\n"
			for k in pattern.pwm[0]:
				r+= str(k)+"\t"
				for i in range(len(pattern.pwm)):
					r+=str(pattern.pwm[i][k])[:5]+"\t"
				r+= "\n" 
		
		return r
	elif pattern.id == PBMID:
		r = "threshold:"+str(pattern.t)+"\n"
		r+="\t"
		for i in range(len(pattern.pwm)):
			r += str(i)+"\t\t"
		r+="\n"
		for k in pattern.pwm[0]:
			r+= str(k)+"\t"
			for i in range(len(pattern.pwm)):
				r+=str(pattern.pwm[i][k])[:5]+"\t"
			r+= "\n" 
		return r
	else:
		raise "not implemented yet. fix!"
	

if __name__ == '__main__':


	import main
	main.main()






