from math import sqrt

class misc_rate:
	"""Misclassification rate"""
	def __init__(self, w):
		self.id = "mr"+str(w)
		self.valueRange = [0, 1]
	
	def fitness(self, TP, FN, FP, TN):
		return -float(FP + FN) / (TP+FN+FP+TN) + 1

class asp:
	"""From Tompa"""
	def __init__(self, w):
		self.id = "as"+str(w)
		self.valueRange = [0, 1]
	
	def fitness(self, TP, FN, FP, TN):
		sn = float(TP) / (TP + FN) #sensitivity
		
		n = (TP + FP)
		if n == 0: return 0.
		ppv = float(TP) / n #positive predictive value
		return (sn+ppv)/2
	

class accuracy:
	"""Accuracy"""
	def __init__(self, w):
		self.w = w
		self.id = "ac"+str(w)
		self.valueRange = [0, 1]
		
	def fitness(self, TP, FN, FP, TN):
		
		weight = 1
		if self.w != 0:
			weight = (float(FP)+float(TN))* self.w / (float(TP)+float(FN)) 
		return ((weight*float(TP)) + float(TN)) / ((weight * (float(TP) + float(FN) ) )+ float(TN) + float(FP))
	
	def getName(self):
		return "Accuracy, w=" + str(self.w)
	
	
class wcc:
	"""Weighted Correlation Coefficient
	
	A weight value of 0 means the correlation coefficient will be used without weighting
	A weight value of 0.5 means the entries in the negative set will be weighted twice as much as entries in the positive set
	A weight value of 1 means the entries in the positive and negative sets will be weighted equally
	A weight value of 2 means the entries in the positive set will be weighted twice as much as entries in the negative set
	"""
	
	def __init__(self, w):
		self.w = w
		self.id = "cc"+str(w)
		self.valueRange = [-1, 1]
		
		
	def fitness(self, TP, FN, FP, TN):
		
		if self.w != 0:
			ratio = (float(FP)+float(TN))/(float(TP)+float(FN))
			TP = TP * ratio * self.w
			FN = FN * ratio * self.w
		check = ((TP*TN)-(FP*FN))
		if check != 0:
			check = check/sqrt((TP+FP)*(FP+TN)*(TN+FN)*(FN+TP))
		return check
	
	def getName(self):
		return "Weighted correlation coefficient, w=" + str(self.w)




class fw:
	"""Fw - measure"""
	
	def __init__(self, w):
		self.w = w
		self.id = "fw"+str(w)
		
	def fitness(self, TP, FN, FP, TN):
		TP, FN, FP, TN = float(TP), float(FN), float(FP), float(TN)
		
		gamma = float(TP)
		beta = (float(FP) + float(TP)) / (float(FP) + float(TP) + float(FN) + float(TN))
		nplus = (float(FN) + float(TP)) / (float(FP) + float(TP) + float(FN) + float(TN))
		
		return (((self.w**2)+1)*gamma)/(((self.w**2)*nplus) + beta)
	
	def getName(self):
		return "Fw-mesasure, w=" + str(self.w)
		



if __name__ == '__main__':
	as = asp(0)
	print as.fitness(11, 11, 11, 11)
	
	
	"""
	cc = wcc(0)
	a = (3, 1), (2, 1), (2, 1), (1, 1)
	pos, neg = 0, 0
	for TP, FP in a:
		pos += TP
		neg += FP
	
	tTP, tFP = 0., 0.
	score = 0
	for TP, FP in a:
		tTP += TP
		tFP += FP
		print cc.fitness(tTP, pos-tTP, tFP, neg-tFP)
		#print "rall", (tTP/pos)/(tFP/neg), (tTP/pos), (tFP/neg), "()", (tTP, pos-tTP, tFP, neg-tFP)
		if cc.fitness(tTP, pos-tTP, tFP, neg-tFP) > score: score = cc.fitness(tTP, pos-tTP, tFP, neg-tFP)
	print "---", score
	"""