import re
from random import random
from sets import Set
import BC

LETTERS = {"A":["C", "G", "T"], "C":["A", "G", "T"], "G":["A", "C", "T"], "T":["A", "C", "G"], \
		"a":["c", "g", "t"], "c":["a", "g", "t"], "g":["a", "c", "t"], "t":["a", "c", "g"]}

DOTS = {"A":["."], "C":["."], "G":["."], "T":["."], \
		"a":["."], "c":["."], "g":["."], "t":["."]}

BITPOS = {"A":0, "a":0, "C":1, "c":1, "G":2, "g":2, "T":3, "t":3}

def PWM_BMhit(sequences, PWM, bg, bgo, threshold):

	l = []
	motiflength = len(PWM)
	for i in range(len(sequences)):
		for j in range(len(sequences[i]) - motiflength + 1):
			substr = sequences[i][j:j+motiflength]
			score = 0
			for k in range(len(substr)):
				score += PWM[k][substr[k]]

				if bg.has_key(sequences[i][j-bgo:j]):
					score -= bg[sequences[i][j-bgo:j]][substr[k]]
				else:
					score -= bg["*"][substr[k]]
			if score >= threshold:
				l.append((i, j))
	return(l)

def PWMhit(sequences, PWM, threshold):
	l = []
	motiflength = len(PWM)
	for i in range(len(sequences)):
		for j in range(len(sequences[i]) - motiflength + 1):
			substr = sequences[i][j:j+motiflength]
			score = 0
			for k in range(len(substr)):
				score += PWM[k][substr[k]]
			if score >= threshold:
				l.append((i, j))
	return(l)


def mismatch(sequences, pattern, maxmut):
	l = []
	r = []
	r = range(len(sequences))
	
	for i in r:
		searchLength = len(sequences[i]) - len(pattern)+1
		for j in range(searchLength):
			if(hdistance(sequences[i][j:j+len(pattern)], pattern, maxmut)): l.append((i, j))
	
	

	return(l)

def mismatchExperimental(sequences, pattern, maxmut):
	l = []
	r = []
	r = range(len(sequences))
	
	
	returnTable = []
	if maxmut == 0:
		returnTable = [pattern]
	else:
		mutIndexTable = [None]*maxmut
		pattern = list(pattern)
		genMut(0, 0, pattern, len(pattern), mutIndexTable, len(mutIndexTable), returnTable)
	
	
	
	
	for i in r:
		seq = sequences[i]
		cp = re.compile("|".join(returnTable))
		m = cp.search(seq)
		while m:
			l.append((i, m.start()))
			m = cp.search(seq, m.start()+1)
		
		#searchLength = len(sequences[i]) - len(pattern)+1
		#for j in range(searchLength):
		#	if(hdistance(sequences[i][j:j+len(pattern)], pattern, maxmut)): l.append((i, j))
	

	return(l)


def mismatchQgram(pattern, maxmut, posset, negset):
	returnTable = {pattern:0}
	
	
	if maxmut > 0:
		
		pattern = list(pattern)
		for varmut in range(1, maxmut+1):
			mutIndexTable = [None]*varmut
			genMut2(0, 0, pattern, len(pattern), mutIndexTable, varmut, returnTable, LETTERS)
	#print pattern, maxmut
	#print returnTable
	
	TP, FP = 0, 0
	for instance in returnTable.keys():
		
		if posset.has_key(instance): TP += posset[instance]
		if negset.has_key(instance): FP += negset[instance]
	
	return TP, FP



def mismatchBSQgram(pattern, maxmut, posset, negset):

	minimum = len(pattern) - maxmut
	p = BC.magic_bitstring(0, 0)
	for i in range(len(pattern)):
		p.setBit((i*4)+BITPOS[pattern[i]])
	
	TP, FP = 0, 0
	
	for key, value in posset.iteritems():
		if(key.andStrings(p).countBits()) >= minimum: TP += value
	for key, value in negset.iteritems():
		if(key.andStrings(p).countBits()) >= minimum: FP += value
	
	return TP, FP
		
def genMut(lvl, prev, pattern, patternLength, mutInd, numMut, returnTable, letters=DOTS):
	for i in range(prev, (patternLength-(numMut-lvl-1))):
		mutInd[lvl]=i
		if lvl < (numMut-1):
			genMut(lvl+1, i+1, pattern, patternLength, mutInd, numMut, returnTable, letters)
		else:
			tmp = pattern[:]
			
			for j in mutInd:
				tmp[j] = "."
			returnTable.append("".join(tmp))
				


def genMut2(lvl, prev, pattern, patternLength, mutInd, numMut, returnTable, letters=LETTERS):
	for i in range(prev, (patternLength-(numMut-lvl-1))):
		mutInd[lvl]=i
		if lvl < (numMut-1):
			genMut2(lvl+1, i+1, pattern, patternLength, mutInd, numMut, returnTable, letters)
		else:
			genLetters(0, pattern, mutInd, [""]*numMut, numMut, returnTable, letters)
			
			

def genLetters(lvl, pattern, mutInd, mutletters, maxmut, returnTable, letters):
	
	for i in letters[pattern[mutInd[lvl]]]:
		mutletters[lvl] = i;
		if lvl<(maxmut-1):
			genLetters(lvl+1, pattern, mutInd, mutletters, maxmut, returnTable, letters)
		else:
			
			tempMotif = list(pattern)
			for i, index in enumerate(mutInd):
				tempMotif[index] = mutletters[i]
			tempMotif = "".join(tempMotif)
			returnTable[tempMotif] = 0
	

def hdistance(string1, string2, maxDist):
	"""Returns True if the Hamming distance of string1 and string2 is less than or equal to maxDist"""
	distance = 0
	for i in range(len(string1)):
		#if (len(string1) - i) <= (maxDist-distance): return True
		if string1[i] != string2[i]: 
			distance = distance + 1
			if distance>maxDist or string1[i]=="\n": return False
		
	return True



def hdistance_int(string1, string2):
	"""Returns True if the Hamming distance of string1 and string2 is less than or equal to maxDist"""
	distance = 0
	for i in range(len(string1)):
		#if (len(string1) - i) <= (maxDist-distance): return True
		if string1[i] != string2[i]: 
			distance = distance + 1
			
		
	return distance


def reghit(sequences, pattern, motiflength):
	"""Return a list of tuples matching the regexp pattern in the dataset"""
	l = []
	p = re.compile(pattern)
	for i in range(len(sequences)):
		for j in range(len(sequences[i]) - motiflength + 1):
			if(p.match(sequences[i][j:j+motiflength])) != None: 
				l.append((i, j))
	return(l)




"""
def reghit2(subStrings, pattern):
	#Returns a bitstring showing what of the subStrings the pattern matches
	p = re.compile(pattern)
	l = 0
	for i in range(len(subStrings)):
		if(p.match(subStrings[i])) != None: l |= 1<<i
	return(l)

def reghit3(subStrings, pattern):
	#Returns the number of subStrings the pattern matches
	p = re.compile(pattern)
	l = 0
	for i in range(len(subStrings)):
		print subStrings[i]
		if(p.match(subStrings[i])) != None: l += 1
			
	return(l)
"""




if __name__ == '__main__':

	
	import main
	main.main()
