"""Used to filter/manipulate datasets/sequences in a dataset in certain ways"""

import os
import re
import GraphGen as g
import Markov_prob as mc
from random import randint
import psyco
Set = set

psyco.full()

BASES = ["a", "c", "g", "t"]
COMPLEMENT = {'a':'t', 'c':'g', 'g':'c', 't':'a'}


def remove_sym(inputfolder, outputfolder, filter = ".+", keep_symbols = BASES):
	
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	#print input
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_sym() error: outputoath is file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		totseq = 0
		removedseq = 0
		line = fp.readline()
		seq = ""
		while line:
			
			OK = True # True as long as no symbol is not in "acgt"
			
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				line = fp.readline()
				continue
			
			while (line) and (not line[0] == '>'):
				if not set(line.strip()) <= set(keep_symbols):
					removedseq +=1
					if OK:
						log += "\t" + seq_id + "\n"
						
					OK = False
					log += "\t\tunallowed character in line: '" + line.strip() + "'\n"
					
				seq += line
				line = fp.readline()
			totseq += 1
			
			if OK: fo.write(seq)
			seq = ""
			
		log += "Total sequences: " + str(totseq) + " Sequences removed: " + str(removedseq) +"\n"
		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def remove_multiple_hits(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: outputoath is file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		totseq = 0
		removedseq = 0
		line = fp.readline()
		seq = ""
		while line:
			
			OK = True # True as long as no symbol is not in "acgt"
			
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			while (line) and (not line[0] == '>'):
				seq += line
				tmp += line
				line = fp.readline()
			totseq += 1
			
			alignment = seq_id.split("|")[4].lower()
			#if seq_id.split("|")[5] == 'n': alignment = negative_complement(alignment)#Only needed if negative complement not found already
			
			
			if tmp.find(alignment) != tmp.rfind(alignment):
				log += "\t" + seq_id + "\n"
				log += "\t\terror: find() and rfind() does not return same position\n"
				removedseq +=1
			else: 
				fo.write(seq)
			seq = ""
			
		log += "Total sequences: " + str(totseq) + " Sequences removed: " + str(removedseq) +"\n"
		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def minimum_number_of_sequences(inputfolder, outputfolder, filter = ".+", minimum_number=5):
	log = ""
	log += "Removing all datasets with less than %s sequences\n" % minimum_number
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: outputpath exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		
		seq_id = ""
		line = fp.readline()
		seq = ""
		outputList = []
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			alignment = seq_id.split("|")[4].lower()
			
			outputList.append(seq + tmp+"\n\n")
			
				
			seq = ""
		if len(outputList) >= minimum_number:
			for line in outputList: fo.write(line)
			log += "%s sequences, dataset kept\n" % len(outputList)
		else:
			log += "%s sequences, dataset removed\n" % len(outputList)
		log += "End file\n\n"
		fo.close()
		fp.close()
	
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def truncate_sequence_end(inputfolder, outputfolder, filter = ".+", total_length = 2000):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: outputpath exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		less = 0
		more = 0
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			alignment = seq_id.split("|")[4].lower()
			
			tmp = tmp[-total_length:]
			if not (tmp.find(alignment) == -1 or tmp.find(alignment) != tmp.rfind(alignment)):
				seq += tmp+"\n\n"
				fo.write(seq)
			else:
				log += "Removed: " + seq_id + "\n"
			
			
			
			seq = ""
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def truncate_sequences(inputfolder, outputfolder, filter = ".+", total_length = 5000):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: outputoath is file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		less = 0
		more = 0
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			alignment = seq_id.split("|")[4].lower()
			#if seq_id.split("|")[5] == 'n': alignment = negative_complement(alignment)#Only needed if negative complement not found already
			
			
			if len(tmp) > total_length:# and tmp.find(alignment) == tmp.rfind(alignment): 
				more += 1
				
				left = int(total_length-len(alignment))
				lflank = left/2 + (left & 1)
				rflank = left/2
				#assert(total_length == len(alignment)+lflank+rflank)
				
				position = tmp.find(alignment)
				if position < lflank:
					rest = lflank - position
					lflank -= rest
					rflank += rest
					#print lflank + rflank
				elif rflank > len(tmp[position+len(alignment):]):
					rest = rflank - len(tmp[position+len(alignment):])
					lflank += rest
					rflank -= rest
				log += "\t" + seq_id + "\n"
				log += "\t\tOriginal sequence length: "+str(len(tmp))
				tmp = tmp[position-lflank:position+len(alignment)+rflank]
				
				log+= ", truncated to "+str(len(tmp))+"\n"
			else: more += 1
			seq += tmp+"\n\n"
			fo.write(seq)
			seq = ""
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()

def generateUniqueIDs(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return

	for f in input:
		
		
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		print f
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		seqidList = []
		motifList = []
		state = 0
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			if seq_id[:20] in seqidList:
				log += "Old ID: " + seq_id[:20] +"\n"
				seq_id = seq_id[0] + ('%04d' % state) + seq_id[5:]
				log += "\tNew ID: " + seq_id[:20] + "\n"
				state +=1
			else: seqidList.append(seq_id[:20])
			
			seq = seq_id+"\n"+tmp+"\n\n"
			fo.write(seq)
			
			seq = ""
			
		print seqidList

		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()

def remove_sequence_duplicates(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return

	for f in input:
		
		
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		print f
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		seqidList = []
		seqList = []
		test = 0
		removed = []
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			#
			#print test
			#test += 1
			#
			wr = True
			for s in seqList:
				if len(s) == len(tmp):
					if s == tmp or s == negative_complement(s):
						wr = False
						break
			if wr:
				seqList.append(tmp)
				seq = seq_id+"\n"+tmp+"\n\n"
				fo.write(seq)
			if not wr:
				removed.append(seq_id)
			
			seq = ""
		
						
		if len(removed)>0:
			log += "Sequences removed:\n"
			log += "\n".join(removed) + "\n"
		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def replace_sequences_with_mchain(inputfolder, outputfolder, sequenceLength = 2000, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return

	for f in input:
		
		
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		print f
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		seqidList = []
		motifList = []
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			seqidList.append(seq_id)
			motifList.append(seq_id.split("|")[4].lower())

		for seq_id, mot in zip(seqidList, motifList):

			tseq = mc.generateSequence(sequenceLength)
			while True:
				found = False
				for motif in motifList:
					if tseq.find(motif) != -1:
						found = True
						break
				if found: tseq = mc.generateSequence(sequenceLength)
				else: break
			
			injectpos = randint(0, sequenceLength-len(mot))
			tseq = list(tseq)
			tseq[injectpos:injectpos+len(mot)] = list(mot)
			tseq = "".join(tseq)
			
			seq = seq_id+"\n"+tseq+"\n\n"
			fo.write(seq)
			seq = ""
			
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()






def remove_short_site_sequences(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			seq += tmp+"\n\n"
			if seq_id.split("|")[6].strip() == "": fo.write(seq)
			else:
				log += "\t" + seq_id + " : removed\n"
				
			seq = ""
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()


def reverse_negative_complement_sequences(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			
			if seq_id.split("|")[5].strip() == "n":
				tmp = negative_complement(tmp)
				log += "\t" + seq_id + " : reversed\n"
			
			seq += tmp+"\n\n"
			fo.write(seq)
			
			seq = ""
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()

def addDatasetnameToID(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return
	
	for f in input:
		sequence_number = 1
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		fo = file(output_dataset, "w")
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				#seq += line
				tmp += line.strip()
				line = fp.readline()
			
			tmpseq_id = list(seq_id)
			tmpseq_id[0:5] = [">"] + list(f[:-4] + "-" + str(sequence_number) + "-embl")
			sequence_number += 1
			
			log += "Changing\n"
			log += "\t" + seq_id +"\n"
			log += "to\n"
			
			seq_id = "".join(tmpseq_id)
			
			log += "\t" + seq_id +"\n"
			
			
			fo.write(seq_id + "\n" + tmp + "\n\n")
			
			seq = ""
			

		log += "End file\n\n"
		fo.close()
		fp.close()
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()
	
	
	
def selection(inputfolder, outputfolder, filter, scorelimit, sequencelimit, keepscoresover=False):
	from ExtractResults import alteredHits
	from ScoreFunc import wcc
	cc = wcc(0)
	
	r_trans_hk, r_trans_hs =  alteredHits("nye_resultat/transfac_onehit_d/", r".+hdi.+")
	r_trans_rk, r_trans_rs =  alteredHits("nye_resultat/transfac_onehit_d/", r".+reg.+")
	r_trans_pk, r_trans_ps =  alteredHits("nye_resultat/transfac_onehit_d/", r".+pwm.+")
	
	
	r_trans_hk.sort()	
	r_trans_rk.sort()
	r_trans_pk.sort()
	connection = {}
	for i in range(len(r_trans_hk)):
		h = cc.fitness(*r_trans_hs[r_trans_hk[i]][0])
		r = cc.fitness(*r_trans_rs[r_trans_rk[i]][0])
		p = cc.fitness(*r_trans_ps[r_trans_pk[i]][0])
		
		t = max((h, r, p))
		connection[r_trans_hk[i].split("_-_")[0]] = t
	print connection["M00795.fas"]
	
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return

	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		tempScore = connection[f]
		
		if tempScore > scorelimit and not keepscoresover: continue
		if tempScore < scorelimit and keepscoresover: continue
		
		
		fp = file(dataset)
		
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		
		motifList = Set([])
		state = 0
		
		datasetList = []
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				tmp += line.strip()
				line = fp.readline()
			
			motifList.add(seq_id.split("|")[4].lower())
			datasetList.append((seq_id, tmp))
		
		if len(datasetList) < sequencelimit:
			log += "End file\n\n"
			fp.close()
			continue
		
		fo = file(output_dataset, "w")
		
		
		
		
		for seq_id, tmp in datasetList:
			fo.write(seq_id+"\n"+tmp+"\n\n")
		log += "End file\n\n"
		fo.close()
		fp.close()
		
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()
	
def remove_all_multiple_hits(inputfolder, outputfolder, filter):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_all_multiple_hits() error: file exists"
		return

	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		
		fp = file(dataset)
		
		
		
		log += "File: " + output_dataset + "\n"
		seq_id = ""
		line = fp.readline()
		seq = ""
		
		motifList = Set([])
		state = 0
		
		datasetList = []
		while line:
			
			if line[0] in ('#','\n'):
				seq += line
				line = fp.readline()
				continue
			if line[0] in ('>'):
				seq += line
				seq_id = line.strip()
				
				line = fp.readline()
				continue
			tmp = ""
			
			while (line) and (not line[0] == '>'):
				tmp += line.strip()
				line = fp.readline()
			
			motifList.add(seq_id.split("|")[4].lower())
			datasetList.append((seq_id, tmp))
		
		fo = file(output_dataset, "w")
		
		for i in range(len(datasetList)):
			seq_id, tmp = datasetList[i]
			keep = True
			for motifInstance in motifList:
				if motifInstance == seq_id.split("|")[4].lower(): continue
				
				if tmp.find(motifInstance) > -1: #Not good, sequence should be removed
					keep = False
					break
			if keep:
				log += "Keeping:" + seq_id + "\n"
				seq = seq_id+"\n"+tmp+"\n\n"
				fo.write(seq)
			else:
				log += "Removing:" + seq_id +"\n"
			seq = ""
			
		

		log += "End file\n\n"
		fo.close()
		fp.close()
	
	fl = file(outputfolder +'/' + "log.txt", "w")
	fl.write(log)
	fl.close()



def oneGiantSet(inputfolder, outputfilename, filter):
	if inputfolder[-1] not in ("\\", "/", os.sep): inputfolder += os.sep
	
	if os.path.exists(inputfolder + outputfilename):
		print "Error: outputfile exists."
		return
	
	
	pattern = re.compile(filter)
	
	files = os.listdir(inputfolder)
	files.sort()
	
	fp = file(inputfolder + outputfilename, "w")
	
	for f in files:
		
		if not pattern.match(f): continue
		fi = file(inputfolder + f)
		for line in fi:
			fp.write(line)
	fp.close()
		
def getSimilar(inputFolder, sourceFolder, outputFolder, pattern):
	if not inputFolder[-1] in ("\\", "/", os.sep): inputFolder += os.sep
	if not outputFolder[-1] in ("\\", "/", os.sep): outputFolder += os.sep
	if not sourceFolder[-1] in ("\\", "/", os.sep): sourceFolder += os.sep
	
	
	
	
	import shutil
	input = os.listdir(inputFolder)
	pattern = re.compile(pattern)
	
	if not os.path.exists(outputFolder): os.makedirs(outputFolder)
	elif os.path.isfile(outputFolder): 
		print "remove_multiple_hits() error: file exists"
		return
	
	for f in input:
		if not pattern.match(f): continue
		shutil.copy(sourceFolder + f, outputFolder + f)
		
	

def negative_complement(input):
	r = ""
	for i in range(len(input)-1, -1, -1):
		r += COMPLEMENT[input[i]]
	return r


def temp(inputfolder, outputfolder, filter = ".+"):
	log = ""
	
	input = os.listdir(inputfolder)
	pattern = re.compile(filter)
	
	if not os.path.exists(outputfolder): os.makedirs(outputfolder)
	elif os.path.isfile(outputfolder): 
		print "remove_multiple_hits() error: file exists"
		return
	
	for f in input:
		dataset = inputfolder +"/" +f
		output_dataset = outputfolder + "/" + f
		
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		
		
		
		
		seq_id = ""
		line = fp.readline()
		seq = ""
		l = []
		while line:
			
			if line[0] in ('#','\n'):
				
				line = fp.readline()
				continue
			if line[0] in ('>'):
				
				seq_id = line.strip()
				l.append(seq_id.split("|")[4])
				line = fp.readline()
				continue
			
			
			while (line) and (not line[0] == '>'):
				
				line = fp.readline()
			
		li = []
		for i in range(len(l[0])):
			t = ""
			for alignment in l:
				t += alignment[i]
			li.append("[" + "".join(list(set(t)))+"]")
		print "".join(li)
		
				
def viz_number_of_sequences(inputfolder, filter=".+"):
	input = os.listdir(inputfolder)
	overview = {}
	
	pattern = re.compile(filter)
	for f in input:
		dataset = inputfolder +"/" +f
		if os.path.isdir(dataset) or not pattern.match(f) : continue
		
		fp = file(dataset)
		
		
		totseq = 0
		for line in fp:
			if line[0] in ('>',):totseq += 1
		if totseq in overview: overview[totseq] += 1
		else: overview[totseq] = 1
	labels = []
	values = []
	for k, v in overview.iteritems():
		labels.append(str(k))
		values.append(v)
	input = []
	input.append({'values':values, 'color':'r', 'label':'ball'})
	groupLabels = labels
	g.simpleBars(input, showWindow=True, xticks = groupLabels, title="Bars", xlabel="", ylabel="sets", fname="c:/*")







#temp("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/onehit/truncated/noshort/ncomplement", 
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/onehit/truncated/noshort/ncomplement", ".+770.+")

#remove_sym("../../shared/md_datasett/dataset/dna/transfac",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC", ".+fas$")

#reverse_negative_complement_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC", 
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences", ".+fas$")

#remove_short_site_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences", 
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort", ".+fas$")

#remove_sequence_duplicates("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq", filter=".+fa")

#generateUniqueIDs("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids", filter=".+fa")

#truncate_sequence_end("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated", ".+fa")

#minimum_number_of_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq", ".+fa")

#replace_sequences_with_mchain("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/mchain", filter=".+fa")


#addDatasetnameToID("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID", filter=".+fa")

#addDatasetnameToID("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/mchain",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/mchain/newID", filter=".+fa")




#oneGiantSet("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID", "big.fasta", filter=".+fa")

#selection("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID",\
#"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID/lowbatch", ".+fas$",\
#0.72, 17, keepscoresover=False)


#selection("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID",\
#"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID/highbatch", ".+fas$",\
#0.79, 0, keepscoresover=True)

getSimilar("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID/highbatch", \
"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/mchain/newID",\
"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/mchain/newID/highbatch", ".+fas$")



#remove_all_multiple_hits("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID/lowbatch",\
#"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/no_d_seq/uniqueids/end_truncated/min_five_seq/newID/lowbatch/removehits", ".+fas$")





#remove_multiple_hits("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/onehit", ".+fas$")

#truncate_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/onehit",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/onehit/truncated", ".+fas$")
	
#truncate_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/truncated", ".+fa")

#replace_sequences_with_mchain("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/onehit/truncated",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/onehit/truncated/mchain", filter=".+fa")



#truncate_sequence_end("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/end_truncated", ".+fa")
	

#minimum_number_of_sequences("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/end_truncated",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/end_truncated/min_five_seq", ".+fa")


#replace_sequences_with_mchain("../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/end_truncated/min_five_seq",\
#	"../../shared/md_datasett/dataset/dna/transfac/noIUPAC/nc_sequences/noshort/end_truncated/min_five_seq/mchain", filter=".+1033.+")

