import os
#from system import data, conf
from math import log, pow
from library.motif.motif import motif_regexp, motif_known
from library.sequence import sequence
#from library.bitset import bitset
#from library.significance.basic import significance
"""
CREBTYPE, MOTIFSEARCHTYPE, TOMPATYPE = 0, 0, 0
try:
	import DB_constants
	CREBTYPE = DB_constants.CREBTYPE
	MOTIFSEARCHTYPE = DB_constants.MOTIFSEARCHTYPE
	TOMPATYPE = DB_constants.TOMPATYPE

except:
	pass
"""

class dataset:
	def __init__(self):
		self._ID = ""
		self._description = ""
		self._filename = ""
		#List of motif objects
		self._motif_list = []
		self._sequence_list = []

	def getComplement(self, input):
		conv = {"a":"t", "t":"a", "g":"c", "c":"g", "A":"T", "T":"A", "C":"G", "G":"C"}
		r = list(input)
		r.reverse()
		for i in range(len(r)):
			if conv.has_key(r[i]):
				r[i] = conv[r[i]]
			else: 
				print r[i], self._filename, "".join(r)
		return "".join(r)
	
	#Returns a motif object
	def getMotif(self, index):
		return self._motif_list[index]
	
	def getID(self):
		return self._ID

	def getSequence(self, index):
		return self._sequence_list[index]

	
	def getSequenceCount(self):
		return len(self._sequence_list)
	
	def getMotifCount(self):
		return len(self._motif_list)
	
	def getSequences(self):
		r = []
		for line in self._sequence_list:
			r.append(line.seq)
		return r
	#TODO:
	def getMotifIndexes(self, motifindex=0):
		return self._motif_list[motifindex].get_motif_positions()
	
	def setMotifHitlist(self, list, motifindex=0):
		self._motif_list[motifindex].set_hitlist(list)
	
	def getMotifHitlist(self, motifindex=0):
		return self._motif_list[motifindex].get_hitlist()
	
	def getMotifWindows(self, motifindex=0):
		return self._motif_list[motifindex].get_motif_windows()
	
	def getMaxMotifLength(self, id=0):
		il = self.getMotifIndexes(id)
		length = -1
		
		for seqid, pos in il:
			#print "inni den domrade", self._filename, seqid, pos
			s = self._sequence_list[seqid].seq
			if len(s)-pos < length or length==-1:
				length = len(s)-pos
		#print "maxmotifblabb", length
		return length
	
	#convenience functions for creators of the statistics object (functions that assigns values to class members)


	#error: funker ikkje aa lese inn meir enn ein treffposisjon per sekvens?
	def parse_teiresias(self, filename,  with_sign=False, num_lines=-1):
		file = open(filename)
		#lines = file.readlines()
		count = 0
		line = file.readline()
		while line:
			if line[0] == '#' or line[0] == '\n':
				line = file.readline()
				continue
			count += 1
			if (num_lines > 0) and (count > num_lines):
				break
			elements = line.split()
			a = motif_regexp()
			#a.set_significance(float(elements[2]))
			if (with_sign):
				a.set_pattern([elements[3]])
				a.set_support(int(elements[1]))
				hits = elements[4:]
			else:
				a.set_pattern([elements[2]])
				a.set_support(int(elements[1]))
				hits = elements[3:]

			hitlist = {}
			i = 0
			while i < len(hits):
				if not hits[i] in hitlist.keys():
					hitlist[int(hits[i])] = [int(hits[i+1])]
				else:
					hitlist[int(hits[i])] = hitlist[int(hits[i])] + [int(hits[i+1])]
				i += 2
			a.set_hitlist(hitlist)
			self._motif_list.append(a)
			line = file.readline()
		return
	
	def parse_fasta(self, filename):
		file = open(filename, 'r')

		number = {}
		total = 0.0
		max_support = 0
		
		id=""
		seq_num = 0
		line = file.readline()
		while line:
			if line[0] == '#' or line[0] == '\n':
				line = file.readline()
				continue
			if line[0] == '>':
				#make a new sequence object with given id
				id = (line[1:].strip())
				line = file.readline()
				continue
			seq_num += 1
			seq = ""
			while (line) and (not line[0] == '>'):
				seq += line.strip()
				line = file.readline()
			self._sequence_list.append( sequence(id, seq) )

		self.char_freq = {}
		for f in number.keys():
			self.char_freq[f] = number[f] / total
		file.close()        
		return 
	
	
	
	
	#Submethods using parse_fasta
	def parse_final(self, filename):
		print "parsing", filename
		self.parse_fasta(filename)
		answerpath = filename[:-4]+"_answer.txt"
		
		mapping = {}
		for line in file(answerpath):
			id, startpos, endpos, motif = line.strip().split(", ")
			#Answer-files are 1-indexed, but the program uses 0-indexing
			startpos = int(startpos) - 1
			endpos = int(endpos) - 1
			if id in mapping:
				mapping[id].append((startpos, endpos, motif))
			else:
				mapping[id] = [(startpos, endpos, motif)]
		mk = motif_known()
		
		for i in range(len(self._sequence_list)):
			self._sequence_list[i].seq = self._sequence_list[i].seq.lower()
			#verify answer (just to be sure)
			#print self._sequence_list[i].id
			for startpos, endpos, motif in mapping[self._sequence_list[i].id]:
				#print self._sequence_list[i].seq[startpos:endpos+1], motif
				if self._sequence_list[i].seq[startpos:endpos+1] != motif:
					print "Error: dataset and answer not equal"
					import sys
					sys.exit(0)
				mk.add_hitlist_entry(i, startpos, len(motif))
				mk._instance_list.append(motif)
		#print mk
		self._motif_list.append(mk)
			
		
	
	def parse_synthetic(self, filename):
		self.parse_fasta(filename)
		mk = motif_known()
		mlen = len(self._sequence_list[0].id.split(",")[2].strip())
		
		for i in range(len(self._sequence_list)):
			fields = self._sequence_list[i].id.split(",")
			pos = int(fields[1])
			if pos != -1:
				mk.add_hitlist_entry(i, pos, mlen)
				mk._instance_list.append(self._sequence_list[i].seq[pos:pos+mlen])
		self._motif_list.append(mk)	
		
	def parse_transfac(self, filename):
		
		
		self.parse_fasta(filename)
		mk = motif_known()
		
		#find motiflength
		motiflength = -1
		for i in range(len(self._sequence_list)):
			fields = self._sequence_list[i].id.split("|")
			if fields[6].strip() == "": 
				motiflength = len(fields[4])
				break
		
				
		for i in range(len(self._sequence_list)):
			fields = self._sequence_list[i].id.split("|")
						
			
			#fields[5]=='n' means fields[4] is negative complement
			#if fields[5]=='n': fields[4] = self.getComplement(fields[4])
			index = self._sequence_list[i].seq.find(fields[4].lower())
			
			#Are there alignment positions?
			if fields[6].strip() != "":
				align = fields[6].split(",")
				for j in range(len(align)): align[j] = int(align[j])
				align.sort()
				
				#Find the number of positions to be added before the instance
				pos = -1
				for k in range(len(align)):
					
					if k+1 != align[k]:
						pos = k
						break
					elif k==len(align)-1: pos = k+1
				
				#Correct the index position
				index -= pos
			
			mk.add_hitlist_entry(i, index, motiflength)
			#if fields[5]!='n': mk._instance_list.append(self._sequence_list[i].seq[index:index+motiflength])
			#else: mk._instance_list.append(self.getComplement(self._sequence_list[i].seq[index:index+motiflength]))
			mk._instance_list.append(self.getComplement(self._sequence_list[i].seq[index:index+motiflength]))
			
		
		self._motif_list.append(mk)	
	
				

				
		
			
			
	def parse_creb(self, filename, motiflength=10):
		self.parse_fasta(filename)
		#self.indexList = []
		#self.windowList = []
		mk = motif_known()
		for i in range(len(self._sequence_list)):
			index = self._sequence_list[i].id.find("m0=")+3
			if index > 2:
				motif_index = int(self._sequence_list[i].id[index:index+len(self._sequence_list[i].id)].strip())
				mk.add_hitlist_entry(i, motif_index, motiflength)
				mk._instance_list.append(self._sequence_list[i].seq[motif_index:motif_index+motiflength])
		self._motif_list.append(mk)

	
	def parse_tompa(self, filename):
		self.parse_fasta(filename)
		for i in range(len(self._sequence_list)):
			self._sequence_list[i].seq = self._sequence_list[i].seq.lower()
	
	def parse_motif_search(self, filename):

		self.parse_fasta(filename)
		#self.indexList = []
		#self.windowList = []
		mk = motif_known()
		for k in range(len(self._sequence_list)):
			for i in range(len(self._sequence_list[k].seq)):
				l = self._sequence_list[k].seq[i]
				if l.isupper():
					for j in range(i, len(self._sequence_list[k].seq)):
						c = self._sequence_list[k].seq[j]
						if c.islower():
							mk.add_hitlist_entry(k, i, j-i)
							mk._instance_list.append(self._sequence_list[k].seq[i:j])
							
							#self.windowList.append((k, i, j))
							#self.indexList.append((k, i))
							self._sequence_list[k].seq = self._sequence_list[k].seq.lower()
							break
					break
		self._motif_list.append(mk)
	
	
	#Methods for returning the positive and negative subsets of a dataset as a tuple
	def getSets(self, length):
		pos = []
		neg = []
		positions = self.getMotifIndexes()
		for i in range(len(self._sequence_list)):
			segment = self._sequence_list[i]
			for j in range(len(segment.seq)-length+1):
				substr = segment.seq[j:j+length]
				if (i, j) in positions:
					pos.append(substr)
				else:
					neg.append(substr)
		return pos, neg
	
	def getSetsAsNumbers(self, length):
		blabb = {"a":"1","c":"2","g":"3","t":"4"}
		pos = []
		neg = []
		positions = self.getMotifIndexes()
		for i in range(len(self._sequence_list)):
			segment = self._sequence_list[i]
			for j in range(len(segment.seq)-length+1):
				substr = segment.seq[j:j+length]
				n = ""
				for c in substr:
					n += blabb[c] + " "
				
				if (i, j) in positions:
					pos.append(n.strip())
				else:
					neg.append(n.strip())
		return pos, neg
	

	
