#TOMPA GENERIC

import os
from library.motif.motif import motif_known
from library.dataset import dataset
from library.md_statistics import md_statistics
from library.sequence import sequence
from dataset.dna.tompa05assesing.pos import getPositions

DM01G = 0
DM02G = 1
DM03G = 2
DM04G = 3
DM05G = 4
DM06G = 5
#DM07 and DM08 does not contain binding sites and was added for "negative control" in the original study

HM01G = 8
HM02G = 9
HM03G = 10
HM04G = 11
HM05G = 12
HM06G = 13
HM07G = 14
HM08G = 15
HM09G = 16
HM10G = 17
HM11G = 18
HM12G = 19
HM13G = 20
HM14G = 21
HM15G = 22
HM16G = 23
HM17G = 24
HM18G = 25
HM19G = 26
HM20G = 27
HM21G = 28
HM22G = 29
HM23G = 30
HM24G = 31
HM25G = 32
HM26G = 33

MUS01G = 34
MUS02G = 35
MUS03G = 36
MUS04G = 37
MUS05G = 38
MUS06G = 39
MUS07G = 40
MUS08G = 41
MUS09G = 42
MUS10G = 43
MUS11G = 44
MUS12G = 45
#MUS07 excluded due to 'n'-characters in sequence MUS07G
#MUS10 excluded due to 'n'-characters in sequence MUS10R


YST01G = 46
YST02G = 47
YST03G = 48
YST04G = 49
YST05G = 50
YST06G = 51
YST08G = 53
YST09G = 54
#YST07 and YST10 does not contain binding sites and was added for "negative control" in the original study


SETS = {DM01G: "dm01g.fasta",  DM02G: "dm02g.fasta",  DM03G: "dm03g.fasta",  DM04G: "dm04g.fasta",\
		DM05G: "dm05g.fasta",  DM06G: "dm06g.fasta",\
		HM01G: "hm01g.fasta",  HM02G: "hm02g.fasta",  HM03G: "hm03g.fasta",  HM04G: "hm04g.fasta",\
		HM05G: "hm05g.fasta",  HM06G: "hm06g.fasta",  HM07G: "hm07g.fasta",  HM08G: "hm08g.fasta",\
		HM09G: "hm09g.fasta",  HM10G: "hm10g.fasta",  HM11G: "hm11g.fasta",  HM12G: "hm12g.fasta",\
		HM13G: "hm13g.fasta",  HM14G: "hm14g.fasta",  HM15G: "hm15g.fasta",  HM16G: "hm16g.fasta",\
		HM17G: "hm17g.fasta",  HM18G: "hm18g.fasta",  HM19G: "hm19g.fasta",  HM20G: "hm20g.fasta",\
		HM21G: "hm21g.fasta",  HM22G: "hm22g.fasta",  HM23G: "hm23g.fasta",  HM24G: "hm24g.fasta",\
		HM25G: "hm25g.fasta",  HM26G: "hm26g.fasta",  MUS01G: "mus01g.fasta",  MUS02G: "mus02g.fasta",\
		MUS03G: "mus03g.fasta",  MUS04G: "mus04g.fasta",  MUS05G: "mus05g.fasta",  MUS06G: "mus06g.fasta",\
		MUS08G: "mus08g.fasta",  MUS09G: "mus09g.fasta",\
		MUS11G: "mus11g.fasta",  MUS12G: "mus12g.fasta",  YST01G: "yst01g.fasta",  YST02G: "yst02g.fasta",\
		YST03G: "yst03g.fasta",  YST04G: "yst04g.fasta",  YST05G: "yst05g.fasta",  YST06G: "yst06g.fasta",\
		YST08G: "yst08g.fasta",  YST09G: "yst09g.fasta"}



		



def getDataset(variant = DM01G):
	filename = ""
	try:
		if variant in SETS.values():
			filename = variant
		else:
			filename = SETS[variant]
	except:
		
		print "Error in transfac.read.getDataset: parameter not valid"
		return
	ds = dataset()
	ds._ID = "TOMPA"
	ds._filename = filename
	ds._description = "TOMPA"
	ds.parse_tompa(get_path_file(filename))
	poslist = getPositions(filename.rstrip("g.fasta"))
	#ds.windowList = []
	#ds.indexList = []
	mk = motif_known()
	for line in poslist:
		mk._instance_list.append(line[2])
		

		seqlen = len(ds._sequence_list[line[0]].seq)
		startPos = seqlen + line[1]
		mk.add_hitlist_entry(line[0], startPos, line[3])
		
	
	ds._motif_list.append(mk)
	"""
		seqID = line[0]
		seqlen = len(ds._sequence_list[seqID].seq)
		startPos = seqlen + line[1]
		endPos = startPos + line[3]
		ds.windowList.append((seqID, startPos, endPos))
		print line[2]
		"""
	#print ds.indexList
	#print ds.windowList

	
	return ds

def get_statistics():
	stat = md_statistics()
	stat.parse_fasta( get_path_file("genes_data") )
	stat.name = "TOMPA_data"
	return stat

def get_path_file(filename):
	#Gets the path that the python file itself is in, which is the same path that the data files will be in. 
	path = os.path.abspath(os.path.split(__file__)[0]) + os.sep
	return path + filename
