#TOMPA

import os
from library.motif.motif import motif_known
from library.dataset import dataset
from library.md_statistics import md_statistics
from library.sequence import sequence
from dataset.dna.tompa05assesing.pos import getPositions

DM01M = 0
DM02M = 1
DM03M = 2
DM04M = 3
DM05M = 4
DM06M = 5
DM07M = 6
DM08M = 7
#DM07 and DM08 does not contain binding sites and was added for "negative control" in the original study

HM01M = 8
HM02M = 9
HM03M = 10
HM04M = 11
HM05M = 12
HM06M = 13
HM07M = 14
HM08M = 15
HM09M = 16
HM10M = 17
HM11M = 18
HM12M = 19
HM13M = 20
HM14M = 21
HM15M = 22
HM16M = 23
HM17M = 24
HM18M = 25
HM19M = 26
HM20M = 27
HM21M = 28
HM22M = 29
HM23M = 30
HM24M = 31
HM25M = 32
HM26M = 33

MUS01M = 34
MUS02M = 35
MUS03M = 36
MUS04M = 37
MUS05M = 38
MUS06M = 39
MUS07M = 40
MUS08M = 41
MUS09M = 42
MUS10M = 43
MUS11M = 44
MUS12M = 45


#MUS07 excluded due to 'n'-characters in sequence MUS07G
#MUS10 excluded due to 'n'-characters in sequence MUS10R

YST01M = 46
YST02M = 47
YST03M = 48
YST04M = 49
YST05M = 50
YST06M = 51
YST07M = 52
YST08M = 53
YST09M = 54
YST10M = 55
#YST07 and YST10 does not contain binding sites and was added for "negative control" in the original study


SETS = {DM01M: "dm01m.fasta",  DM02M: "dm02m.fasta",  DM03M: "dm03m.fasta",  DM04M: "dm04m.fasta",\
		DM05M: "dm05m.fasta",  DM06M: "dm06m.fasta",\
		HM01M: "hm01m.fasta",  HM02M: "hm02m.fasta",  HM03M: "hm03m.fasta",  HM04M: "hm04m.fasta",\
		HM05M: "hm05m.fasta",  HM06M: "hm06m.fasta",  HM07M: "hm07m.fasta",  HM08M: "hm08m.fasta",\
		HM09M: "hm09m.fasta",  HM10M: "hm10m.fasta",  HM11M: "hm11m.fasta",  HM12M: "hm12m.fasta",\
		HM13M: "hm13m.fasta",  HM14M: "hm14m.fasta",  HM15M: "hm15m.fasta",  HM16M: "hm16m.fasta",\
		HM17M: "hm17m.fasta",  HM18M: "hm18m.fasta",  HM19M: "hm19m.fasta",  HM20M: "hm20m.fasta",\
		HM21M: "hm21m.fasta",  HM22M: "hm22m.fasta",  HM23M: "hm23m.fasta",  HM24M: "hm24m.fasta",\
		HM25M: "hm25m.fasta",  HM26M: "hm26m.fasta",  MUS01M: "mus01m.fasta",  MUS02M: "mus02m.fasta",\
		MUS03M: "mus03m.fasta",  MUS04M: "mus04m.fasta",  MUS05M: "mus05m.fasta",  MUS06M: "mus06m.fasta",\
		MUS08M: "mus08m.fasta",  MUS09M: "mus09m.fasta",\
		MUS11M: "mus11m.fasta",  MUS12M: "mus12m.fasta",  YST01M: "yst01m.fasta",  YST02M: "yst02m.fasta",\
		YST03M: "yst03m.fasta",  YST04M: "yst04m.fasta",  YST05M: "yst05m.fasta",  YST06M: "yst06m.fasta",\
		YST08M: "yst08m.fasta",  YST09M: "yst09m.fasta"}
		



def getDataset(variant = DM01M):
	filename = ""
	try:
		if variant in SETS.values():
			filename = variant
		else:
			filename = SETS[variant]
	except:
		
		print "Error in transfac.read.getDataset: parameter not valid"
		return
	ds = dataset()
	ds._ID = "TOMPA"
	ds._filename = filename
	ds._description = "TOMPA"
	ds.parse_tompa(get_path_file(filename))
	poslist = getPositions(filename.rstrip("m.fasta"))
	#ds.windowList = []
	#ds.indexList = []
	mk = motif_known()
	for line in poslist:
		mk._instance_list.append(line[2])
		

		seqlen = len(ds._sequence_list[line[0]].seq)
		startPos = seqlen + line[1]
		mk.add_hitlist_entry(line[0], startPos, line[3])
		
	
	ds._motif_list.append(mk)
	"""
		seqID = line[0]
		seqlen = len(ds._sequence_list[seqID].seq)
		startPos = seqlen + line[1]
		endPos = startPos + line[3]
		ds.windowList.append((seqID, startPos, endPos))
		print line[2]
		"""
	#print ds.indexList
	#print ds.windowList

	
	return ds

def get_statistics():
	stat = md_statistics()
	stat.parse_fasta( get_path_file("genes_data") )
	stat.name = "TOMPA_data"
	return stat

def get_path_file(filename):
	#Gets the path that the python file itself is in, which is the same path that the data files will be in. 
	path = os.path.abspath(os.path.split(__file__)[0]) + os.sep
	return path + filename
