#TOMPA REAL

import os
from library.motif.motif import motif_known
from library.dataset import dataset
from library.md_statistics import md_statistics
from library.sequence import sequence
from dataset.dna.tompa05assesing.pos import getPositions

DM01R = 0
DM02R = 1
DM03R = 2
DM04R = 3
DM05R = 4
DM06R = 5
#DM07 and DM08 does not contain binding sites and was added for "negative control" in the original study

HM01R = 6
HM02R = 7
HM03R = 8
HM04R = 9
HM05R = 10
HM06R = 11
HM07R = 12
HM08R = 13
HM09R = 14
HM10R = 15
HM11R = 16
HM12R = 17
HM13R = 18
HM14R = 19
HM15R = 20
HM16R = 21
HM17R = 22
HM18R = 23
HM19R = 24
HM20R = 25
HM21R = 26
HM22R = 27
HM23R = 28
HM24R = 29
HM25R = 30
HM26R = 31

MUS01R = 32
MUS02R = 33
MUS03R = 34
MUS04R = 35
MUS05R = 36
MUS06R = 37
MUS07R = 38
MUS08R = 39
MUS09R = 40
MUS10R = 41
MUS11R = 42
MUS12R = 43
#MUS07 excluded due to 'n'-characters in sequence MUS07G
#MUS10 excluded due to 'n'-characters in sequence MUS10R



YST01R = 44
YST02R = 45
YST03R = 46
YST04R = 47
YST05R = 48
YST06R = 49
YST08R = 50
YST09R = 51
#YST07 and YST10 does not contain binding sites and was added for "negative control" in the original study







SETS = {DM01R: "dm01r.fasta",  DM02R: "dm02r.fasta",  DM03R: "dm03r.fasta",  DM04R: "dm04r.fasta",\
		DM05R: "dm05r.fasta",  DM06R: "dm06r.fasta",  HM01R: "hm01r.fasta",  HM02R: "hm02r.fasta",\
		HM03R: "hm03r.fasta",  HM04R: "hm04r.fasta",  HM05R: "hm05r.fasta",  HM06R: "hm06r.fasta",\
		HM07R: "hm07r.fasta",  HM08R: "hm08r.fasta",  HM09R: "hm09r.fasta",  HM10R: "hm10r.fasta",\
		HM11R: "hm11r.fasta",  HM12R: "hm12r.fasta",  HM13R: "hm13r.fasta",  HM14R: "hm14r.fasta",\
		HM15R: "hm15r.fasta",  HM16R: "hm16r.fasta",  HM17R: "hm17r.fasta",  HM18R: "hm18r.fasta",\
		HM19R: "hm19r.fasta",  HM20R: "hm20r.fasta",  HM21R: "hm21r.fasta",  HM22R: "hm22r.fasta",\
		HM23R: "hm23r.fasta",  HM24R: "hm24r.fasta",  HM25R: "hm25r.fasta",  HM26R: "hm26r.fasta",\
		MUS01R: "mus01r.fasta",  MUS02R: "mus02r.fasta",  MUS03R: "mus03r.fasta",  MUS04R: "mus04r.fasta",\
		MUS05R: "mus05r.fasta",  MUS06R: "mus06r.fasta",  MUS08R: "mus08r.fasta",\
		MUS09R: "mus09r.fasta",  MUS11R: "mus11r.fasta",  MUS12R: "mus12r.fasta",\
		YST01R: "yst01r.fasta",  YST02R: "yst02r.fasta",  YST03R: "yst03r.fasta",  YST04R: "yst04r.fasta",\
		YST05R: "yst05r.fasta",  YST06R: "yst06r.fasta",  YST08R: "yst08r.fasta",  YST09R: "yst09r.fasta"}



def getDataset(variant = DM01R):
	filename = ""
	try:
		if variant in SETS.values():
			filename = variant
		else:
			filename = SETS[variant]
	except:
		
		print "Error in transfac.read.getDataset: parameter not valid"
		return
	ds = dataset()
	ds._ID = "TOMPA"
	ds._filename = filename
	ds._description = "TOMPA"
	ds.parse_tompa(get_path_file(filename))
	poslist = getPositions(filename.rstrip("r.fasta"))

	#ds.windowList = []
	#ds.indexList = []
	mk = motif_known()
	for line in poslist:
		#print line
		mk._instance_list.append(line[2])
		seqlen = len(ds._sequence_list[line[0]].seq)
		startPos = seqlen + line[1]
		mk.add_hitlist_entry(line[0], startPos, line[3])
		
	
	ds._motif_list.append(mk)
	"""
		seqID = line[0]
		seqlen = len(ds._sequence_list[seqID].seq)
		startPos = seqlen + line[1]
		endPos = startPos + line[3]
		ds.windowList.append((seqID, startPos, endPos))
		print line[2]
		"""
	#print ds.indexList
	#print ds.windowList

	
	return ds

def get_statistics():
	stat = md_statistics()
	stat.parse_fasta( get_path_file("genes_data") )
	stat.name = "TOMPA_data"
	return stat

def get_path_file(filename):
	#Gets the path that the python file itself is in, which is the same path that the data files will be in. 
	path = os.path.abspath(os.path.split(__file__)[0]) + os.sep
	return path + filename
